/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

/* @author HyperCast Team
 */

// TODO No commenting in this file 
 
package gui;

import java.io.*;
import java.lang.*;
import java.net.*;
import java.util.*;
import java.awt.*;

public class DNSCache extends Thread{
    static Vector entries;
    static public Component toUpdate;
    static DNSCache dns;
    
    public DNSCache(){
    	entries=new Vector();
    }
    
    public void run(){
    	while(true){
    		try{
    			for(int i=0;i<entries.size();i++){
    				DNSEntry entry=(DNSEntry)entries.get(i);
    				if(!entry.isUpdated()){
    					entry.update();
    					if(toUpdate!=null)toUpdate.repaint();
    					
    				} 
    			}
    			
    			
    			Thread.sleep(1000);
    			
    		}catch(Exception e){
    			lotusDebug.errorln("Exception in DNSCache: "+e.toString());
    		}
    		
    	}
    	
    }
    synchronized static protected DNSEntry findEntry(String ip, String hname){
    	if(dns==null){
    		dns=new DNSCache();
    		dns.start();
    	}
    	
    	for(int i=0;i<entries.size();i++){
    		DNSEntry entry=(DNSEntry)entries.get(i);
    		
    		if(ip!=null && entry.getIPAddress()!=null){
    			if(entry.getIPAddress().equals(ip)){
    				return entry;
    			}
    		}else if(hname!=null && entry.getHostName()!=null){
    			if(entry.getHostName().equals(hname)){
    				return entry;
    			}
    		}
    		
    	}
    	//not found
    	return null;
    		
    }
    
    synchronized static public String getHostName(String addr){
    	
    	addr=addr.trim();
    	//find from the entries
    	DNSEntry entry= findEntry(addr, null);
    	
    	if(entry!=null)return entry.getHostName();
    	
    	//else add a entry
    	entry= new DNSEntry(addr, null);
    	entries.add(entry);
    	return entry.getHostName();
    }
    
    synchronized static public String getIPAddress(String hname){
    	
    	hname=hname.trim();
    	//find from the entries
    	DNSEntry entry= findEntry(null, hname);
    	
    	if(entry!=null)return entry.getIPAddress();
    	
    	//else add a entry
    	entry= new DNSEntry(null, hname);
    	entries.add(entry);
    	return entry.getIPAddress();
    }

    static class DNSEntry{
    	String ipAddress, hostName;
    	
        DNSEntry(String ip, String hname){
        	ipAddress=ip;
        	hostName=hname;	
        }
    	
    	boolean isUpdated(){
    		return hostName!=null && ipAddress!=null;	
    	}
    	
    	String getHostName(){
    		return hostName;
    	}
    	String getIPAddress(){
    		return ipAddress;
    	}
    	synchronized boolean update(){
    		
    		if(hostName!=null && ipAddress!=null)return false;
    		try{
    		
    			
    			InetAddress addr;
    			if(hostName==null){
    				addr = InetAddress.getByName(ipAddress);
    				// Get the host name
            		if(!addr.getHostName().equals(ipAddress))hostName=addr.getHostName();
    			}else{
    				//ipAddress == null
	    			// Get the host address
    				addr = InetAddress.getByName(hostName);
    				ipAddress = addr.getHostAddress();
    			}
    			
            	
            }catch(Exception e){
            	lotusDebug.errorln("Exception while doing DNS lookup: "+e.toString());	
            }
    		if(hostName!=null && ipAddress!=null)return true;
    		else return false;
    	}
    }

}
