/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

/* 
 * @author HyperCast Team
 */

package gui;

/**************************************************************************
    Graphics RunControl interface      
**************************************************************************/

import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Panel;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.Date;
import java.util.Enumeration;
import java.util.Vector;

/**
 * the display window of the GUI.
 */

class GUI_DisplayWindow
    extends Panel
    implements MouseMotionListener, MouseListener, ActionListener {
    
    //Edit Mode ??
    final static int EDIT_NODE = 0; //display node information
    final static int EDIT_TREE = 1; //display multicast tree
    final static int EDIT_MOVE = 2; //move the display
    final static int EDIT_CENTER = 4; //centralize the location
    final static int EDIT_ZOOMIN = 5; //zoom in and centralize
    final static int EDIT_ZOOMOUT = 6; //zoom out and centralize
    final static int EDIT_ZOOMWIN = 7; //define a window and zoom to the window
    
    final static int EDIT_REMOVE =8;
    
    MouseAdapter current_listener;

    /** the current edit mode of the display window */
    int edit_mode;
    int view_mode;
    
    /** the history of the view changes */
    ViewHistory viewHistory;
    View geoView;
    View logView;
    
    /** the display scale */
    float scale; // node address * scale ==> point on the screen.

    /** view point, used to move the part of display shown */
    int viewX, viewY;

    /** a reference to class GUI */
    RunControlGUI the_gui;

    // These variables determine what will be drawn in the window.
    boolean display_logical;
    boolean display_node_labels;
    boolean display_node_phyaddr;
    boolean display_node_hostname;
    
    boolean display_multicast;
    
    boolean auto_fit;

    static Font nodeFont = new Font("Helvetica", Font.PLAIN, 10);
    Font   time_font = new Font("Helvetica", Font.BOLD, 14);

    /** record the root of the multicast tree ??*/
    OverlayNode multicastRoot;
    OverlayNode selectedNode;
    
    /** the map facotry */
    MapFactory mapFactory;
    Map map;
    boolean map_fixed=false;
    
    
    /**
     * Constructor. It initializes display reated setting
     */
    public GUI_DisplayWindow(RunControlGUI gui_arg) {

        //set up node modes
        edit_mode = EDIT_NODE;
        view_mode = View.VIEW_LOGICAL;
        
        scale = 1.0f;
        
        the_gui = gui_arg;

        display_logical = true;
       
        display_node_labels = true;
        display_node_phyaddr = false;
        display_node_hostname = true;
        
        display_multicast = true;
        auto_fit=true;

        setBackground(the_gui.dwColor);
        addMouseListener(this);
        addMouseMotionListener(this);

		viewHistory=new ViewHistory();
		
		
    }
    void setScale(float sc){
    	if(scale!=sc){
    		scale=sc;
    		//recordCurrentView();
    	}
    }
    boolean isHighLight(OverlayNode onode){
    	RunNode rnode=(RunNode)onode.getProtocol();
    	//search??
    	if( !the_gui.cb_hideother.getState() && !searchText.equals("") && rnode.matchSearch(searchText))return true;	
    	
    	return false;
    }
    
    boolean isValidNode(OverlayNode onode){
    	RunNode rnode=(RunNode)onode.getProtocol();
    	//search??
    	if( the_gui.cb_hideother.getState()&& !rnode.matchSearch(searchText))return false;
    	
    	return isRunningNode(onode);
    }
    boolean isRunningNode(OverlayNode onode){
    	RunNode rnode=(RunNode)onode.getProtocol();
    	//return rnode.running && rnode.getState()!=DT_Node_ServerJoin.DEAD;
		return rnode.running;
    	//return rnode.running;
    	//return true;
    }
    
    View getCurrentView(){
    	return new View(scale, viewX, viewY, view_mode, auto_fit,
    			the_gui.getOverlay().getOverlayID());
    }
    /**add current view to the view history*/
    void recordCurrentView(){
    	viewHistory.addView(getCurrentView());
    	if(geoView==null)geoView=getCurrentView();
    	
    }
    /**restore view from view class*/
    void restoreView(View v){
    	restoreView(v, true);	
    }
    void restoreView(View v, boolean olid){
    	
    	scale=v.scale;
    	viewX=v.viewX;
    	viewY=v.viewY;
    	view_mode=v.mode;
    	
    	setAutoFit(v.autofit);
    	
    	if(v.autofit && edit_mode!=EDIT_TREE){
    		setEditMode(EDIT_NODE);	
    		the_gui.setMode(EDIT_NODE);
    	}
    	//if(edit_mode==EDIT_NODE || edit_mode==EDIT_TREE)
    		//setAutoFit(v.autofit);
    	
    	
    	if(olid)
    		the_gui.getOverlay().setOverlayID(v.overlayID);
    		
    	the_gui.setViewState();
    	
    	
    	repaint();
    	
    	//if(v.autofit)bestfit();
    	
    }
    /**go back to the previous view in the history*/
    void backView(){
    	View v=viewHistory.back();
    	if(v!=null)restoreView(v);	
    }
    /**go forward to the next view in the history*/
    void forwardView(){
    	View v=viewHistory.forward();
    	if(v!=null)restoreView(v);	
    }
    
    GeoAddress upleft, downright;
	public void setMapFactory(MapFactory mf){
		mapFactory=mf;
		upleft=new GeoAddress((float)-180, (float)90);
		downright=new GeoAddress((float)180, (float)-90);
		map=mapFactory.getMap(upleft, downright);
	}
	public void setMap(int index){
		if(index==0){
			/* automatic map */
			map_fixed=false;
			updateMap();
		}else{
			/* fixed map */
			map_fixed=true;
			map=mapFactory.getMap(index-1);
		}
		
	}
	public void updateMap(){
		
		if(view_mode==View.VIEW_GEOGRAPHIC && !map_fixed){
			RunNetwork psim = the_gui.getOverlay();
			float minx=180, miny=90, maxx=-180, maxy=-90;
			for(int i=0;i<psim.active_nodes.size();i++){
				RunNode rnode=(RunNode)((OverlayNode)psim.active_nodes.elementAt(i)).getProtocol();
				GeoAddress geo=rnode.getGeoAddress();
				if(geo.longitude<minx)minx=geo.longitude;
				if(geo.longitude>maxx)maxx=geo.longitude;
				if(geo.latitude<miny)miny=geo.latitude;
				if(geo.latitude>maxy)maxy=geo.latitude;
			}
			if(psim.active_nodes.size()>0){
				Map old_map=map;
				map=mapFactory.getMap(new GeoAddress(minx, maxy), new GeoAddress(maxx, miny));
				if(old_map!=map)
					System.out.println("Map switching to "+map.getName());
			}
		}
		
	}

	/** get the node address according to the mode of view */
	XYAddress getNodeAddress(OverlayNode onode){
		RunNode rnode=(RunNode)onode.getProtocol();
		if(view_mode==View.VIEW_LOGICAL)
			return onode.getAddress();
		else return map.getPositionOnMap(rnode.getGeoAddress());	
	}

    private Image offScreenBuffer;
    private Graphics offGraphics;

    /**
     * Update the display in the window.
     * Use double buffers to avoid the flickering when updating
     */
    synchronized public void update(Graphics g)
    {
        if (offScreenBuffer==null ||
            (! (offScreenBuffer.getWidth(this) == this.getSize().width &&
                offScreenBuffer.getHeight(this) == this.getSize().height)))
        {
            offScreenBuffer = this.createImage(getSize().width, getSize().height);
        }

        // We need to use our buffer Image as a Graphics object:
        offGraphics = offScreenBuffer.getGraphics();
        paint(offGraphics);
        // Passes our off-screen buffer to our paint method, which,
        // unsuspecting, paints on it just as it would on the Graphics
        // passed by the browser or applet viewer.
        g.drawImage(offScreenBuffer, 0, 0, this);
        // And now we transfer the info in the buffer onto the
        // graphics context we got from the browser in one smooth motion.

    }

    /**
     * Paint the window and invoke the paint subroutine based on
     * the setting
     */
     
    String searchText;
    synchronized public void paint(Graphics g)
    {
    try{
    	
    	updateMap();
    	
    	searchText=the_gui.tf_search.getText().trim();
    	
        if(auto_fit)bestfit();
        
        Dimension dim = this.getSize();
        g.setColor( the_gui.dwColor );
        g.fillRect(0, 0, dim.width, dim.height);
		RunNetwork psim = the_gui.getOverlay();
		
        if(view_mode==View.VIEW_GEOGRAPHIC){
        	if(map!=null){
        		//draw the map;
        		g.drawImage(map.getImage(), 
        				(int)((0 - viewX)*scale), 
        				(int)((0 - viewY)*scale),
        				(int)((map.getWidth())*scale),
        				(int)((map.getHeight())*scale),
        				this);	
        	}
        }else psim.paint(g, this.viewX, this.viewY, this.scale);
        
        if( display_logical ) paint_logical_edges(g);
        paint_nodes(g);
        if( display_node_labels ) paint_nodes_name(g);
        if( display_node_phyaddr) paint_nodes_phyaddr(g);
        if( display_node_hostname) paint_nodes_hostname(g);

        
        g.setColor(Color.black);
        g.drawLine(0, 0, 0, dim.height-1);
        g.drawLine(0, 0, dim.width-1, 0);
        g.drawLine(dim.width-1, 0, dim.width-1, dim.height-1);
        g.drawLine(0, dim.height-1, dim.width-1, dim.height-1);
        // Draw current time
        // Draw the time it went stable
        String[] sarray = new String[3];
        
        //sarray[0] = psim.getProtocols().;
        sarray[0] = "Time = " + (new Date()).toString();
        /*
        if(the_gui.getOverlay().showingFile){
        	sarray[1]="Showing file";	
        }else{
        	if(the_gui.runControl.getExperiment() == null)sarray[1]="Experiment not created";
        	else {
        		String exprStr;
        		if(the_gui.runControl.experimentMode==RunControl.MODE_RUNSERVER)exprStr="RunServer experiment";
        		else exprStr="Application experiment";
        		
        		if(the_gui.runControl.getExperiment().state!=Experiment.RUNNING)sarray[1]=exprStr+  " stopped";
        		else sarray[1]=exprStr+ " running";
        	}
        }
        */
        sarray[1]=""+the_gui.getOverlay().getRunningNodesCount()+" nodes";
//        sarray[3]="reply/query: "+the_gui.getOverlay().getRepliesReceived()+"/"+the_gui.getOverlay().getRequestsSent();
        sarray[2]="reply/query: "+the_gui.getOverlay().replyReceived+"/"+the_gui.getOverlay().questSent;

        //float time_stable = topo.stableSinceTime();
        //if (time_stable > 0.0)
        //    sarray[2] = "Stable at Time = " + Float.toString(time_stable);
        //else
        //    sarray[2] = "Not Stable";
        
        g.setFont(time_font);
        FontMetrics time_font_metrics = g.getFontMetrics();
        drawStringArray(sarray, 4, 4, g, time_font_metrics);
        drawMulticastTree( g );
        
        //support the EDIT_ZOOMWIN mode
        if(edit_mode==EDIT_ZOOMWIN)
        	paint_zoomwin(g);
        	
        	
        the_gui.updateOverlayIDList();
        
        
    }catch(Exception e){}
		
    }
    private void drawStringArray(String[] sarray, int x, int y, Graphics g, FontMetrics fm) 
    {
        y += fm.getAscent();
        for (int i = 0; i < sarray.length; i++) 
        {
            g.drawString(sarray[i], x, y);
            y += fm.getHeight();
        }
    }
    
    private boolean outsidePanel( Point p ){
        Dimension dim = this.getSize();
        boolean b = p.x < 0 ||
                    p.y < 0 ||
                    p.x > dim.width ||
                    p.y > dim.height;

        return b;
    }

    /**
     * Paint the name of the nodes
     */
    private void paint_nodes_name(Graphics g){
        Dimension dim = this.getSize();
        for( int i=0; i<the_gui.getOverlay().getPhyNodes().size(); i++){
            OverlayNode onode = (OverlayNode ) the_gui.getOverlay().getPhyNodes().elementAt(i);
            
            RunNode rnode = (RunNode)onode.getProtocol();
            if(!isValidNode(onode))continue;
            
            //paint each node
            if( null == onode.getName() ) continue;

            XYAddress addr = getNodeAddress(onode);
            Point p = scaleToPoint( addr );
            p.move((int)p.getX(), (int)p.getY()+15);
            
            if( outsidePanel( p ) ) continue;
            g.setFont(nodeFont);
            g.setColor(Color.black);
            g.drawString( onode.getName(), p.x, p.y);
            
        }
    }
    
    private void paint_nodes_phyaddr(Graphics g){
        Dimension dim = this.getSize();
        for( int i=0; i<the_gui.getOverlay().getPhyNodes().size(); i++){
            OverlayNode onode = (OverlayNode ) the_gui.getOverlay().getPhyNodes().elementAt(i);
            
            RunNode rnode = (RunNode)onode.getProtocol();
            if(!isValidNode(onode))continue;
            
            //paint each node
            if( null == onode.getName() ) continue;
            
            XYAddress addr = getNodeAddress(onode);
            Point p = scaleToPoint( addr );
            p.move((int)p.getX(), (int)p.getY()+25);
            if( outsidePanel( p ) ) continue;
            g.setFont(nodeFont);
            g.setColor(Color.black);
            g.drawString( rnode.getPhysicalNetworkAddress(), p.x, p.y);
            
        }
    }
    
    private void paint_nodes_hostname(Graphics g){
        Dimension dim = this.getSize();
        for( int i=0; i<the_gui.getOverlay().getPhyNodes().size(); i++){
            OverlayNode onode = (OverlayNode ) the_gui.getOverlay().getPhyNodes().elementAt(i);
            
            RunNode rnode = (RunNode)onode.getProtocol();
            if(!isValidNode(onode))continue;
            
            //paint each node
            if( null == onode.getName() ) continue;
            
            rnode = (RunNode) onode.getProtocol();
            
            int dy=25;
            if(display_node_phyaddr)dy+=10;

            XYAddress addr = getNodeAddress(onode);
            Point p = scaleToPoint( addr );
            p.move((int)p.getX(), (int)p.getY()+dy);
            if( outsidePanel( p ) ) continue;
            g.setFont(nodeFont);
            g.setColor(Color.black);
            g.drawString( rnode.getHostName(), p.x, p.y);
            
        }
    }

    /**
     * Paint nodes, including physical and logical part.
     * The display of the logical nodes depends on the configuration
     * of each protocol.
     */
    private void paint_nodes(Graphics g)
    {
        Enumeration nodes = the_gui.getOverlay().getPhyNodes().elements();
        while (nodes.hasMoreElements())
        {
            OverlayNode onode = (OverlayNode) nodes.nextElement();
            
            RunNode rnode = (RunNode)onode.getProtocol();
            if(!isValidNode(onode))continue;
            
            XYAddress addr = getNodeAddress(onode);
            Point p = scaleToPoint( addr );
            if( outsidePanel(p) ) continue;
            if( onode.getProtocol() == null ) {
            //only physical node is painted
                g.setColor(RunNetwork.phyNodeColor);
                g.fillOval(p.x - the_gui.ndDiameter/2,
                   p.y - the_gui.ndDiameter/2,
                   the_gui.ndDiameter,
                   the_gui.ndDiameter);
            }else
            {
                protocolInterface protocol = onode.getProtocol();
                
                if(multicastRoot==onode)g.setColor(Color.red);
                else if(selectedNode==onode)g.setColor(Color.cyan);
                else if(isHighLight(onode))g.setColor(Color.magenta);
                else g.setColor( protocol.getColor() );
                
                if( protocol.getShape() == protocolInterface.shapeCircle )
                    g.fillOval(p.x - (int)(the_gui.ndDiameter*0.8),
                        p.y - (int)(the_gui.ndDiameter*0.8),
                        (int)(the_gui.ndDiameter*1.6),
                        (int)(the_gui.ndDiameter*1.6));
                else if( protocol.getShape() == protocolInterface.shapeSquare )
                    g.fillRect(p.x - (int)(the_gui.ndDiameter*0.8),
                        p.y - (int)(the_gui.ndDiameter*0.8),
                        (int)(the_gui.ndDiameter*1.6),
                        (int)(the_gui.ndDiameter*1.6));
                else if( protocol.getShape() == protocolInterface.shapeTriangle ){
                    int []xline = new int[3];
                    int []yline = new int[3];
                    xline[0] = p.x;
                    xline[1] = p.x - (int)(the_gui.ndDiameter*0.86);
                    xline[2] = p.x + (int)(the_gui.ndDiameter*0.86);
                    yline[0] = p.y - the_gui.ndDiameter;
                    yline[1] = p.y + the_gui.ndDiameter/2;
                    yline[2] = p.y + the_gui.ndDiameter/2;
                    g.fillPolygon( xline, yline, 3 );
                }
                
                g.setColor(RunNetwork.phyNodeColor);
                
                g.fillOval(p.x - the_gui.ndDiameter/2,
                   p.y - the_gui.ndDiameter/2,
                   the_gui.ndDiameter,
                   the_gui.ndDiameter);
            }
        }
    }

    /**
     * Paint the logical edges
     */
    private void paint_logical_edges(Graphics g)
    {
        Color edge_color;

        Enumeration nodes = the_gui.getOverlay().getPhyNodes().elements();

        while (nodes.hasMoreElements())
        {
            OverlayNode fromnode = (OverlayNode) nodes.nextElement();
			
			RunNode rnode = (RunNode)fromnode.getProtocol();
            if(!isValidNode(fromnode))continue;
            
			
            Enumeration other_ends = fromnode.getLogicalNeighbor();
            Enumeration other_colors = fromnode.getLogicalColor();
            Enumeration other_widths = fromnode.getLogicalWidth();

            while(other_ends.hasMoreElements())
            {
                OverlayNode tonode = (OverlayNode) other_ends.nextElement();
                
                XYAddress addr1 = getNodeAddress(fromnode);
                XYAddress addr2 = getNodeAddress(tonode);
            
            	Point  p1 = scaleToPoint( addr1 );
                Point  p2 = scaleToPoint( addr2 );
                if( outsidePanel(p1) && outsidePanel(p2) ) continue;
                Point  pm = new Point( (int) ((p1.x+p2.x)/2), (int) ((p1.y+p2.y)/2) );
                g.setColor((Color)other_colors.nextElement());
                int xinterval = ( p1.x - pm.x > 0 ) ? (p1.x-pm.x) : (pm.x-p1.x);
                int yinterval = ( p1.y - pm.y > 0 ) ? (p1.y-pm.y) : (pm.y-p1.y);
                int width = ((Integer)other_widths.nextElement()).intValue();

                if( xinterval > yinterval ){
                    for( int i=0; i<width; i++) g.drawLine( p1.x, p1.y-width/2+i, pm.x, pm.y-width/2+i);
                }else{
                    for( int i=0; i<width; i++) g.drawLine( p1.x-width/2+i, p1.y, pm.x-width/2+i, pm.y);
                }
            }
        }
    }

    
    
    /**
     * set the display of logical edge
     */
    public final void setDisplayLogical(boolean b)
    {
        display_logical = b;
        repaint();
    }


    /**
     * set the display of node labels
     */
    public final void setDisplayNodeLabels(boolean b)
    {
        display_node_labels = b;
        repaint();
    }
    public final void setDisplayNodePhyAddr(boolean b)
    {
        display_node_phyaddr = b;
        repaint();
    }
    public final void setDisplayNodeHostName(boolean b)
    {
        display_node_hostname = b;
        repaint();
    }

    /**
     * set whether a multicast tree is displayed
     */
    public final void setDisplayMulticast(boolean b)
    {
        display_multicast = b;
        repaint();
    }
    
    /** 
     * set whether automatically fit the size of window
     */
    public final void setAutoFit(boolean b){
        auto_fit = b;
        repaint();
    }

    /**
     * draw the multicast tree rooted at multicastRoot
     */
    private void drawMulticastTree( Graphics g ){
        OverlayNode root = this.multicastRoot;
        RunNetwork net = the_gui.getOverlay();
        Vector nodes=net.getPhyNodes();
        
        if( nodes.contains(root) == false ) return ;
        
        Color edge_color = Color.blue;
        //Graphics g = this.getGraphics();

        OverlayNode [] nodequeue = new OverlayNode[nodes.size()];
        int tail = 0, head = 1;
        nodequeue[0] = root;

        while( tail != head ){
            OverlayNode parent = nodequeue[tail++];
            Enumeration e = parent.getLogicalNeighbor();
            while( e.hasMoreElements() ){
                OverlayNode child = (OverlayNode)e.nextElement();
                if( net.getLogicalNextHop(child, root) == parent ) {
                    nodequeue[head++] = child;
                    
                    Point  p1 = scaleToPoint( getNodeAddress(parent) );
                    Point  p2 = scaleToPoint( getNodeAddress(child) );
                    
                    if( outsidePanel(p1) && outsidePanel(p2) ) continue;
                    g.setColor(edge_color);
                    int xinterval = ( p1.x - p2.x > 0 ) ? (p1.x-p2.x) : (p2.x-p1.x);
                    int yinterval = ( p1.y - p2.y > 0 ) ? (p1.y-p2.y) : (p2.y-p1.y);
                    int width = 2;
                    if( xinterval > yinterval ){
                        for( int i=0; i<width; i++) g.drawLine( p1.x, p1.y-width/2+i, p2.x, p2.y-width/2+i);
                    }else{
                        for( int i=0; i<width; i++) g.drawLine( p1.x-width/2+i, p1.y, p2.x-width/2+i, p2.y);
                    }
                }
            }
        }
    }

    
    /**
     * process the selection event of the popup menu items
     */
    public void actionPerformed(ActionEvent ae){
        //currently do nothing
     
    }



  
    Point arrowStart;
    Point arrowEnd;
    
    OverlayNode lineStart;
    Point linePrev;

    /**
     * Handle of mouse pressed event
     * Left button:
     * arrow mode, record the coordinates to move
     * node mode, add a node
     * line mode, see if it is a possible starting point
     * denode mode, see if a node is selected
     * deline mode, see if a line is selected
     * Right button:
     * popup the menu
     */
    synchronized public void mousePressed(MouseEvent me) {
        //for findCloset method works properly under Geo View
        //we set the RunNetwork to Geographic
        
        if(view_mode==View.VIEW_GEOGRAPHIC)
        	((RunNetwork)the_gui.getOverlay()).setGeographic(map, scale);
        
        
        Point p=me.getPoint();
        XYAddress xyaddr=scaleToNode(me.getPoint());
        
        //find the closest node to where the mouse click
        OverlayNode test = the_gui.getOverlay().findClosest(
                                xyaddr, (int)(the_gui.ndDiameter/scale));
        
        
        //restore the RunNetwork to Logical
        if(view_mode==View.VIEW_GEOGRAPHIC)
        	((RunNetwork)the_gui.getOverlay()).setLogical();                     
                              
        if(edit_mode==EDIT_NODE || edit_mode==EDIT_TREE){
            if( test != null ){
                RunNode rnode = (RunNode)test.getProtocol();
                
                the_gui.printWarning( 
                		rnode.getDetails()
                        );
            }
                
            selectedNode = test;
            this.repaint();
         
        }
        if(edit_mode==EDIT_TREE){
            //display multicast tree
            
            if( test != null ){
                
                multicastRoot = test;
                this.repaint();
            }
            
            
        }else if(edit_mode==EDIT_MOVE || edit_mode==EDIT_ZOOMWIN){
            arrowStart = p;
            
        }else if(edit_mode==EDIT_CENTER){
            centralize(xyaddr);
            repaint();
            
            recordCurrentView();
            
        }else if(edit_mode==EDIT_ZOOMIN){
        	scaleBy( the_gui.zoominFactor );
        	centralize(xyaddr);
            repaint();
            
            recordCurrentView();
            
        }else if(edit_mode==EDIT_ZOOMOUT){
        	scaleBy( the_gui.zoomoutFactor );
        	centralize(xyaddr);
            repaint();
            
            recordCurrentView();
            
        }else if(edit_mode==EDIT_REMOVE){
        	//kill the portal
        	if(test!=null){
        		RunNode rnode=(RunNode)test.getProtocol();
	        	if(rnode!=null){
        			int portal=	rnode.getPortalIndex();
        			
        			the_gui.cb_runControl_input.setSelectedItem("set_value "+portal+" KillServer true");
        			if(the_gui.submitRunControlCommand()){
        			
        				rnode.setStatus(RunNode.DEAD);
        				the_gui.setMode(EDIT_NODE);
        			}
        		}
        	}
        }

        int mods = me.getModifiers();
        
        //currently do nothing
        
        
    }
    void centralize(XYAddress xy){
    	Dimension dim=this.getSize();
    	    
        setView((int)((xy.getX()*scale -dim.getWidth()*0.5)/scale), (int)((xy.getY()*scale -dim.getHeight()*0.5 )/scale));
        
    }

    /**
     * Process mouse drag event. Currently, only two edit modes
     * utilize mouse drag, arrow and line modes
     */
    public void mouseDragged(MouseEvent me) {
        //if( the_gui.animated() ) return;
        int mods = me.getModifiers();
        
        if ((mods & MouseEvent.BUTTON1_MASK) == 0 ) return; //not left button
		
		Point p = me.getPoint();
            
        if( edit_mode == EDIT_MOVE ) {
            this.setView( (int)(viewX - (p.x-arrowStart.x)/scale),
                          (int)(viewY - (p.y-arrowStart.y)/scale) );
            arrowStart = p;
            //XYAddress addr = scaleToNode( new Point( p.x - arrowStart.x, p.y - arrowStart.y ) );
            //this.setView( viewX - addr.getX(), viewY - addr.getY());
            repaint();
        }else if(edit_mode == EDIT_ZOOMWIN){
        	arrowEnd = p;
        	repaint();
        	
        }
    }
    void paint_zoomwin(Graphics g){
    	int x1, y1, x2, y2;
    	
    	if(arrowStart==null || arrowEnd==null)return;
    	
    	if(arrowStart.x<=arrowEnd.x){
    		x1=arrowStart.x;
    		x2=arrowEnd.x;
    	}else{
    		x1=arrowEnd.x;
    		x2=arrowStart.x;
    	}
    	if(arrowStart.y<=arrowEnd.y){
    		y1=arrowStart.y;
    		y2=arrowEnd.y;
    	}else{
    		y1=arrowEnd.y;
    		y2=arrowStart.y;
    	}
    	
    	g.setColor(Color.blue);
        g.drawRect(x1, y1, x2-x1, y2-y1); 
        
    }
	void zoomToWin(){
		int x1, y1, x2, y2;
    	
    	if(arrowStart==null || arrowEnd==null)return;
    	
    	if(arrowStart.x<=arrowEnd.x){
    		x1=arrowStart.x;
    		x2=arrowEnd.x;
    	}else{
    		x1=arrowEnd.x;
    		x2=arrowStart.x;
    	}
    	if(arrowStart.y<=arrowEnd.y){
    		y1=arrowStart.y;
    		y2=arrowEnd.y;
    	}else{
    		y1=arrowEnd.y;
    		y2=arrowStart.y;
    	}
    	
    	Dimension dim=this.getSize();
    	
    	float xscale=((float)(x2-x1))/(float)dim.getWidth();
        float yscale=((float)(y2-y1))/(float)dim.getHeight();
        
        if(xscale>yscale){
        	
        	y1=(int)(y1-(y2-y1)*(xscale/yscale-1)/2);
        	y2=(int)(y1+(y2-y1)*xscale/yscale);
        		
        }else{
        	
        	x1=(int)(x1-(x2-x1)*(yscale/xscale-1)/2);
        	x2=(int)(x1+(x2-x1)*yscale/xscale);
        	
        }
    	
    	XYAddress xyaddr=scaleToNode(new Point(x1, y1));
        setView(xyaddr.getX(), xyaddr.getY());
        
        if(xscale>yscale)setScale(scale/xscale);
        else setScale(scale/yscale);
        
	}

    public void mouseReleased(MouseEvent me) {
        //if( the_gui.animated() ) return;
        int mods = me.getModifiers();

        Point p = me.getPoint();
        OverlayNode test = the_gui.getOverlay().findClosest(
                                scaleToNode(p),
                                (int)(the_gui.ndDiameter/scale) );
		
		if(edit_mode==EDIT_ZOOMWIN){
			
			zoomToWin();
			arrowEnd=null;
			repaint();
			
			recordCurrentView();
			
		}else if(edit_mode==EDIT_MOVE){
			recordCurrentView();
		}
		                                
        
       
    }


    public void mouseEntered(MouseEvent e) {

    }

    public void mouseExited(MouseEvent e) {

    }

    public void mouseClicked(MouseEvent e) {

    }

    public void mouseMoved(MouseEvent e) {
    }

    void reset(){
        restore();
    }
    /** set the mode of view */
    void setViewMode(int mode){
    	if(mode!=view_mode){
    		switch(mode){
    			case View.VIEW_LOGICAL:
    				geoView=getCurrentView();
    				if(logView!=null)restoreView(logView, false);
    				break;
    			
    			case View.VIEW_GEOGRAPHIC:
    				logView=getCurrentView();
    				if(geoView!=null)restoreView(geoView, false);
    				break;	
    		}
    		view_mode = mode;
    		
    		recordCurrentView();
    	}
    }
    
    /** @return the current edit mode */
    int getEditMode()  { return edit_mode; }
    /** set the edit mode */
    void setEditMode( int mode ) {
        
        edit_mode = mode;
        
        switch (mode){
        case EDIT_MOVE:
        	setAutoFit(false);
            this.setCursor(new Cursor(Cursor.MOVE_CURSOR) );
            break;
        case EDIT_NODE:
        	multicastRoot = null;
            this.setCursor( new Cursor(Cursor.DEFAULT_CURSOR) );
            break;
        case EDIT_ZOOMIN:
        case EDIT_ZOOMOUT:
        case EDIT_ZOOMWIN:
        case EDIT_CENTER:
        	setAutoFit(false);
            this.setCursor( new Cursor(Cursor.CROSSHAIR_CURSOR ) );
            break;
        case EDIT_TREE:
        	selectedNode=null;
            this.setCursor( new Cursor( Cursor.DEFAULT_CURSOR) );
            break;
        case EDIT_REMOVE:
        	this.setCursor( new Cursor( Cursor.DEFAULT_CURSOR) );
            break;
        default:
            this.setCursor( new Cursor(Cursor.DEFAULT_CURSOR) );
            break;
        }
        
    }

    /**
     * change of the view  point and scale
     */
    public void scaleBy( float factor ) {
        scale *= factor;
        the_gui.printWarning("scale:"+scale);
    }

    /**
     * restore the scale to the default value
     */
    public void restore( ) {
        setScale(1.0f);
        viewX = 0;
        viewY = 0;
        the_gui.printWarning("scale:"+scale + "; viewX:" + viewX + "; viewY:" + viewY);
    }

    /**
     * fit the display to the window
     */
    synchronized void bestfit( ) {
    	
    	if(view_mode==View.VIEW_GEOGRAPHIC){
    		//viewX=0;viewY=0;
    		viewX=map.getEffStartPoint().getX();
    		viewY=map.getEffStartPoint().getY();
    		
    		Dimension dim = this.getSize();
    		float scale1=((float)dim.getHeight())/map.getEffHeight();
    		float scale2=((float)dim.getWidth())/map.getEffWidth();
    		if(scale1<scale2)scale=scale1;
    		else scale=scale2;
    		
    		return;
    	}
    	
    	//scale the topology to fit into the display
        if( the_gui.getOverlay().getPhyNodes().size() <= 0 ) return;
        
        Enumeration nodes = the_gui.getOverlay().getPhyNodes().elements();

        int fxmax = Integer.MIN_VALUE;
        int fymax = Integer.MIN_VALUE;
        int fxmin = Integer.MAX_VALUE;
        int fymin = Integer.MAX_VALUE;

        while( nodes.hasMoreElements() ) {
            OverlayNode onode = (OverlayNode) nodes.nextElement();
            
            if(!isValidNode(onode) && isRunningNode(onode))continue;
            
            XYAddress addr = getNodeAddress(onode);
            
            if( addr.getX() > fxmax ) fxmax = addr.getX();
            if( addr.getY() > fymax ) fymax = addr.getY();
            if( addr.getX() < fxmin ) fxmin = addr.getX();
            if( addr.getY() < fymin ) fymin = addr.getY();
        }
        
        
        Dimension dim = this.getSize();
        double dimy = dim.getHeight();
        double dimx = dim.getWidth();
		
		if(the_gui.getOverlay().getPhyNodes().size()==1){
        	//only one node
        	//lotusDebug.errorln("one node");
        	scale=1;
        	viewX = fxmin-(int)(dimx/2);
        	viewY = fymin-(int)(dimy/2);
        	return;
        }

        
        double scaleX = dimx/(fxmax - fxmin+20);
        double scaleY = dimy/(fymax - fymin+20);
        if( scaleX > scaleY ) setScale((float)scaleY);
        else setScale((float)scaleX);
        
        //adjust
        fxmin=fxmin-(int)((double)10/scale);
        fymin=fymin-(int)((double)10/scale);
        fxmax=fxmax+(int)((double)10/scale);
        fymax=fymax+(int)((double)10/scale);
        scaleX = dimx/(fxmax - fxmin+20);
        scaleY = dimy/(fymax - fymin+20);
        if( scaleX > scaleY ) setScale((float)scaleY);
        else setScale((float)scaleX);
        
        viewX = fxmin - 10;
        viewY = fymin - 10;
        
        
    }

    void setView( int x, int y ){
        viewX = x; viewY = y;
        //recordCurrentView();
    }

    Point scaleToPoint( XYAddress addr ) {
        return new Point( (int)( (addr.getX() - viewX)*scale ),
                          (int)( (addr.getY() - viewY)*scale ) );
    }

    XYAddress scaleToNode( Point p ) {
        return new XYAddress( (int) ( p.x/scale + viewX ),
                              (int) ( p.y/scale + viewY ) );
    }    
}
