/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

/* 
 * @author HyperCast Team
 */

// TODO No comments in this file


package gui;

/* this class map the IP address to geoGraphic location */
import java.lang.*;
import java.text.ParseException;
import java.util.*;
import java.io.*;
import java.net.*;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import hypercast.util.XmlUtil;

public class LocationMapper{
	static Vector entries;
	static Vector all_entries;
	static Random rd;
	
	static final float MIN_DISTANCE=(float)0.02;
	
	synchronized static public GeoAddress getGeoAddress(String phyAddr){
		HostEntry entry=findEntry(phyAddr);
		GeoAddress ll;
		if(entry==null){
		
			//not found. add the entry
			LocationEntry locentry=findEntryFromDB(phyAddr);
			ll = new GeoAddress(locentry.getLocation().longitude, locentry.getLocation().latitude);
			entry = new HostEntry(phyAddr, ll);
			entries.add(entry);
			
			/*
			int adjust_num=countAddress(ll);
			*/
			
			//adust the location due to multiple nodes at the same address
			//adjustGeoAddress(ll, adjust_num);
			adjustGeoAddress(ll);
			
		}else ll=entry.getLocation();
		
		return ll;
	}
	static void clearEntries(){
		entries=new Vector();
	}
	static void adjustGeoAddress(GeoAddress ll){
		
		Random r = new Random();
        
        do 
        {
            int i = r.nextInt() % 8;
            float dx, dy;
        
            //120
            //4 3
            //756

            if (i < 4)
                dy = 1;
            else if (i > 5)
                dy = -1;
            else
                dy = 0;
        
            if (i%3 == 0)
                dx = 1;
            else if (i%3 == 1)
                dx = -1;
            else
                dx = 0;

            ll.shift((float)(dx*0.1), (float)(dy*0.1));
            
            //lotusDebug.errorln("ll="+ll.getAddressAfterShift().toString());

        } while (isNearToSomeNode(ll));
        	
		
	}
	
	static void adjustGeoAddress(GeoAddress ll, int adjust_num){
		//adjust the geo address
		if(adjust_num<=0)return;
		
		if(rd==null)rd=new Random();
			
		int max;
		if(adjust_num>5)max=100;
		else max=100*adjust_num/5;
		
		float adjust_x=(float)(((float)rd.nextInt(max)-50.0)*5)/(float)100.0;
		float adjust_y=(float)(((float)rd.nextInt(max)-50.0)*5)/(float)100.0;
				
		//adjust
		ll.adjust(adjust_x, adjust_y);
	}
	
	static boolean isNearToSomeNode(GeoAddress g){
		for(int i=0;i<entries.size();i++){
    		HostEntry entry=(HostEntry)entries.get(i);
    		GeoAddress g2=entry.getLocation();
    		
    		if(g!=g2 && g.distanceTo(g2)<=MIN_DISTANCE){
    			return true;
    		}
    		
    	}
    	
    	return false;
	}
	
	synchronized static protected int countAddress(GeoAddress g){
			
		if(entries==null){
			entries = new Vector();
		}
    	
    	int num=0;
    	for(int i=0;i<entries.size();i++){
    		LocationEntry entry=(LocationEntry)entries.get(i);
    		GeoAddress g2=entry.getLocation();
    		
    		if(g.distanceTo(g2)<=MIN_DISTANCE){
    			num++;
    		}
    		
    	}
    	
    	return num;	
	}
	private static void readXmlLocationDB()
		
	{
		String filename = "location.xml";
		all_entries = new Vector();
		
		try{
			Document doc=XmlUtil.createDocument(filename);
			NodeList hosts = doc.getChildNodes().item(0).getChildNodes();
			for(int i=0;i<hosts.getLength();i++){
				Element host = (Element)hosts.item(i);
				String Addr = host.getElementsByTagName("Address").item(0).getChildNodes().item(0).getNodeValue();
				Element location = (Element)host.getElementsByTagName("Location").item(0); 
				String latitude = location.getElementsByTagName("Latitude").item(0).getChildNodes().item(0).getNodeValue();
				String longitude = location.getElementsByTagName("Longitude").item(0).getChildNodes().item(0).getNodeValue();
				String locname = "";
				NodeList nl = location.getElementsByTagName("LocationName");
				if(nl!=null && nl.getLength()>0){
					if(nl.item(0).getChildNodes().getLength()>=1){
						locname = nl.item(0).getChildNodes().item(0).getNodeValue();
					}
				}
				
				//lotusDebug.println("Adding " + ip+"/"+maskLen + " = " + longitude + ", " + latitude);
				
				all_entries.add(new LocationEntry(Addr, new GeoAddress(longitude+","+latitude), locname));
				
				
			}
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ParseException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}finally{
			/* add a default entry */
			all_entries.add(new LocationEntry(null, new GeoAddress("-79.40,43.67"), "University of Toronto"));
		}
		
	}
	synchronized static protected LocationEntry findEntryFromDB(String phyAddr) {
		if(all_entries==null){
			readXmlLocationDB();
		}
		
    	for(int i=0;i<all_entries.size();i++){
    		LocationEntry entry=(LocationEntry)all_entries.get(i);
    		
    		if(entry.match(phyAddr)){
    			return entry;
    		}
    		
    	}
    	//not found
    	return null;
	}
	synchronized static protected HostEntry findEntry(String phyAddr){
    	if(entries==null)entries = new Vector();	
    	
    	for(int i=0;i<entries.size();i++){
    		HostEntry entry=(HostEntry)entries.get(i);
    		
    		if(entry.getPhysicalAddress().equals(phyAddr)){
    			return entry;
    		}
    		
    	}
    	//not found
    	return null;
	}
	
	
	
	static class HostEntry{
		String phyAddr;
		GeoAddress ll;
		public HostEntry(String phyAddr, GeoAddress ll){
			this.phyAddr = phyAddr;
			this.ll = ll;
		}
		public String getPhysicalAddress(){
			return phyAddr;
		}
		public GeoAddress getLocation(){
			return ll;
		}
	}
	
	
	
	static class LocationEntry{
		String Addr;
		String LocName;
		GeoAddress ll;
		
		public LocationEntry(String phyAddr, GeoAddress ll, String LocName){
			Addr = phyAddr;
			this.ll = ll;
			this.LocName = LocName;
			
		}
		
		public boolean match(String phyAddr){
			if(Addr == null || Addr.equals(""))
				return true;
			
			if(phyAddr == null)return false;
			
			return phyAddr.startsWith(Addr);
		}
		public String getAddressPrefix(){
			return Addr;
		}
		
		public GeoAddress getLocation(){
			return ll;
		}
		
		public String getLocationName(){
			return LocName;
		}
	}
	
}
