/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

/* 
 * @author HyperCast Team
 */

package gui;

/**************************************************************************
 *  Graphics RunControl interface      
 **************************************************************************/

import java.awt.Frame;
import java.awt.Image;
import java.awt.Toolkit;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

/**
 * 
 * This class manages a list of Maps used to display on the GUI
 * 
 * @author Guangyu Dong
 *  
 */
public class MapFactory {

	/** a list of maps */
	private Vector mapList;

	/** the Frame object to create Toolkit */
	private Frame appWin;

	/**
	 * Constructor
	 * 
	 * @param mapconfig
	 *            the config file to list the available maps
	 */
	public MapFactory(String mapconfig, Frame _appWin) {
		mapList = new Vector();
		appWin = _appWin;
		initMaps(mapconfig);
	}

	/** initiate map list */
	private void initMaps(String mapconfig) {
		//Load XML into DOM Document
		System.out.println("Initializing maps...");
		DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
		DocumentBuilder parser;
		try {
			parser = dbf.newDocumentBuilder();
			Document doc = parser.parse(new InputSource(new FileReader(mapconfig)));

			for (int k = 0; k < doc.getChildNodes().getLength(); k++) {
				Node mapsRoot = (doc.getChildNodes().item(k));
				if (mapsRoot.getNodeName().equals("Maps")) {
					NodeList nl = mapsRoot.getChildNodes();
					for (int i = 0; i < nl.getLength(); i++) {
						if (nl.item(i).getNodeName().equals("Map")) {
							Element mapItem = (Element) nl.item(i);
							String name = mapItem.getAttribute("name");
							String type = mapItem.getAttribute("type");
							int width = Integer.parseInt(mapItem.getAttribute("width"));
							int height = Integer.parseInt((mapItem.getAttribute("height")));
							GeoAddress upleft = new GeoAddress(mapItem.getAttribute("upleft"));
							GeoAddress downright = new GeoAddress(mapItem.getAttribute("downright"));
							
							XYAddress sp=new XYAddress(0,0);
							if(!mapItem.getAttribute("startpoint").equals(""))
								sp= new XYAddress(mapItem.getAttribute("startpoint"));
							
							int effWidth = width;
							if(!mapItem.getAttribute("effwidth").equals(""))
								effWidth=Integer.parseInt(mapItem.getAttribute("effwidth"));
							
							int effHeight = height;
							if(!mapItem.getAttribute("effheight").equals(""))
								effHeight=Integer.parseInt((mapItem.getAttribute("effheight")));
							
							String filename = mapItem.getChildNodes().item(0).getNodeValue();

							System.out.println("Adding map " + name + " <" + filename + ">...");
							Map map = createMap(mapItem, type, name, filename, width, height, upleft, downright, sp, effWidth, effHeight);
							mapList.addElement(map);
						}
					}
				}
			}

		} catch (Exception e) {
			System.err.println("Couldn't read map config file: " + e.toString());
		}
	}

	/** number of maps */
	public int size() {
		return mapList.size();
	}

	/** find a map by index */
	public Map getMap(int index) {
		return (Map) mapList.elementAt(index);
	}

	/** find a best map for the size specified */
	public Map getMap(GeoAddress upleft, GeoAddress downright) {
		Map best_map = null;
		float best_distance = 0;

		for (int i = 0; i < mapList.size(); i++) {
			Map map = (Map) mapList.elementAt(i);

			if (map.getNW().longitude <= upleft.longitude
					&& map.getNW().latitude >= upleft.latitude
					&& map.getSE().longitude >= downright.longitude
					&& map.getSE().latitude <= downright.latitude) {
				/* it's a good map */
				/* is it better? */
				float distance = Math.abs(upleft.longitude - map.getNW().longitude)
						+ Math.abs(upleft.latitude - map.getNW().latitude)
						+ Math.abs(downright.longitude - map.getSE().longitude)
						+ Math.abs(downright.latitude - map.getSE().latitude);

				if (best_map == null || distance < best_distance) {
					best_map = map;
					best_distance = distance;
				}

			}

		}

		return best_map;
	}
	
	/** Create a map from a file */
	private Map createMap(Element mapItem, String type, String name, String filename, int width, int height, GeoAddress upleft,
			GeoAddress downright, XYAddress sp, int ew, int eh) {
		Toolkit tk = appWin.getToolkit();
		if (type.equals("mercator")){
			String ref1_loc, ref1_pos, ref2_loc, ref2_pos;
			ref1_loc=mapItem.getAttribute("ref1_location");
			ref1_pos=mapItem.getAttribute("ref1_position");
			ref2_loc=mapItem.getAttribute("ref2_location");
			ref2_pos=mapItem.getAttribute("ref2_position");
			
			return new MercatorMap(name, FileResourceManager.createImage(tk,filename), width, height, upleft, downright, sp, ew, eh, ref1_loc, ref1_pos, ref2_loc, ref2_pos);
		}else{
			throw new IllegalArgumentException("Map type not supported: "+type);
		}
		
	}

}

/**
 * Map using Mercator Projection
 */

class MercatorMap implements Map {

	/**
	 * the longitude and latitude of the northwest point and southeast point of
	 * the map
	 */
	GeoAddress nw_p, se_p;
	/**
	 * the map parameters
	 */
	double x_scale, y_scale, x_dist, y_dist;
	
	Image mapImage;

	int width, height;
	int effWidth, effHeight;
	XYAddress startPoint;

	String name;
	public MercatorMap(String _name, Image img, int w, int h, GeoAddress nw, GeoAddress se, XYAddress sp, int ew, int eh, 
			String ref1_loc, String ref1_pos, String ref2_loc, String ref2_pos){
		
		
		name = _name;
		mapImage = img;
		nw_p = nw;
		se_p = se;
		width = w;
		height = h;
		
		effWidth=ew;
		effHeight=eh;
		startPoint=sp;
		
		computeParameters(ref1_loc, ref1_pos, ref2_loc, ref2_pos);
		
	}
	static void usage(String args[], int argsStart){
		String fix_args=" ";
		for(int i=0;i<argsStart;i++){
			fix_args+=args[i]+" ";
		}
		System.out.println("Computing parameters for Mercator projection map:");
		System.out.println("Usage: java MapFactory"+fix_args+" <Location of Ref1> <Position of Ref1> <Localtion of Ref2> <Position of Ref2");
		System.out.println("    Ref1: Reference city 1");
		System.out.println("    Ref2: Reference city 2");
		System.out.println("    Location: <longitude>, <latitude>");
		System.out.println("    Position of the city on the map image: <X>, <Y>");
	}
	void computeParameters(String ref1_loc_str, String ref1_pos_str, String ref2_loc_str, String ref2_pos_str){
		try{
			GeoAddress ref1_loc=new GeoAddress(ref1_loc_str);
			XYAddress ref1_pos=new XYAddress(ref1_pos_str);
			GeoAddress ref2_loc=new GeoAddress(ref2_loc_str);
			XYAddress ref2_pos=new XYAddress(ref2_pos_str);
			
			x_scale = ((double)(ref1_pos.getX()-ref2_pos.getX()))/(ref1_loc.longitude-ref2_loc.longitude);
			x_dist = ((double)ref1_pos.getX()- ref1_loc.longitude*x_scale);
			
			y_scale = ((double)(ref1_pos.getY()-ref2_pos.getY()))/Math.log(((1+Math.sin(Math.PI/180*ref1_loc.latitude))*(1-Math.sin(Math.PI/180*ref2_loc.latitude)))/((1-Math.sin(Math.PI/180*ref1_loc.latitude))*(1+Math.sin(Math.PI/180*ref2_loc.latitude))));
			y_dist = ((double)ref1_pos.getY())- y_scale*Math.log(((1+Math.sin(Math.PI/180*ref1_loc.latitude))/((1-Math.sin(Math.PI/180*ref1_loc.latitude)))));
			
			//System.out.println("Mercator Projection");
			//System.out.println("Parameter1="+x_scale);
			//System.out.println("Parameter2="+x_dist);
			//System.out.println("Parameter3="+y_scale);
			//System.out.println("Parameter4="+y_dist);
		}catch(Exception e){
			e.printStackTrace();
		}
	}

	public String getName() {
		return name;
	}

	public Image getImage() {
		return mapImage;
	}

	public GeoAddress getNW() {
		return nw_p;
	}

	public GeoAddress getSE() {
		return se_p;
	}

	public int getHeight() {
		return height;
	}

	public int getWidth() {
		return width;
	}

	public XYAddress getPositionOnMap(GeoAddress b) {
		GeoAddress a=new GeoAddress(b.longitude+b.adjustX, b.latitude+b.adjustY);
		int x = (int)(x_scale*a.longitude + x_dist);
		int y = (int)((1*y_scale*Math.log((1+Math.sin(Math.PI/180*a.latitude))/(1-Math.sin(Math.PI/180*a.latitude))))+y_dist);
		return new XYAddress(x, y);

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see Map#getEffHeight()
	 */
	public int getEffHeight() {
		return effHeight;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see Map#getEffWidth()
	 */
	public int getEffWidth() {
		return effWidth;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see Map#getEffStartPoint()
	 */
	public XYAddress getEffStartPoint() {
		return startPoint;
	}
}

