/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

/* 
 * Created on Jul 7, 2005
 *
 * TODO To change the template for this generated file go to
 * Window - Preferences - Java - Code Style - Code Templates
 */

package gui;

import hypercast.HyperCastStatsException;
import hypercast.I_Address;
import hypercast.I_LogicalAddress;
import hypercast.MonitorAndControl.MonMessage;
import runcontrol.RunControl;

/**
 * @author HyperCast Team
 * @author Wittawat Tantisiriroj
 */
public class RunCTNetwork extends RunNetwork {

	/**
	 * @param runControl
	 */
	public RunCTNetwork(RunControl runControl) {
		super(runControl);
	}

	/* (non-Javadoc)
	 * @see gui.RunNetwork#parseLogicalAddress(java.lang.String)
	 */
	public I_LogicalAddress parseLogicalAddress(String la) {
		return new CT_LogicalAddress(la);
	}

	/* (non-Javadoc)
	 * @see gui.RunNetwork#createRunNodeInst(gui.physicalInterface, int, int)
	 */
	public RunNode createRunNodeInst(physicalInterface p, int pIndex, int sIndex) {
		return new RunCTNode(p, pIndex, sIndex);
	}

	/* (non-Javadoc)
	 * @see gui.RunNetwork#getLogicalNextHop(gui.OverlayNode, gui.OverlayNode)
	 */
	public OverlayNode getLogicalNextHop(OverlayNode src, OverlayNode dst) {
		return null;
	}

	/**
	 * add a node to the overlay network
	 */
	//synchronized public OverlayNode addNode(Element sock, int pIndex, int
	// sIndex, boolean running){
	synchronized public OverlayNode addNode(MonMessage reply, String prefix, int pIndex, int sIndex,
			boolean running) {
		RunNode rnode = this.createRunNodeInst(null, pIndex, sIndex);
		//rnode.setRunNetwork(this);

                try
                {
		//rnode.readQueryResult(sock);
		if (!rnode.readQueryResult(reply, prefix))
			return null;
		} catch (HyperCastStatsException e) {
			lotusDebug.errorln(e.toString());
			return null;
                }
		
		OverlayNode onode=findNode(rnode.getPhysicalNetworkAddress());
		//OverlayNode onode = findNode(rnode.getLogicalAddress(), rnode.getOverlayID());
		//OverlayNode onode=this.findNode(pIndex, sIndex);

		if (onode != null) {
			//this node exists
			//dtnode=new RunDTNode(onode, la, pIndex, sIndex);
			RunNode rnode2 = (RunNode) onode.getProtocol();
			if(!rnode2.getLogicalAddress().equals(rnode.getLogicalAddress())){
				removeNode(onode);
				((RunCTNode)rnode).setXYAddress(rnode2.getXYAddress());
			}
			else{
				rnode = rnode2;
				try {
					//rnode.readQueryResult(sock);
					rnode.readQueryResult(reply, prefix);
				} catch (HyperCastStatsException e1) {
					lotusDebug.errorln(e1.toString());
					return null;
				}
	
				if (rnode.running)
					rnode.setStatus(RunNode.ACTIVE); //normal node
				else if (running)
					rnode.setStatus(RunNode.NEW); //new node
	
				rnode.running = running;
				rnode.portalIndex = (pIndex);
				rnode.socketIndex = (sIndex);
	
				if (!running)
					onode.clearLogicalEdges();
	
				activateNode(onode, rnode);
	
				rnode.touch();
	
				//if(!isValidNode(dtnode))dtnode.setStatus(RunNode.INVALID);
				setLogical();
	
				return onode;
			}
		}

		I_LogicalAddress la = rnode.getLogicalAddress();
		//Guangyu 12-04-03
		//onode=addNode(new XYAddress((int)la.getX(), (int)la.getY()),
		// dtnode.getOverlayID(), dtnode.getPhysicalNetworkAddress());
		onode = addNode(rnode.getXYAddress(), rnode.getOverlayID(), rnode
				.getPhysicalNetworkAddress());
		onode.setName("(" + la + ")");

		rnode.setPhysical(onode);
		rnode.setRunNetwork(this);

		rnode.setStatus(RunNode.NEW); //new node
		rnode.running = running;

		activateNode(onode, rnode);

		try {
			rnode.readQueryResult(reply, prefix);
		} catch (HyperCastStatsException e1) {
			lotusDebug.errorln(e1.toString());
			return null;
		}

		//if(!isValidNode(dtnode))dtnode.setStatus(RunNode.INVALID);

		setLogical();
		return onode;
	}
}

/**
 * Dummy LogicalAddress class for CT protocol
 * 
 * @author HyperCast Team
 * @author Wittawat Tantisiriroj
 */
final class CT_LogicalAddress implements hypercast.I_LogicalAddress
{
    String laStr;
    public CT_LogicalAddress (String nodeId)
    {
        laStr=nodeId;
    }

    public boolean equals (Object object)
    {
        if (! (object instanceof CT_LogicalAddress))
        {
            throw new IllegalArgumentException ("Object is not of type CT_LogicalAddress");
        }
        if(object==null)
        	return false;
        
        CT_LogicalAddress other = (CT_LogicalAddress) object;
        return this.toString().equals(object.toString());
    }

    
    public String toString ()
    {
        return laStr;
    }

	/* (non-Javadoc)
	 * @see hypercast.I_Address#greaterThan(hypercast.I_Address)
	 */
	public boolean greaterThan(I_Address a) {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see hypercast.I_Address#toByteArray()
	 */
	public byte[] toByteArray() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see hypercast.I_Address#getSize()
	 */
	public int getSize() {
		// TODO Auto-generated method stub
		return 0;
	}    
}
