/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

/* 
 * @author HyperCast Team
 *  Mar 2003 
 */

package gui;

import hypercast.HyperCastStatsException;
import hypercast.util.XmlUtil;

import java.io.BufferedReader;
import java.io.File;
import java.io.FilenameFilter;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.Reader;
import java.net.URL;

import java.applet.Applet;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;

import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Checkbox;
import java.awt.CheckboxMenuItem;
import java.awt.Choice;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FileDialog;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.Image;
import java.awt.Label;
import java.awt.MenuItem;
import java.awt.Panel;
import java.awt.Point;
import java.awt.PopupMenu;
import java.awt.TextArea;
import java.awt.TextField;
import java.awt.Toolkit;

import javax.swing.ImageIcon;
import javax.swing.JTextField;
import javax.swing.JComboBox;
import javax.swing.JSplitPane;
import javax.xml.transform.TransformerException;

import commandline.RunControlCLI;

import runcontrol.RunControl;
import runcontrol.CommandFailureException;

// *** PASTRY *** - uncomment this
//import hypercast.Pastry.Pastry_RC2;

/*
 * TODO
 *
 * input area has focus by default on startup
 * various commands are available via menus (perhaps chosing these menu options makes the command appear in the text area)
 * The large text area automatically scrolls to the bottom when new information is written there
 * Make command interrupt work properly
 * AWT -> Swing
 * help command sorts commands alphabetically
 * BUG: select the drop down for the text entry, use alt-tab to focus another window, runcontrol gui stays on top (linux)
 */

/**
 * Main Graphics User Interface Window
 * original version created in LoTos simulator
 * revised by Guangyu Dong, Mar 19, 2003
 * to fit the functionality of Monitor.
 */
public class RunControlGUI
    extends Applet
    implements KeyListener, ActionListener, ItemListener
{
    // NOTE: most of the members having package protection are used by the GUI_DisplayWindow class.
    // TODO check out all of these and determine if they should be here or elsewhere, or should be accessed differently

    /** name of the file that stores command histories */
    private static final String DEFAULT_COMMAND_HISTORY_FILENAME = ".RC_Cmd_history";

    public final static String DEFAULT_PROTOCOL = "NONE";
    public final static String DEFAULT_CONFIGFILENAME = "RunControlSocket.xml";

    private String protocol = DEFAULT_PROTOCOL;
    private String configurationFilename = DEFAULT_CONFIGFILENAME;

    /** background color */
    private static final Color bkColor = new Color(223, 223, 223);

    /** background color for display window */
    /* package */ static final Color dwColor = new Color(160, 160, 223);

    /** main window size */
    private static final Dimension winSize = new Dimension(1024, 768);

    /** node color */
    private static final Color ndColor = Color.black;

    /** node size */
    /* package */ final static int ndDiameter = 8;

    /** physical link color */
    private final static Color edgeColor = Color.black;

    /** time interval in millisecond between two updating */
    private int simSleep = 1000;

    /**
     * items below are derived from the LoTos GUI.
     */
    /* package */ final static Color packetColor = Color.green;
    /* package */ final static int packetSRadius = 10;
    /* package */ final static int packetLRadius = 60;
    /* package */ final static int packetAngle = 24;

    /* package */ static float zoominFactor = 1.25f;
    /* package */ static float zoomoutFactor = 0.75f;

    /** the object periodically updates the display window 
     *  by sending message to overlay application to query
     *  overlay socket states
     */
    private AutoUpdater autoUpdater;

    /** a handle to the simulated network */
    private RunNetwork gOverlay;

    /** a RunControl object */
    /* package */ RunControl runControl;

    /** applet mode or application mode */
    private boolean isApplet;

    /** Main application window */
    private Frame theApplicationWindow;

    /** the main display window showing the network structure */
    private GUI_DisplayWindow dw;

    /** Menu definitions */

    /** Main Menus */
    private PopupMenu pm_file;
    private MenuItem mi_quit;

    private MenuItem mi_restart;
    private MenuItem mi_save;
    private MenuItem mi_open;

    /** View menu and contained items */
    private PopupMenu pm_packet_view;
    private CheckboxMenuItem cbmi_display_logical_net;
    private CheckboxMenuItem cbmi_display_node_labels;
    private CheckboxMenuItem cbmi_display_node_phyaddr;
    private CheckboxMenuItem cbmi_display_node_hostname;

    /** menu items to zoo in and out the display */
    private MenuItem mi_zoomin;
    private MenuItem mi_zoomout;
    private MenuItem mi_bestfit;
    private MenuItem mi_restore;
    private CheckboxMenuItem cbmi_autofit;

    // Buttons which activate the  menues
    private Button b_filemenu;
    private Button b_viewmenu;
    private Button b_update;
    private Button b_reset;
    private Button b_autoupdate;

    // Buttions which enable the view history
    private Button b_backview;
    private Button b_forwardview;

    // let user to choose the overlay network to display
    private Choice ch_overlayid;
	
    // let user to choose map
    private Choice ch_map;
    
    // let user to choose protocol
    private Choice ch_protocol;

    private TextField tf_timer;
    /* package */ TextField tf_search;

    /* package */ Checkbox cb_hideother;

    private Checkbox cb_info;
    private Checkbox cb_tree;
    private Checkbox cb_move;
    private Checkbox cb_center;
    private Checkbox cb_zoomin;
    private Checkbox cb_zoomout;
    private Checkbox cb_zoomwin;
    private Checkbox cb_remove;

    private Checkbox cb_log; //logical view
    private Checkbox cb_geo; //geographic view

    private Button b_tree;
    private Button b_move;

    // Run control items
    private Panel run_palette;
    private Panel edit_palette;

    // for warning message at bottom
    private Label l_warning_message;

    // for inputing command line
    private JTextField tf_runControl_input;
    /* package */ JComboBox cb_runControl_input;

    private Button b_runControl_submit;
    private Button b_runControl_stop;
    //private Button b_runControl_interrupt;

    // for showing runControl command output
    private TextArea ta_runControl_message;

    private JSplitPane split_pane;

    private GUICommandRunner foregroundCommand = null;

    /**
     * This object is passed to a running command. Anything written to
     * this stream will be automatically displayed in the text area of
     * the GUI. The PrintWriter also provides synchronization so that
     * writes from various commands are atomic.
     */
    private PrintWriter commandOutputWriter = new PrintWriter (new CommandOutputStream(), true);
    private CommandReader commandIS = new CommandReader();
    private BufferedReader commandInputReader = new BufferedReader(commandIS);

   

    /**
     * @see #main
     * @see #init
     */
    public RunControlGUI ()
    {
       //do nothing
    }

    /* package */ RunNetwork getOverlay()
    {
        return gOverlay;
    }
    
    /**
     * Prints a short message at the bottom of the GUI interface.
     */
    public void printWarning (final String s)
    {
        l_warning_message.setText (s);
    }

    /**
     * Displays the text history of the interaction between the user and gui.
     */
    public void printMessage (final String s)
    {
        ta_runControl_message.append (s);
        ta_runControl_message.setCaretPosition (Integer.MAX_VALUE);
    }

    // TODO: can there be more than one overlay running simultaneously with this gui???
    // TODO: which threads are competing for this?
    synchronized void updateOverlayIDList()
    {
        String [] idList = gOverlay.getOverlayIDList();
        boolean changed = false;

        if ((idList.length + 1) != ch_overlayid.getItemCount())
        {
            changed = true;
        }
        else
        {
            for (int i = 0; i < idList.length; i++)
            {
                String id = ch_overlayid.getItem (i + 1);
                if (! id.equals (idList[i]))
                {
                    changed = true;
                    break;
                }
            }
        }

        if (changed)
        {
            ch_overlayid.removeAll();
            ch_overlayid.addItem ("All Networks");
            for (int i = 0; i < idList.length; i++)
            {
                ch_overlayid.addItem (idList[i]);
            }
        }

        if (gOverlay.getOverlayID() == null)
        {
            if (ch_overlayid.getSelectedIndex() != 0)
            {
                ch_overlayid.select (0);
            }
        }
        else if (! ch_overlayid.getSelectedItem().equals (gOverlay.getOverlayID()))
        {
            ch_overlayid.select (gOverlay.getOverlayID());
        }
    }

    //update nodes and repaint them
    // TODO: what exactly is going on here? It looks like the autoupdater is a new thing that does what this used to do.
    // TODO: which threads are competing for this?
    synchronized void updateNodes()
    {

        if (gOverlay.running)
        {
            return;
        }

        if (autoUpdater != null
            &&
            autoUpdater.running)
        {
            return;
        }

        autoUpdater = new AutoUpdater (true);
        new Thread (autoUpdater, "GUI Automatic Display Updater").start();
    }

    // why updateNodes() and doUpdateNodes()???
    // TODO: which threads are competing for this?
    synchronized void doUpdateNodes()
    {
        if (foregroundCommand != null)
        {
            return;
        }
		
        // TODO: HUH?
        /*
        if (runControl.getExperiment() != null
            &&
            ((runControl.experimentMode == RunControl.MODE_RUNSERVER
              &&
              runControl.getExperiment().state != Experiment.STOPPED
              &&
              runControl.getExperiment().state != Experiment.NEW)
             ||
             (runControl.experimentMode == RunControl.MODE_APPLICATION
              &&
              runControl.getExperiment().state == Experiment.RUNNING)))
         
        {
        */
        
        gOverlay.sendUnblockedQuests();
        
        /*
        }
        else if (! gOverlay.showingFile)
        {
            gOverlay.clearNodes();
        }
        */
        dw.repaint();

        updateOverlayIDList();
    }
    private String mapProtocolString(String displayString){
    	if (displayString.equals ("Delaunay Triangulation"))
        {
            return "DT";
        }
        else if (displayString.equals ("Spanning Tree"))
        {
            return "SPT";
        }
        else if (displayString.equals ("Hyper Cube"))
        {
            return "HC";
        }
        else if (displayString.equals ("Any"))
        {
            return "NONE";
        }
        else if (displayString.equals ("Pastry"))
        {
            return "Pastry";
        }
        else if (displayString.equals ("Clustering"))
        {
            return "CT";
        }
        else
        {
            System.err.println ("Protocol not supported: " + displayString);
            System.exit (1);
        }
    	return null;
    }
    private RunNetwork createNetwork (final RunControl runControl)
    {
        if (protocol.equals ("DT"))
        {
            return new RunDTNetwork (runControl);
        }
        else if (protocol.equals ("SPT"))
        {
            return new RunSPTNetwork (runControl);
        }
        else if (protocol.equals ("HC"))
        {
            return new RunHCNetwork (runControl);
        }
        else if (protocol.equals ("Pastry"))
        {
            return new RunPastryNetwork (runControl);
        }
        else if (protocol.equals ("NONE"))
        {
            return new RunNONENetwork (runControl);
        }
        else if (protocol.equals ("CT"))
        {
            return new RunCTNetwork (runControl);
        }
        else
        {
            System.err.println ("Protocol not supported: " + protocol);
            System.exit (1);
        }
        return null;
    }
	
    /**
     * Reads command history from a file.
     *
     * @param historyfile The file from which a command history is read.
     *
     * @return true if command history file was read successfully.
     */
    private boolean readCommandHistory (final String historyfile)
    {
        BufferedReader input = null;
		
        try
        {
            input = new BufferedReader (new FileReader (historyfile));
		
            String line = input.readLine().trim();
            while (line != null)
            {
                line = line.trim();
                if (! line.equals (""))
                {
                    cb_runControl_input.addItem (line);
                }
                line = input.readLine();
            }
        }
        catch (IOException ioe)
        {
            // TODO: this is a gui: popup a dialog
            System.err.println ("Error while reading command history file \"" + historyfile + "\": " + ioe.toString());
            return false;
        }
        finally
        {
            try { if (input != null) input.close(); } catch (IOException ignored) { }
        }
        return true;
    }

    /**
     * Writes commands history to a file.
     *
     * @param historyfile The file to which command history is written
     */
    private void writeCommandHistory (final String historyfile)
    {
        System.out.println ("Saving command history..."); // TODO: this is a gui...
        PrintWriter output = null;
		
        try
        {
            output = new PrintWriter (new FileWriter (historyfile));
			
            for (int i = 0; i < cb_runControl_input.getItemCount(); i++)
            {
                output.println (cb_runControl_input.getItemAt (i).toString());
            }
        }
        catch (final IOException ioe)
        {
            // TODO: this is a gui: popup a dialog
            System.err.println ("Can't write to " + historyfile + " : " + ioe.toString());
        }
        finally
        {
            if (output != null) output.close();
        }
    }
	
    /**
     * Updates the command history after a new command has been
     * issued. If command already appears in the history, places the
     * command at the most recently issued position of the history.
     */
    private void updateCommandHistory (final String input)
    {
        if (input == null)
            throw new NullPointerException();

        if (! input.equals (""))
        {
            synchronized (cb_runControl_input)
            {
                for (int i = 0; i < cb_runControl_input.getItemCount(); i++)
                {
                    final String current = (String) cb_runControl_input.getItemAt (i);
                    if (current.equals (input))
                    {
                        cb_runControl_input.removeItemAt (i);
                        break;
                    }
                }
                cb_runControl_input.insertItemAt (input, 1);
            }
        }
    }

    /**
     * Initializes the GUI widgets.
     * TODO: can most of these "new's" be added to the declaration of the instance variables? It would eliminate a lot of lines of code.
     */
    public void init(String args[])
    {
        isApplet = (this.theApplicationWindow == null);

        runControl = RunControlCLI.createRunControl(args, "gui.RunControlGUI");
        gOverlay = createNetwork (runControl);

        setBackground (bkColor);

        /*
         * Create file menu
         */
        pm_file = new PopupMenu ("File");
        
        /* now the "Open" and "Save" is disabled 
         * because no one need to use it.
         */
        /*
        mi_open = new MenuItem ("Open");
        pm_file.add (mi_open);
        mi_open.addActionListener (this);

        mi_save = new MenuItem ("Save");
        pm_file.add (mi_save);
        mi_save.addActionListener (this);
        */

        /*
          // TODO: why is restart disabled? should it be removed? "restart == reset"?
          mi_restart = new MenuItem ("Restart RunControl");
          pm_file.add (mi_restart);
          mi_restart.addActionListener (this);
        */

        if (this.isApplet != true)
        {
            pm_file.addSeparator();

            mi_quit = new MenuItem ("Quit");
            pm_file.add (mi_quit);
            mi_quit.addActionListener (this);
        }

        this.add (pm_file);

        /*
         * Create view menu
         */
        pm_packet_view = new PopupMenu ("View");

        mi_zoomin = new MenuItem ("zoomin");
        pm_packet_view.add (mi_zoomin);
        mi_zoomin.addActionListener (this);

        mi_zoomout = new MenuItem ("zoomout");
        pm_packet_view.add (mi_zoomout);
        mi_zoomout.addActionListener (this);

        mi_restore = new MenuItem ("actual size");
        pm_packet_view.add (mi_restore);
        mi_restore.addActionListener (this);

        mi_bestfit = new MenuItem ("best fit");
        pm_packet_view.add (mi_bestfit);
        mi_bestfit.addActionListener (this);

        cbmi_autofit = new CheckboxMenuItem ("auto fit", true);
        pm_packet_view.add (cbmi_autofit);
        cbmi_autofit.addItemListener (this);

        pm_packet_view.addSeparator();

        cbmi_display_logical_net = new CheckboxMenuItem ("Show Logical Links", true);
        pm_packet_view.add (cbmi_display_logical_net);
        cbmi_display_logical_net.addItemListener (this);

        cbmi_display_node_labels = new CheckboxMenuItem ("Show Logical Addresses", true);
        pm_packet_view.add (cbmi_display_node_labels);
        cbmi_display_node_labels.addItemListener (this);

        cbmi_display_node_phyaddr = new CheckboxMenuItem ("Show Physical Addresses", false);
        pm_packet_view.add (cbmi_display_node_phyaddr);
        cbmi_display_node_phyaddr.addItemListener (this);

        cbmi_display_node_hostname = new CheckboxMenuItem ("Show Host Names", true);
        pm_packet_view.add (cbmi_display_node_hostname);
        cbmi_display_node_hostname.addItemListener (this);

        this.add (pm_packet_view);

        /*
         * Create layout
         */
        setLayout (new BorderLayout());

        Panel north_panel = new Panel();
        Panel north_panel2 = new Panel ();
        
        north_panel.setLayout (new FlowLayout (FlowLayout.LEFT));
        north_panel2.setLayout (new FlowLayout (FlowLayout.LEFT));

        b_filemenu = new Button ("File");
        north_panel.add (b_filemenu);
        b_filemenu.addActionListener (this);

        b_viewmenu = new Button ("View");
        north_panel.add (b_viewmenu);
        b_viewmenu.addActionListener (this);

        b_update = new Button ("Update");
        north_panel.add (b_update);
        b_update.addActionListener (this);

        b_reset = new Button ("Reset");
        north_panel.add (b_reset);
        b_reset.addActionListener (this);
        
        b_backview = new Button ("<<");
        north_panel.add (b_backview);
        b_backview.addActionListener (this);

        b_forwardview = new Button (">>");
        north_panel.add (b_forwardview);
        b_forwardview.addActionListener (this);

        north_panel.add (new Label ("  Auto Update Timer(ms)"));
        tf_timer = new TextField ("2000");
        north_panel.add (tf_timer);
        
        b_autoupdate = new Button ("Start");
        north_panel.add (b_autoupdate);
        b_autoupdate.addActionListener (this);
        
        north_panel.add (new Label ("Search"));

        tf_search = new TextField ("", 10);
        north_panel.add (tf_search);
        tf_search.addKeyListener (this);

        cb_hideother = new Checkbox ("Hide Others", false);
        north_panel.add (cb_hideother);
        cb_hideother.addItemListener (this);
        

        ch_overlayid = new Choice();
        this.updateOverlayIDList();
        north_panel2.add(new Label("Select Overlay"));
        north_panel2.add (ch_overlayid);
        ch_overlayid.addItemListener (this);

        

        //add (BorderLayout.NORTH,  north_panel);

        /*
         * Add controls for RunControl
         */
        Panel runControl_buttion_panel = new Panel();
        //runControl_buttion_panel.setLayout (new BorderLayout());
        
        Panel runControl_panel = new Panel();
        runControl_panel.setLayout (new BorderLayout());

        Panel runControl_input_panel = new Panel();
        runControl_input_panel.setLayout (new BorderLayout());

        l_warning_message = new Label ("RunControl Started.");
        runControl_input_panel.add (BorderLayout.SOUTH, l_warning_message);

        /*
         * Text entry field for entering RunControl commands
         */
        cb_runControl_input = new JComboBox(); // was AWT TextField
        cb_runControl_input.addItem ("");
        cb_runControl_input.setEditable (true);
        cb_runControl_input.addKeyListener (this);

        // for listening to the key event of JComboBox, we have to do this
        tf_runControl_input = (JTextField) cb_runControl_input.getEditor().getEditorComponent();
        tf_runControl_input.addKeyListener (this);

        boolean commandHistoryFileSuccess = this.readCommandHistory (DEFAULT_COMMAND_HISTORY_FILENAME);
        if (! commandHistoryFileSuccess)
        {
            /*
             * Add some frequently used commands
             */
            cb_runControl_input.addItem ("list_portals");
            cb_runControl_input.addItem ("list_sockets");
            cb_runControl_input.addItem ("start_sockets");
            cb_runControl_input.addItem ("stop_sockets");
            cb_runControl_input.addItem ("flush_sockets");
            
        }

        b_runControl_submit = new Button ("Run");
        tf_runControl_input.setBackground(Color.white);
        
        b_runControl_stop = new Button ("Stop");
        b_runControl_stop.setEnabled(false);
        
        b_runControl_submit.addActionListener (this);
        b_runControl_stop.addActionListener (this);
        
        
		
/*
        b_runControl_interrupt = new Button ("Stop");
        b_runControl_interrupt.addActionListener (this);

        runControl_input_panel.add (BorderLayout.EAST, b_runControl_interrupt); // TODO: useful?
*/
        
        runControl_buttion_panel.add (b_runControl_submit);
        runControl_buttion_panel.add (b_runControl_stop);
        
        runControl_input_panel.add (BorderLayout.EAST, runControl_buttion_panel);
        runControl_input_panel.add (BorderLayout.CENTER, cb_runControl_input);

        ta_runControl_message = new TextArea (5, 10);
        ta_runControl_message.setEditable (false);
        runControl_panel.add (BorderLayout.SOUTH, runControl_input_panel);
        runControl_panel.add (BorderLayout.CENTER, ta_runControl_message);

        // add (BorderLayout.SOUTH, runControl_panel); // TODO useful?

        // Image loading is different for applet-based case and application TODO: what does this mean/refer to?

        run_palette = new Panel();
        run_palette.setLayout (new GridLayout(0, 1));

        edit_palette = new Panel();
        edit_palette.setLayout (new GridLayout (0, 1));

        cb_info = new Checkbox ("Select", true);
        edit_palette.add (cb_info);
        cb_info.addItemListener (this);

        cb_tree = new Checkbox ("Tree", false);
        edit_palette.add (cb_tree);
        cb_tree.addItemListener (this);

        cb_move = new Checkbox ("Move", false);
        edit_palette.add (cb_move);
        cb_move.addItemListener (this);

        cb_center = new Checkbox ("Center", false);
        edit_palette.add (cb_center);
        cb_center.addItemListener (this);

        cb_zoomin = new Checkbox ("Z+", false);
        edit_palette.add (cb_zoomin);
        cb_zoomin.addItemListener (this);

        cb_zoomout = new Checkbox ("Z-", false);
        edit_palette.add (cb_zoomout);
        cb_zoomout.addItemListener (this);

        cb_zoomwin = new Checkbox ("Z Win", false);
        edit_palette.add (cb_zoomwin);
        cb_zoomwin.addItemListener (this);

        edit_palette.add (new Label(""));

        cb_remove = new Checkbox ("Delete", false);
        edit_palette.add (cb_remove);
        cb_remove.addItemListener (this);

        edit_palette.add (new Label("")); // seperator

        cb_log = new Checkbox ("LView", true);
        edit_palette.add (cb_log);
        cb_log.addItemListener (this);

        cb_geo = new Checkbox ("GView", false);
        edit_palette.add (cb_geo);
        cb_geo.addItemListener (this);

        Panel dw_panel = new Panel();
        dw_panel.setLayout (new BorderLayout());

        Panel west_panel = new Panel();
        west_panel.setLayout (new BorderLayout());

        west_panel.add (BorderLayout.SOUTH, run_palette);
        west_panel.add (BorderLayout.NORTH, edit_palette);

                

        /*
         * Create DisplayWindow
         */
        dw = new GUI_DisplayWindow (this);
        dw_panel.add (BorderLayout.NORTH, north_panel);
        
        Panel panel2=new Panel();
        panel2.setLayout (new BorderLayout());
        
        dw_panel.add(BorderLayout.CENTER, panel2);
        panel2.add (BorderLayout.NORTH, north_panel2);
        panel2.add (BorderLayout.WEST, west_panel);
        
        panel2.add (BorderLayout.CENTER, dw);

        split_pane = new JSplitPane (JSplitPane.VERTICAL_SPLIT, true, dw_panel, runControl_panel);
        split_pane.setOneTouchExpandable (true);
        split_pane.setDividerLocation (400);
        //split_pane.setDividerSize (5); // TODO: useful?

        add (BorderLayout.CENTER, split_pane);

        dw.setEditMode (GUI_DisplayWindow.EDIT_NODE);
		
        /*
         * Initialize the map
         */
        MapFactory mapFactory = new MapFactory ("map.xml", this.theApplicationWindow);
		
        dw.setMapFactory (mapFactory);
		
        // choices for maps
        ch_map = new Choice();
        ch_map.add ("Automatic map");
        for (int i = 0; i < mapFactory.size(); i++)
        {
            ch_map.add (mapFactory.getMap (i).getName());
        }
        north_panel2.add(new Label("Select Map"));
        north_panel2.add (ch_map);
        ch_map.addItemListener (this);
        
        ch_protocol = new Choice();
        
        ch_protocol.add("Any");
        ch_protocol.add("Delaunay Triangulation");
        ch_protocol.add("Hyper Cube");
        ch_protocol.add("Spanning Tree");
        ch_protocol.add("Pastry");
        ch_protocol.add("Clustering");
        ch_protocol.addItemListener(this);
        
        north_panel2.add(new Label("Select Protocol"));
        north_panel2.add(ch_protocol);

        /*
         * This ensures that these things will happen no matter how
         * the gui is terminated (File->Exit, Close button, Ctrl-C).
         *
         * The only thing that can stop these operations from occuring
         * is "kill -9," a power failure, etc.
         */ 
/*

// TODO: this only works on Linux - it seems like there is some bug in
//  the Windows impl of JVM that causes it to hang when this is run

        Runtime.getRuntime().addShutdownHook (new Thread()
            {
                public void run()
                {
                    final String shutdownMessage = "Shutting down Run Control GUI...";
                    printMessage (shutdownMessage);
                    System.out.println (shutdownMessage);
                    System.out.flush();
                    writeCommandHistory (DEFAULT_COMMAND_HISTORY_FILENAME);
                }
            });
*/

        this.repaint();
        dw.repaint();
        dw.recordCurrentView();

        DNSCache.toUpdate = dw;

        startAutoUpdate();
    }

    /**
     * Called when the applet starts running or becomes visible
     */
    public void start()
    {
    }

    /**
     * Called when the applet is no longer visible
     */
    public void stop()
    {
    }

    /**
     * handles buttons, menu items, returns entered in textfield
     */
    public void actionPerformed (final ActionEvent ae)
    {
        // synchronizing to prevent bad interaction from autoUpdater thread
        if (ae.getSource() == b_filemenu)
        {
            Point p = b_filemenu.getLocation();
            pm_file.show (this, p.x, p.y);
        }
        else if (ae.getSource() == b_viewmenu)
        {
            Point p = b_viewmenu.getLocation();
            pm_packet_view.show (this, p.x, p.y);
        }
        else if (ae.getSource() == mi_restart) // TODO: restart appears to be disabled elsewhere in this file
        {
            runControl.getMonitor().closeMonitor();

            runControl = new RunControl (configurationFilename);

            gOverlay.setRunControl (runControl);
            
            String la = "Unknown";
        	
        	try {
				la = XmlUtil.getScalarStatsResult(runControl.getMonitor(), "/Monitor/Socket/Node/LogicalAddress");
			} catch (HyperCastStatsException e) {
				e.printStackTrace();
			} catch (TransformerException e) {
				e.printStackTrace();
			}
            
            printMessage ("RunControl restarted at"+la+"...\n");
            printWarning ("RunControl Restarted.");
        }
        else if (ae.getSource() == mi_save)
        {
            synchronized (gOverlay)
            {
                String filename = getFileName(FileDialog.SAVE);
                if (filename != null)
                {
                    if (gOverlay.protocolWriter(filename) == false)
                    {
                        this.printWarning("save file fails");
                    }
                    else
                    {
                        this.printWarning("file saved");
                    }
                }
            }
        }
        else if (ae.getSource() == mi_open)
        {
            synchronized (gOverlay)
            {
                stopAutoUpdate();
                String filename = getFileName (FileDialog.LOAD);
                if (null != filename)
                {
                    if (gOverlay.protocolReader (filename) == false)
                    {
                        this.printWarning ("load file fails");
                    }
                    else
                    {
                        this.printWarning ("file loaded");
                    }
                    dw.repaint();
                    updateOverlayIDList();
                }
            }
        }
        else if (ae.getSource() == mi_quit)
        {
            quitRunControl();
        }
        else if (ae.getSource() == mi_zoomin)
        {
            cbmi_autofit.setState (false);
            dw.setAutoFit (false);

            dw.scaleBy (zoominFactor);
            dw.repaint();

            dw.recordCurrentView();
        }
        else if (ae.getSource() == mi_zoomout)
        {
            cbmi_autofit.setState (false);
            dw.setAutoFit (false);

            dw.scaleBy (zoomoutFactor);
            dw.repaint();

            dw.recordCurrentView();
        }
        else if (ae.getSource() == mi_bestfit)
        {
            dw.bestfit();
            printWarning ("scale:" + dw.scale + "; viewX:" + dw.viewX + "; viewY:" + dw.viewY);
            dw.repaint();

            dw.recordCurrentView();
        }
        else if (ae.getSource() == mi_restore)
        {
            cbmi_autofit.setState (false);
            dw.setAutoFit (false);

            dw.restore();
            dw.repaint();

            dw.recordCurrentView();
        }
        else if (ae.getSource() == b_runControl_submit )
        {
            submitLine();
        }
        else if(ae.getSource() == b_runControl_stop){
        	stopRunControlCommand();
        }
        
/*
        else if (ae.getSource() == b_runControl_interrupt)
        {
            runControl.interruptCommand();
        }
*/
        else if (ae.getSource() == b_update)
        {
            updateNodes();
        }
        else if (ae.getSource() == b_reset) // TODO: is this disabled or enabled?
        {
            gOverlay.reset();
            dw.repaint();
        }
        else if (ae.getSource() == b_autoupdate)
        {
            if (b_autoupdate.getLabel().equals("Stop"))
            {
                stopAutoUpdate();
            }
            else
            {
                startAutoUpdate();
            }
        }
        else if (ae.getSource() == b_backview)
        {
            dw.backView();
        }
        else if (ae.getSource() == b_forwardview)
        {
            dw.forwardView();
        }
    }

    // TODO: presentation of autoupdater in GUI is awkward
    // TODO: requires explanation
    private void stopAutoUpdate()
    {
        //gOverlay.stopUpdate();

        simSleep = Integer.parseInt (tf_timer.getText());

        //autoUpdater.stop();
        gOverlay.stopAutoUpdate();

        b_autoupdate.setLabel ("Start");
        tf_timer.setEnabled (true);
    }

    // TODO: requires explanation
    private void startAutoUpdate()
    {
        simSleep = Integer.parseInt (tf_timer.getText());

        gOverlay.startAutoUpdate (simSleep, dw);

        /*    
              autoUpdater= new AutoUpdater(false);
              autoUpdater.start();
        */
        if (! gOverlay.stopped)
        {
            b_autoupdate.setLabel ("Stop");
            tf_timer.setEnabled (false);
        }
    }

    /**
     * Causes the run control gui application to quit. Note that an
     * experiment could be executing. No check is made for this,
     * however, it is reasonable to at least prompt the use if they
     * want to quit even though an experiment is active. If the user
     * says "yes stop anyhow" there could be a further question that
     * asks if the experiment should be stopped gracefully first.
     *
     * TODO: answer the design question posed here.
     */
    private void quitRunControl()
    {
        if (isApplet)
        {
            printMessage ("Applet cannot quit!");
        }
        else
        {
            // TODO: these commands should be place in a shutdown hook.
            // The code for shutdown hook appears in this file, see the comment there.
        	try{
	            final String shutdownMessage = "Shutting down Run Control GUI...";
	            gOverlay.stopAutoUpdate();
	            printMessage (shutdownMessage);
	            System.out.println (shutdownMessage);
	            System.out.flush();
	            writeCommandHistory (DEFAULT_COMMAND_HISTORY_FILENAME);
	            //runControl.getMonitor().closeMonitor();
        	}finally{

	            // TODO: This should *NOT* be part of the shutdown
	            // hook. Calling exit during the shutdown sequence is a
	            // bad thing according to The Java Programming Language
	            // book.
	            System.exit (0);
        	}
        }
    }

    /**
     * Enables the close button on the main frame for an application;
     * grants initial focus to the text input area.
     */
    /* package */ void enableWindowListener() // called from main method
    {
        theApplicationWindow.addWindowListener(
            new WindowAdapter()
            {
                public void windowClosing (final WindowEvent ignored)
                {
                    quitRunControl();
                }

                public void windowOpened (final WindowEvent ignored)
                {
                	String la = "Unknown";
                	
                	try {
						la = XmlUtil.getScalarStatsResult(runControl.getMonitor(), "/Monitor/MonitorSocket/Node/LogicalAddress");
					} catch (HyperCastStatsException e) {
						e.printStackTrace();
					} catch (TransformerException e) {
						e.printStackTrace();
					}
                    printMessage ("RunControl started at "+la+"...\n");
                    cb_runControl_input.requestFocus();
                }
            }
        );
    }

    /*
     * NOTE: FileNameFilter doesn't work in Windows Environment
     */
    class XmlFileNameFilter implements FilenameFilter
    {
        public XmlFileNameFilter() { }

        public boolean accept (final File dir, final String name)
        {
            final int i = name.lastIndexOf (".");
            final String ext = name.substring (i).toUpperCase();
            return ext.equals (".XML");
        }
    }

    private String getFileName (final int save_or_load_state)
    {
        String win_title = (save_or_load_state == FileDialog.LOAD) ? "Load File" : "Save File";
        FileDialog fd = new FileDialog (theApplicationWindow, win_title, save_or_load_state);
        fd.setFilenameFilter (new XmlFileNameFilter());
        fd.show();

        // null -> Cancel button was pressed.
        String result = (fd.getFile() == null) ? null : fd.getDirectory() + fd.getFile();

        fd.dispose();
        return result;
    }

    /** 
     * TODO: why setViewMode and setViewState
     * TODO: validation
     */
    /* package */ void setViewMode (final int viewmode)
    {
        dw.setViewMode (viewmode);
        setViewState();
    }

    /* package */ void setViewState()
    {
        cb_log.setState (dw.view_mode == View.VIEW_LOGICAL);
        cb_geo.setState (dw.view_mode == View.VIEW_GEOGRAPHIC);

        cbmi_autofit.setState (dw.auto_fit);
        updateOverlayIDList();
    }

    /** 
     * Sets the mouse pointer mode
     * TODO: validation
     */
    /* package */ void setMode (final int mode)
    {
        dw.setEditMode (mode);

        cb_info.setState    (mode == GUI_DisplayWindow.EDIT_NODE);
        cb_move.setState    (mode == GUI_DisplayWindow.EDIT_MOVE);
        cb_tree.setState    (mode == GUI_DisplayWindow.EDIT_TREE);
        cb_center.setState  (mode == GUI_DisplayWindow.EDIT_CENTER);
        cb_zoomin.setState  (mode == GUI_DisplayWindow.EDIT_ZOOMIN);
        cb_zoomout.setState (mode == GUI_DisplayWindow.EDIT_ZOOMOUT);
        cb_zoomwin.setState (mode == GUI_DisplayWindow.EDIT_ZOOMWIN);
        cb_remove.setState  (mode == GUI_DisplayWindow.EDIT_REMOVE);
    }

    /*
     * Implementation of the KeyListener interface
     */
    public void keyPressed (final KeyEvent ignored) { }
    public void keyReleased (final KeyEvent ignored) { }

    public void keyTyped (final KeyEvent e)
    {
        if (e.getSource() == tf_runControl_input)
        {
            if (e.getKeyChar() == '\n')
            {
                submitLine();
            }
        }
        // TODO: what does this mean?
        else if (e.getSource() == tf_search)
        {
            dw.repaint();
        }
    }

    /**
     * handle the events from checked menu items
     */
    public void itemStateChanged (final ItemEvent ie)
    {
        if (ie.getSource() == ch_map)
        {
            dw.setMap (ch_map.getSelectedIndex());
        }
        else if (ie.getSource() == ch_protocol)
        {
            String new_protocol = mapProtocolString(ch_protocol.getSelectedItem());
            if(!new_protocol.equals(protocol)){
            	
            	protocol = new_protocol;
            	boolean running=gOverlay.running;
            	
            	gOverlay.clearNodes();
            	gOverlay.stopAutoUpdate();
            	gOverlay = createNetwork(this.runControl);
            	
            	if(running){
            		gOverlay.startAutoUpdate (simSleep, dw);
            	}
            		
            	
            }
        }
        else if (ie.getSource() == ch_overlayid)
        {
            final String overlayID = (ch_overlayid.getSelectedIndex() == 0) ? null : ch_overlayid.getSelectedItem();

            gOverlay.setOverlayID (overlayID);

            dw.repaint();
            dw.recordCurrentView();
        }
        else if (ie.getSource() == cbmi_autofit)
        {
            dw.setAutoFit (cbmi_autofit.getState());
            setMode (GUI_DisplayWindow.EDIT_NODE);
        }
        else if (ie.getSource() == cbmi_display_logical_net)
        {
            dw.setDisplayLogical (cbmi_display_logical_net.getState());
        }
        else if (ie.getSource() == cbmi_display_node_labels)
        {
            dw.setDisplayNodeLabels (cbmi_display_node_labels.getState());
        }
        else if (ie.getSource() == cbmi_display_node_phyaddr)
        {
            dw.setDisplayNodePhyAddr (cbmi_display_node_phyaddr.getState());
        }
        else if (ie.getSource() == cbmi_display_node_hostname)
        {
            dw.setDisplayNodeHostName (cbmi_display_node_hostname.getState());
        }
        else if (ie.getSource() == cb_info)
        {
            setMode (GUI_DisplayWindow.EDIT_NODE);
            dw.repaint();
        }
        else if (ie.getSource() == cb_tree)
        {
            setMode (GUI_DisplayWindow.EDIT_TREE);
            dw.repaint();
        }
        else if (ie.getSource() == cb_move)
        {
            setMode (GUI_DisplayWindow.EDIT_MOVE);
            cbmi_autofit.setState(false);
            dw.repaint();
        }
        else if (ie.getSource() == cb_center)
        {
            setMode (GUI_DisplayWindow.EDIT_CENTER);
            cbmi_autofit.setState(false);
            dw.repaint();
        }
        else if (ie.getSource() == cb_zoomin)
        {
            setMode (GUI_DisplayWindow.EDIT_ZOOMIN);
            cbmi_autofit.setState(false);
            dw.repaint();
        }
        else if (ie.getSource() == cb_zoomout)
        {
            setMode (GUI_DisplayWindow.EDIT_ZOOMOUT);
            cbmi_autofit.setState(false);
            dw.repaint();
        }
        else if (ie.getSource() == cb_zoomwin)
        {
            cbmi_autofit.setState(false);
            dw.setAutoFit(false);

            setMode (GUI_DisplayWindow.EDIT_ZOOMWIN);
            dw.repaint();
        }
        else if (ie.getSource() == cb_remove)
        {
            setMode (GUI_DisplayWindow.EDIT_REMOVE);
            dw.repaint();
        }
        else if (ie.getSource() == cb_log)
        {
            setViewMode(View.VIEW_LOGICAL);
            dw.repaint();
        }
        else if (ie.getSource() == cb_geo)
        {
            setViewMode(View.VIEW_GEOGRAPHIC);
            dw.repaint();
        }
        else if (ie.getSource() == cb_hideother)
        {
            dw.repaint();
        }
    }
    synchronized private void submitLine(){
    	if(foregroundCommand != null){
    		submitCommandInput();
    	}else{
    		submitRunControlCommand();
    	}
    }
    synchronized private void submitCommandInput(){
    	final String input = ((String) cb_runControl_input.getSelectedItem()).trim();
    	cb_runControl_input.setSelectedItem ("");
    	this.commandIS.writeLine(input+"\n");
    	this.commandOutputWriter.println(input);
    }
    synchronized private void stopRunControlCommand(){
    	if (foregroundCommand != null)
        {
            // TODO: Q: what caused this to happen? A: another command was issues. Is new command is simply ignored
            // TODOTODOTODO: interrupt current command and issue the next one!
            foregroundCommand.interruptCommand();
            this.printWarning("Interrupting RunControl command...");
            //runControl.interruptCommand(); // TODO: perhaps this should be the ultimate solution once the runcontrol is totally overhauled
           
        }
    }
    synchronized public boolean submitRunControlCommand()
    {
    	if (foregroundCommand != null)
        {
    		return false;
        }
        try
        {
            final String input = ((String) cb_runControl_input.getSelectedItem()).trim();
            cb_runControl_input.setSelectedItem ("");
            printMessage (input + "\n"); // TODO: command echo to text area?
            
            b_runControl_submit.setLabel ("Input");
            tf_runControl_input.setBackground(Color.lightGray);
            b_runControl_stop.setEnabled(true);
            commandIS.clearBuffer();
            
            this.printWarning("Running RunControl command...");
            
            //tf_runControl_input.setEditable (false);

            // TODO: is it necessary to make a new thread for each command?
            foregroundCommand = new GUICommandRunner (input, commandInputReader, commandOutputWriter);
            new Thread (foregroundCommand, "GUI Command Runner").start();
            
        }
        catch (Exception e) // TOOD: really bad
        {
            // TODO: this is a GUI - popup a window or show this in the text area
            System.out.println ("Exception: " + e.toString());
            return false;
        }
        return true;
    }

    /**
     * This class is used to display command output in the textarea. Using
     * PipedReader/PipeWriter would have required another thread. With
     * this implementation the command runner thread updates the GUI
     * directly/immediately.
     */
    class CommandOutputStream
        extends OutputStream
    {
        private StringBuffer buffer = new StringBuffer();

        public void flush()
            throws IOException
        {
            if (buffer == null)
                throw new IOException ("Attempted operation on a closed stream");

            super.flush();
            final String bufferString = buffer.toString();
            printMessage (bufferString);
            buffer.delete (0, buffer.length());
        }

        public void write (final int singleByte)
            throws IOException
        {
            if (buffer == null)
                throw new IOException ("Attempted operation on a closed stream");

            buffer.append ((char) singleByte);
        }

        public void close()
            throws IOException
        {
            buffer = null;
        }
    }
    /**
     * This class is used to provide input for the running command.
     */
    class CommandReader extends Reader{
    	
    	public void clearBuffer(){
    		buffer = "";
    	}
    	
		private synchronized char readChar() throws InterruptedException {
			char result;
			while(buffer.length()==0){
				wait();
				
			}
			result = (buffer.charAt(0));
			buffer = buffer.substring(1);
			return result;
		}
		
		private String buffer="";
		
		public synchronized void writeLine(String line){
			buffer += line;
			notifyAll();
		}

		/* (non-Javadoc)
		 * @see java.io.Reader#close()
		 */
		public void close() throws IOException {
			buffer = "";
		}

		/* (non-Javadoc)
		 * @see java.io.Reader#read(char[], int, int)
		 */
		public int read(char[] arg0, int arg1, int arg2) throws IOException {
			int count = 0;
			try{
				for(int i=0;i<arg2;i++){
					arg0[i+arg1] = readChar();
					count ++;
					if(buffer.length()==0){
						break;
					}
				}
			}catch(InterruptedException ie){
				return -1;
			}
			return count;
		}
    	
    }

    //this class run a RunControl command in background thread
    class GUICommandRunner
        implements Runnable
    {
        private String line;
        private PrintWriter output;
        private BufferedReader input;

        private Thread commandThread = null;

        public GUICommandRunner (final String input, final BufferedReader in, final PrintWriter output)
        {
            this.line = input;
            this.output = output;
            this.input = in;
        }

        public void interruptCommand()
        {
            if (commandThread != null)
            {
                commandThread.interrupt();
            }
        }

        public void run()
        {
            try
            {
                commandThread = Thread.currentThread();
                updateCommandHistory (line);
                runControl.processCommand (protocol, line, input, output, output);
                output.flush();
                printWarning("RunControl command finished.");
                
            }
            catch (final CommandFailureException cfe)
            {
                output.println (cfe.getMessage());
                printWarning("RunControl command failed.");
            }
            catch (final InterruptedException ie)
            {
                output.println ("Command interrupted");
                printWarning("RunControl command interrupted.");
            }
            catch (final Exception e) 
			// TODO: bad - using a "catch all" structure for regular control?
			// commented by Guangyu: we want to catch all possible unknow exception
			// and since this is the root of the running thread, I dont think it's
			// too bad to "catch all" here
            {
                if (e.getMessage() != null
                    &&
                    e.getMessage().toUpperCase().trim().equals ("QUIT"))
                {
                    quitRunControl();
                }

                output.println("Exception when running command: "+e.toString());
                e.printStackTrace (System.err);
                printWarning("RunControl command failed.");
            }
            finally
            {
                foregroundCommand = null;
                b_runControl_submit.setLabel ("Run");
                tf_runControl_input.setBackground(Color.white);
                b_runControl_stop.setEnabled(false);
                
                
                tf_runControl_input.setEditable (true);
                cb_runControl_input.requestFocus();
            }
        }

    }

    class AutoUpdater
        implements Runnable
    {
        private boolean exit = false;
        private boolean running = false;
        private boolean oneTimeRun;

        public AutoUpdater (final boolean oneTimeRun)
        {
            this.oneTimeRun = oneTimeRun;
        }

        synchronized public void run()
        {
            exit = false;
            running = true;
            while (! exit)
            {
                synchronized (gOverlay)
                {
                    try
                    {
                        doUpdateNodes();
                    }
                    catch (Exception e) // TODO: really bad - never throw or catch plain old Exception
                    {
                        System.err.println ("Exception while updating nodes: " + e.toString());
                    }
                }

                dw.repaint();

                if (oneTimeRun
                    ||
                    exit)
                {
                    break;
                }

                try { wait (simSleep); } catch (InterruptedException ignored) { }
            }
            running = false;
        }

        synchronized public void stop()
        {
            exit = true;
        }
    }

   

    /**
     *  Called when the simulation runs as an application.
     */
    public static void main (final String [] args)
    {
        String protocol = DEFAULT_PROTOCOL;
        String configurationFilename = DEFAULT_CONFIGFILENAME;

        
        RunControlGUI gui = new RunControlGUI ();
        gui.theApplicationWindow = new Frame ("RunControl GUI");

        gui.enableWindowListener(); // close button and initial component focus

        gui.theApplicationWindow.add (BorderLayout.CENTER, gui);
        gui.theApplicationWindow.setSize (winSize);

        gui.init(args);
        gui.start();

        Toolkit tk = gui.theApplicationWindow.getToolkit();
        gui.theApplicationWindow.setIconImage (gui.createIcon("hypercast_logo.gif").getImage());

        gui.theApplicationWindow.show();
    }
    
    
    private ImageIcon createIcon(String iconFileName)
	{
		URL url = this.getClass().getResource(iconFileName);
		return new ImageIcon(url);
	}
    
}
