/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

/* 
 * @author HyperCast Team
 */

// TODO Add comments 

package gui;

import java.awt.Color;
import java.util.Properties;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import hypercast.HyperCastConfig;
import hypercast.HyperCastStatsException;
import hypercast.I_LogicalAddress;
import hypercast.DT.DT_LogicalAddress;
import hypercast.DT.DT_Node_ServerJoin;
import hypercast.MonitorAndControl.MonMessage;
import hypercast.util.XmlUtil;
import hypercast.util.XmlUtil;

public class RunDTNode extends RunNode {
	DT_LogicalAddress logicalAddress;
	int state = 1;

	public RunDTNode(physicalInterface p, DT_LogicalAddress la, int pIndex, int sIndex) {
		super(p, pIndex, sIndex);
		setLogicalAddress(la);
	}
	public void setLogicalAddress(I_LogicalAddress la) {
		logicalAddress = (DT_LogicalAddress) la;
	}
	public I_LogicalAddress getLogicalAddress() {
		return logicalAddress;
	}

	public int getState() {
		return state;
	}
	public String getDetails() {
		return super.getDetails() + ":State=" + getState();
	}

	public Color getColor() {
		/*if( neighbors.isLeader() )
			return Color.orange;
		else*/
		if (!running)
			return Color.lightGray;

		if (status == RunNode.NEW)
			return Color.green; //new node
		else if (status == RunNode.DEAD || isOld())
			return Color.gray; //possibly dead node

		switch (getState()) {
			case DT_Node_ServerJoin.DEAD :
				return Color.lightGray;
			case DT_Node_ServerJoin.LEADER_STABLE :
			case DT_Node_ServerJoin.LEADER_INCOMPLETE :
			case DT_Node_ServerJoin.UNCONFIRMED_LEADER_STABLE :
				//case DT_Node_ServerJoin.UNCONFIRMED_LEADER_INCOMPLETE:
				return Color.orange;

		}

		return Color.yellow;

	}
	/* (non-Javadoc)
	 * @see RunNode#setLogicalAddress(java.lang.String)
	 */
	public void setLogicalAddress(String la) {
		this.logicalAddress = new DT_LogicalAddress(la);
	}
	
	public String [] createQuery(){
		String []common=super.createQuery();
		String []query=new String[common.length+1];
		for(int i=0;i<common.length;i++)
			query[i]=common[i];
		
		query[common.length]="/Node/State";
		
		return query;
	}
	public boolean readQueryResult(MonMessage reply, String prefix) throws NumberFormatException, HyperCastStatsException{
		if(super.readQueryResult(reply, prefix)){
			this.state=Integer.parseInt(XmlUtil.getScalarValue(reply.getStatValue(XmlUtil.createXPath(prefix+"/Node/State"))));
			return true;
		}
		else return false;
		
	}
	
	/* (non-Javadoc)
	 * @see RunNode#getCoords()
	 */
	public XYAddress getXYAddress() {

		//return new XYAddress((int) logicalAddress.getX(), 1800 - (int) logicalAddress.getY());
		return new XYAddress((int) logicalAddress.getX(), (int) logicalAddress.getY());
	}

}
