/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

/* 
 * @author HyperCast Team
 */

package gui;

import java.awt.Component;
import java.awt.Graphics;
import java.awt.Color;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Enumeration;
import java.util.Properties;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import hypercast.HyperCastConfig;
import hypercast.HyperCastStatsException;
import hypercast.IP_Address;
import hypercast.I_LogicalAddress;
import hypercast.MonitorAndControl.I_ReceiveCallback;
import hypercast.MonitorAndControl.I_Monitor;
import hypercast.MonitorAndControl.MonMessage;
import hypercast.util.XmlUtil;
import hypercast.util.XmlUtil;
import hypercast.MonitorAndControl.NoSuchPortalException;

import runcontrol.RunControl;
//import runcontrol.Experiment;

/**
 * This class builds an Overlay based on the data collected by RunControl
 */
public abstract class RunNetwork
    implements I_ReceiveCallback, Runnable
{
	public static final boolean debug = false;

	RunControl runControl;

	Element replySchema;

	public boolean showingFile = false;

	//the current overlayID
	public String overlayID;

	Vector olID_list; //store all the overlayIDs got from nodes

	//store the results getting from the portal
	Vector portalResults = new Vector();

	I_Monitor monitor;

	int update_interval = 2000;

	//for statistic purpose
	int questSent = 0;

	int replyReceived = 0;

	//last time the network got updating information
	long time_stamp = 0;

	private static final int SOCKETS_PER_QUERY = 10;

	public RunNetwork(final RunControl runControl) {
		sim_time = 0.0f;
		nodes = new Vector();
		active_nodes = new Vector();
		
		init(runControl);
	}

	private void init(final RunControl runControl) {
		this.runControl = runControl;
		this.monitor = runControl.getMonitor();
	}

	//for auto-update
	boolean running = false;

	boolean stopped = true;

	//need update the component while nodes is updated
	Component toUpdate;

	public void touch() {
		time_stamp = System.currentTimeMillis();
	}

	public void paint(Graphics g, int viewX, int viewY, float scale) {
		//do nothing
	}

	public synchronized void startAutoUpdate(int interval, Component to_update) {
		if (!running) {
			setUpdateInterval(interval);
			running = true;
			stopped = false;
			toUpdate = to_update;
			(new Thread(this)).start();
		}
	}

	public synchronized void stopAutoUpdate() {
		stopped = true;
		running = false;
	}

	//int active_portal_count=1;

	public void run() {
		//run auto update
		running = true;
		while (!stopped) {
			try {
				if (toUpdate != null) {
					toUpdate.repaint();
				}
				/*
				if (runControl.getExperiment() != null
						&& ((runControl.experimentMode == RunControl.MODE_RUNSERVER
								&& runControl.getExperiment().state != Experiment.STOPPED && runControl
								.getExperiment().state != Experiment.NEW) || (runControl.experimentMode == RunControl.MODE_APPLICATION && runControl
								.getExperiment().state == Experiment.RUNNING))) {
					sendUnblockedQuests();
				} else {
					if (!showingFile) {
						clearNodes();
					}
				}
				*/
				sendUnblockedQuests();
				
				Thread.sleep(update_interval);

			} catch (Exception e) {
				lotusDebug.errorln("Warning: Exception in RunNetwork.run(): " + e.toString());
				e.printStackTrace();
			}
		}
		running = false;
	}

	public void setUpdateInterval(int ui) {
		update_interval = ui;
	}

	//clean the nodes not related to any active portal
	synchronized void cleanUnknownNodes() {
		try {
			for (int i = 0; i < nodes.size(); i++) {
				OverlayNode onode = (OverlayNode) nodes.get(i);

				RunNode rnode = (RunNode) (onode.getProtocol());

				int portal = rnode.getPortalIndex();
				PortalResult pr = getPortalResult(portal);

				if (pr == null || portal > (monitor.getAllPortalIndices().length)
						|| !monitor.isPortalActive(portal) || pr.time_stamp > rnode.time_stamp
						|| (System.currentTimeMillis() //added by Guangyu, Feb
													   // 24th, 2004
						> (rnode.time_stamp + rnode.RUNNODE_TIMEOUT))) {

					//System.err.println("Clean dead node1");

					if (pr != null && !monitor.isPortalActive(portal))
						pr.alive = false;

					if (rnode.getStatus() == RunNode.DEAD) {
						//remove it
						onode.clearLogicalEdges();

						removeNode(onode);

						i--;
					} else {
						rnode.setStatus(RunNode.DEAD); //set as dead node

					}
				}
			}
		} catch (final NoSuchPortalException nspe) {
			System.err.println("NoSuchPortalException caught");
			nspe.printStackTrace();
		}
	}

	//clean the nodes belonging to a dead portal
	synchronized void cleanDeadNodes(int portal) {
		for (int i = 0; i < nodes.size(); i++) {
			OverlayNode onode = (OverlayNode) nodes.get(i);

			RunNode rnode = (RunNode) (onode.getProtocol());

			if (rnode.getPortalIndex() == portal) {
				if (rnode.getStatus() == RunNode.DEAD) {
					//remove it
					onode.clearLogicalEdges();

					removeNode(onode);

					i--;
				} else {
					rnode.setStatus(RunNode.DEAD); //set as dead node
					PortalResult pr = getPortalResult(portal);
					if (pr != null)
						pr.alive = false;
				}
			}
		}

	}

	//this is a implementation of a method of the XmlSocketCallback
	//receive Xml message when the RunNetwork works in the callback mode
	//message is replied to the quest sent by RunNetwork with
	// sendUnblockedQuest()
	synchronized public void receiveMessage(MonMessage msg) {
		replyReceived++;

		if (showingFile)
			return;

		if (portalResults == null)
			return;

		I_LogicalAddress portalLogicalAddress = msg.getSender();
		int portal = -1;
		try {
			portal = monitor.indexOfPortal(portalLogicalAddress);
		} catch (final NoSuchPortalException nspe) {
			System.err.println("Message received but cannot determine Portal index, sender: "
					+ portalLogicalAddress);
			return;
		}

		PortalResult pr = getPortalResult(portal);

		if (pr == null)
			return;

		long seqNum = msg.getMessageID();

		if (seqNum <= pr.seqNum && (pr.seqNum - seqNum) <= 100)
			return; //message older than the portal received last time
		pr.seqNum = seqNum;

		try {
			int msgType = readMessage(msg, portal, -1);
		} catch (NumberFormatException e) {
			e.printStackTrace();
		} catch (HyperCastStatsException e) {
			e.printStackTrace();
		}

		if (toUpdate != null)
			toUpdate.repaint();

	}

	/**
	 * find the closest node to a physical location. Used in GUI interface
	 */
	public OverlayNode findClosest(XYAddress addr, int radius) {
		int i;
		float dist = Float.MAX_VALUE;
		int index = -1;

		Vector cur_nodes = getPhyNodes();

		for (i = 0; i < cur_nodes.size(); i++) {
			OverlayNode onode = (OverlayNode) cur_nodes.elementAt(i);
			RunNode rnode = (RunNode) onode.getProtocol();

			if (dist > addr.distanceTo(onode.getAddress())) {
				//if(overlayID!=null &&
				// !overlayID.equals(rnode.getOverlayID()))continue;

				dist = addr.distanceTo(onode.getAddress());
				index = i;
			}
		}

		if (-1 == index)
			return null;
		if (dist > radius)
			return null;

		return (OverlayNode) cur_nodes.elementAt(index);

	}

	//return the list of the overlay id
	public String[] getOverlayIDList() {
		if (olID_list == null) {
			olID_list = new Vector();
		}

		String idList[] = new String[olID_list.size()];

		for (int i = 0; i < olID_list.size(); i++) {
			idList[i] = (String) olID_list.get(i);
		}
		return idList;
	}

	//this overlay id exists in the network?
	boolean existOverlayID(String id) {
		if (olID_list == null) {
			olID_list = new Vector();
		}
		for (int i = 0; i < olID_list.size(); i++) {
			String id0 = (String) olID_list.get(i);
			if (id0.equals(id))
				return true;
		}
		return false;
	}

	/** add overlay id to the list */
	public void addOverlayID(String id) {
		if (!existOverlayID(id))
			olID_list.add(id);
	}

	//current overlay id of the network. maybe null
	public String getOverlayID() {
		return overlayID;
	}

	public void setOverlayID(String olid) {
		overlayID = olid;
	}

	//set the address of node according to logial address
	public void setLogical() {
		for (int i = 0; i < nodes.size(); i++) {
			OverlayNode onode = (OverlayNode) nodes.elementAt(i);
			RunNode rnode = (RunNode) onode.getProtocol();
			onode.setAddress(rnode.getXYAddress());
		}
	}

	//set the address of node according to location
	public void setGeographic(Map map, float scale) {

		for (int i = 0; i < nodes.size(); i++) {
			OverlayNode onode = (OverlayNode) nodes.elementAt(i);
			RunNode rnode = (RunNode) onode.getProtocol();
			onode.setAddress(map.getPositionOnMap(rnode.getGeoAddress()));
		}
	}

	public void setRunControl(RunControl r) {
		runControl = r;
	}

	synchronized public void clearNodes() {

		nodes = new Vector();
		active_nodes = new Vector();
		portalResults = new Vector();
		LocationMapper.clearEntries();
	}

	synchronized protected void markNodes() {
		//delete all DEAD nodes
		//mark all other nodes as DEAD

		for (int i = 0; i < nodes.size(); i++) {
			OverlayNode onode = (OverlayNode) nodes.get(i);

			RunNode rnode = (RunNode) (onode.getProtocol());
			if (rnode.getStatus() == RunNode.DEAD) {
				//delte dead node
				onode.clearLogicalEdges();

				removeNode(onode);

				i--;
			} else
				rnode.setStatus(RunNode.DEAD); //set as dead node
		}
	}

	public OverlayNode findNode(String phyAddress) {
		phyAddress = phyAddress.trim();
		for (int i = 0; i < nodes.size(); i++) {
			OverlayNode onode = (OverlayNode) nodes.get(i);
			RunNode rnode = (RunNode) (onode.getProtocol());
			if ((rnode != null) && rnode.getPhysicalNetworkAddress() != null)
				if (rnode.getPhysicalNetworkAddress().equals(phyAddress))
					return onode;

		}
		return null; //not found
	}

	public OverlayNode findNode(int pIndex, int sIndex) {

		for (int i = 0; i < nodes.size(); i++) {
			OverlayNode onode = (OverlayNode) nodes.get(i);
			RunNode rnode = (RunNode) (onode.getProtocol());

			if (rnode != null && (rnode.getPortalIndex() == pIndex)
					&& (rnode.getSocketIndex() == sIndex))
				return onode;

		}
		return null; //not found
	}

	synchronized public boolean protocolReader(String filename) {
		try {
			FileInputStream input = new FileInputStream(filename);

			DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();

			DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();

			Document loadDoc = docBuilder.parse(input);

			updateNodesFromDoc(loadDoc);

			input.close();

		} catch (Exception e) {
			lotusDebug.println("error when loading the file:" + e.toString());
			return false;
		}
		return true;

	}

	synchronized public boolean protocolWriter(String filename) {
		try {
			Document saveDoc = mergeResults();

			FileOutputStream output = new FileOutputStream(filename);

			XmlUtil.writeXml(saveDoc, output);

			output.close();

		} catch (Exception e) {
			lotusDebug.println("error when writing the file");
			return false;
		}
		return true;
	}

	synchronized Document mergeResults() {

		if (portalResults == null)
			return null;

		int count = 0;
		for (int i = 0; i < portalResults.size(); i++) {
			PortalResult pr = getPortalResult(i);
			if (pr != null && pr.alive && pr.result != null)
				count++;
		}

		Document[] results = new Document[count];

		count = 0;
		for (int i = 0; i < portalResults.size(); i++) {
			PortalResult pr = getPortalResult(i);
			if (pr != null && pr.alive && pr.result != null) {
				results[count] = pr.result.getDocument();
				count++;
			}
		}
		return mergeResults(results);

	}

	synchronized Document mergeResults(Document[] results) {
		Document saveDoc;

		//merge all results in to a singel document for saving
		if (results == null)
			return null;

		saveDoc = XmlUtil.createDocument();

		Element rootElement = saveDoc.createElement("MonitorGUIData");
		saveDoc.appendChild(rootElement);

		Element res = saveDoc.createElement("Summary");

		rootElement.appendChild(res);

		for (int i = 0; i < results.length; i++) {
			Element portal = saveDoc.createElement("Portal");
			portal.setAttribute("index", "" + i);
			res.appendChild(portal);

			if (results[i] != null) {
				Node nd = results[i].getChildNodes().item(0);

				XmlUtil.changeNodeOwner(saveDoc, nd);

				portal.appendChild(nd);
			}

		}

		return saveDoc;
	}

	synchronized protected boolean updateNodesFromDoc(Document loadDoc) {

		showingFile = true;
		clearNodes();

		Node rs2Data = XmlUtil.findChildNode(loadDoc, "MonitorGUIData");

		if (rs2Data != null) {

			NodeList rl = rs2Data.getChildNodes();

			//read results
			int count = 0;
			for (int i = 0; i < rl.getLength(); i++) {
				if (rl.item(i).getNodeName().equals("Summary")) {

					NodeList sl = rl.item(i).getChildNodes();

					for (int j = 0; j < sl.getLength(); j++) {
						if (sl.item(j).getNodeName().equals("Portal")) {
							NodeList pl = sl.item(j).getChildNodes();
							Document portal = XmlUtil.createDocument();

							for (int k = 0; k < pl.getLength(); k++) {
								//TODO
								//need to change to use MonMessage
								if (pl.item(k).getNodeName().equals("ACK")) {
									Node ack = pl.item(k);

									XmlUtil.changeNodeOwner(portal, ack);
									portal.appendChild(ack);

									//lotusDebug.println("read portal "+count);
									count++;
									/* TODO */
									//readMessage(portal, count, 0);
									//resV.add(portal);
								}
							}
						}
					}

				}
			}

			//set status to normal
			for (int i = 0; i < nodes.size(); i++) {
				OverlayNode onode = (OverlayNode) nodes.get(i);

				RunNode rnode = (RunNode) onode.getProtocol();
				rnode.setStatus(RunNode.ACTIVE);

			}

			return true;

		} else
			return false;

	}

	long getPortalTimeStamp(int pIndex) {

		if (portalResults == null)
			return 0;

		PortalResult pr = (PortalResult) portalResults.get(pIndex);
		if (pr == null)
			return 0;

		return pr.time_stamp;

		/*
		 * long ts=0; for(int i=0;i <nodes.size();i++){ OverlayNode
		 * onode=(OverlayNode)nodes.get(i); RunNode
		 * rnode=(RunNode)(onode.getProtocol());
		 * 
		 * if((rnode.getPortalIndex()==pIndex) && rnode.time_stamp>ts)
		 * ts=rnode.time_stamp;
		 *  } return ts;
		 */
	}

	synchronized public boolean sendUnblockedQuests() {
		try {
			//lotusDebug.errorln("Start to update");

			//update the nodes information from Run Servers
			/*
			if (runControl.getExperiment() == null)
				return false;
				*/
			
			if (showingFile) {
				clearNodes();
				showingFile = false;
			}

			updatePortalResult(-1, null, monitor.getAllPortalIndices().length - 1);
			cleanUnknownNodes();

			// record the portal index of each message
			final int portalCount = monitor.getAllPortalIndices().length;

			/*
			 * Holds active portal indices followed by -1 entries
			 */
			final int[] activePortals = new int[portalCount];
			final int[] messagesPerPortal = new int[portalCount];

			int activePortalCount = 0;
			int messagesNeeded = 0; // TODO: is this needed?
			int activePortalIndex = 0;
			int inactivePortalIndex = portalCount - 1;
			for (int i = 0; i < activePortals.length; i++) {
				if (monitor.isPortalActive(i)) {
					++activePortalCount;
					activePortals[activePortalIndex] = i;
					/*
					try {
						runControl.updateExperiment(false);
					} catch (InterruptedException e) {
						//do nothing
					} catch (HyperCastStatsException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					*/
					/*
					messagesPerPortal[activePortalIndex] = runControl.getExperiment().nodesOnServers[i]
							/ SOCKETS_PER_QUERY
							+ ((runControl.getExperiment().nodesOnServers[i] % SOCKETS_PER_QUERY != 0) ? 1
									: 0);
									*/
					
					messagesPerPortal[activePortalIndex] = getPortalResult(i).numOfSocks / SOCKETS_PER_QUERY
							+ ((getPortalResult(i).numOfSocks % SOCKETS_PER_QUERY != 0)?1:0);
					
					if(messagesPerPortal[activePortalIndex]<=0)
						messagesPerPortal[activePortalIndex] = 1;

					messagesNeeded += messagesPerPortal[activePortalIndex];
					
					/*
					if (debug)
						System.err.println(messagesPerPortal[activePortalIndex]
								+ " messages needed for portal with "
								+ runControl.getExperiment().nodesOnServers[i] + " nodes ");
								*/

					++activePortalIndex;
				} else {
					activePortals[inactivePortalIndex--] = -1;

					cleanDeadNodes(i); // seems strange here, but ok...
				}
			}

			// create messages
			MonMessage[] msgs = createMessages(activePortals, messagesNeeded);

			if (debug)
				System.err.println("There are " + msgs.length + " messages");

			int pi = 0;
			for (int i = 0; i < msgs.length; ++i) {
				long ts = getPortalTimeStamp(activePortals[pi]);
				if ((System.currentTimeMillis() - ts) > update_interval) {
					if (debug)
						System.err.println("Sending message " + i + " to portal "
								+ activePortals[pi]);

					monitor.sendTo(activePortals[pi], msgs[i], this, null);
					questSent++;
					/*
					 * try{ Thread.sleep(update_interval/active_portal_count);
					 * }catch(Exception e){}
					 */
				}
				if (--messagesPerPortal[pi] <= 0)
					++pi;
			}
			return true;
		} catch (NoSuchPortalException TODO) {
			System.err.println("NoSuchPortalException caught");
			TODO.printStackTrace();
		}
		return true;
	}

	synchronized void updatePortalResult(int nOfSocks, MonMessage result, int portal) {
		if (portalResults == null)
			portalResults = new Vector();

		if (portal < 0)
			return;

		while (portal > (portalResults.size() - 1)) {
			portalResults.add(new PortalResult(0, null));
			//lotusDebug.errorln("Add a new Portal "+(portalResults.size()-1));
		}
		PortalResult pr = getPortalResult(portal);

		if (nOfSocks >= 0 && nOfSocks < pr.numOfSocks) {
			//lotusDebug.errorln("Number of Sockets drops at Portal "+portal+"
			// to "+nOfSocks);
			cleanDeadNodes(portal);
			cleanDeadNodes(portal);
		}

		if (nOfSocks >= 0 && nOfSocks > pr.numOfSocks) {
			//lotusDebug.errorln("Number of Sockets increases at Portal
			// "+portal+" to "+nOfSocks);
			//cleanDeadNodes(portal);
			//cleanDeadNodes(portal);
		}

		pr.updateResult(nOfSocks, result);

	}

	PortalResult getPortalResult(final int portal) {
		if (portalResults == null) // TODO: what does it mean if the
								   // portalResults are null?
		{
			return null;
		} else if (portalResults.size() < (portal + 1)) // TODO: what does it
														// mean if the requested
														// portal is out of
														// range?
		{
			return null;
		} else {
			return (PortalResult) portalResults.get(portal);
		}
	}

	synchronized protected int readMessage(final MonMessage result, final int portal,
			final int msg_type) throws NumberFormatException, HyperCastStatsException {
		// TODO: possible silent failure if pr == null
		PortalResult pr = getPortalResult(portal); // pr is never used in this
												   // method: why not?
		touch(); // TODO: is this supposed to be pr.touch()?

		boolean cleaned = false;
		int msgType = -1;

		if (result == null) {
			// TODO: should this be a serious error? what does null mean here?
			// did we block indefinitely on read? why is callback invoked if
			// there is no message?
			System.out.println("RunServer " + portal + " did not report.");
		} else {
			//lotusDebug.errorln("parse ACK");

			//Properties props = parseReplyMessage(result, "/Portal/Appl/NumOfSockets", "NumOfSockets", null);

			// TODO
			// No longer need to ask for RunningTable since
			// 1. it no longer exists
			// 2. information that was gained by it is now given by
			// SocketTableEntry
			
			int num = Integer.parseInt(XmlUtil.getScalarValue(result.getStatValue(XmlUtil.createXPath("/Portal/Appl/NumOfSockets"))));
			updatePortalResult(num, null, portal); // TODO: why is this not
												   // pr.update()?

			// This line caused a bug after splitting request messages into 10
			// socket chunks
			// Guangyu proposed that we comment this out.
			//cleanDeadNodes(portal);

			String query[] = this.createRunNodeInst().createQuery();

			// TODO: this seems inefficient, why not parse the whole thing at
			// once?
			for (int i = 1; i <= num; i++) {
				try {
					String prefix = "/Portal/Appl/Socket["+i+"]";

					String runningstr;
					
					if(result.getStatValue(XmlUtil.createXPath(prefix+"/Running")).length==0)
						continue;
					
					runningstr = XmlUtil.getScalarValue(result.getStatValue(XmlUtil.createXPath(prefix+"/Running")));
					
					boolean running = runningstr != null && runningstr.equals("true");
	
					updatePortalResult(-1, result, portal);
					
					if(running)
						addNode(result, prefix, portal, i, running);
					
				
				} catch (HyperCastStatsException e) {
					//do nothing
				}
			}
		}
		return msgType;
	}
	

	/**
	 * @param offset
	 *            Each portal has some number of nodes/socket that are thought
	 *            of as an array. This parameter is an index into that array. It
	 *            specifies the first socket to use in this query message.
	 * 
	 * @param length
	 *            The number of sockets to use from this portal. The indices of
	 *            the sockets that will be used are [offset, length-1]
	 *            inclusive.
	 */
	protected synchronized MonMessage createMessage(final int portalIndex, final int offset,
			final int length) {
		final MonMessage queryMessage = monitor.createGetRequest();
		queryMessage.addStat(XmlUtil.createXPath("/Portal/Appl/NumOfSockets"));

		// TODO
		// No longer need to ask for RunningTable since
		// 1. it no longer exists
		// 2. information that was gained by it is now given by SocketTableEntry

		//queryMessage.addStat ("/Portal/Appl/RunningTable");
		//queryMessage.addStat ("/Portal/Appl/SocketTable");

		final PortalResult pr = getPortalResult(portalIndex);
		for (int k = offset; k < length + offset; ++k) {
			/* socekt table index starts from 1 */
			String scktPrefix = "/Portal/Appl/Socket[" + (k+1) + "]";
			String query[] = createRunNodeInst().createQuery();
			for (int i = 0; i < query.length; i++) {
				String node_query = scktPrefix + query[i];
				queryMessage.addStat(XmlUtil.createXPath(node_query));
			}
		}
		return queryMessage;
	}

	synchronized protected MonMessage[] createMessages(int portals[], int messageCount) {
		MonMessage[] msgs = new MonMessage[messageCount];
		int m = 0;
		for (int i = 0; i < portals.length && portals[i] != -1; i++) {
			//int nodeCount = runControl.getExperiment().nodesOnServers[portals[i]];
			int nodeCount = getPortalResult(portals[i]).numOfSocks;

			int offset = 0;
			if(nodeCount<=0){
				msgs[m++] = createMessage(portals[i], 0, 0);
			}else{
				while (nodeCount > 0) {
			
					int nodes = Math.min(nodeCount, SOCKETS_PER_QUERY);
	
					if (debug)
						System.err.println("Calling createMessage with: " + offset + ", " + nodes);
	
					msgs[m++] = createMessage(portals[i], offset, nodes);
					nodeCount -= SOCKETS_PER_QUERY;
					offset += SOCKETS_PER_QUERY;
				}
			}
		}

		return msgs;
	}

	synchronized public Vector getPhyNodes() {
		//only return the nodes with the right overlay ID
		if (overlayID == null)
			return nodes;

		Vector current_nodes = new Vector();

		for (int i = 0; i < nodes.size(); i++) {
			OverlayNode onode = (OverlayNode) nodes.get(i);
			RunNode rnode = (RunNode) onode.getProtocol();
			if (getOverlayID().equals(rnode.getOverlayID()))
				current_nodes.add(onode);
		}

		return current_nodes;
	}
	
	int getRunningNodesCount(){
		
		int count = 0;
		for (int i = 0; i < this.active_nodes.size(); i++) {
			OverlayNode onode = (OverlayNode)active_nodes.get(i);
			RunNode rnode = (RunNode) onode.getProtocol();
			if (overlayID==null || getOverlayID().equals(rnode.getOverlayID())){
				if(rnode!=null && rnode.running)
					count++;
			}
				
		}
		
		return count;
	}

	public boolean removeNode(OverlayNode node) {
		//Thread.dumpStack();

		boolean result = false;
		if (nodes.contains(node) == false)
			result = false;
		else {
			reroute = true;
			Enumeration enumeration = node.getPhyNeighbor();
			while (enumeration.hasMoreElements()) {
				OverlayNode onode = (OverlayNode) enumeration.nextElement();
				onode.removePhyEdge(node);
			}
			nodes.remove(node);
			active_nodes.remove(node);
			result = true;
		}
		this.setLogical();
		return result;
	}

	/**
	 * add a node to the overlay network
	 */
	//synchronized public OverlayNode addNode(Element sock, int pIndex, int
	// sIndex, boolean running){
	synchronized public OverlayNode addNode(MonMessage reply, String prefix, int pIndex, int sIndex,
			boolean running) {
		RunNode rnode = this.createRunNodeInst(null, pIndex, sIndex);
		//rnode.setRunNetwork(this);

                try
                {
		//rnode.readQueryResult(sock);
		if (!rnode.readQueryResult(reply, prefix))
			return null;
		} catch (HyperCastStatsException e) {
			lotusDebug.errorln(e.toString());
			return null;
                }
		
		//OverlayNode onode=findNode(dtnode.getPhysicalNetworkAddress());
		OverlayNode onode = findNode(rnode.getLogicalAddress(), rnode.getOverlayID());
		//OverlayNode onode=this.findNode(pIndex, sIndex);

		if (onode != null) {
			//this node exists
			//dtnode=new RunDTNode(onode, la, pIndex, sIndex);
			rnode = (RunNode) onode.getProtocol();
			try {
				//rnode.readQueryResult(sock);
				rnode.readQueryResult(reply, prefix);
			} catch (HyperCastStatsException e1) {
				lotusDebug.errorln(e1.toString());
				return null;
			}

			if (rnode.running)
				rnode.setStatus(RunNode.ACTIVE); //normal node
			else if (running)
				rnode.setStatus(RunNode.NEW); //new node

			rnode.running = running;
			rnode.portalIndex = (pIndex);
			rnode.socketIndex = (sIndex);

			if (!running)
				onode.clearLogicalEdges();

			activateNode(onode, rnode);

			rnode.touch();

			//if(!isValidNode(dtnode))dtnode.setStatus(RunNode.INVALID);
			setLogical();

			return onode;
		}

		I_LogicalAddress la = rnode.getLogicalAddress();
		//Guangyu 12-04-03
		//onode=addNode(new XYAddress((int)la.getX(), (int)la.getY()),
		// dtnode.getOverlayID(), dtnode.getPhysicalNetworkAddress());
		onode = addNode(rnode.getXYAddress(), rnode.getOverlayID(), rnode
				.getPhysicalNetworkAddress());
		onode.setName("(" + la + ")");

		rnode.setPhysical(onode);
		rnode.setRunNetwork(this);

		rnode.setStatus(RunNode.NEW); //new node
		rnode.running = running;

		activateNode(onode, rnode);

		try {
			rnode.readQueryResult(reply, prefix);
		} catch (HyperCastStatsException e1) {
			lotusDebug.errorln(e1.toString());
			return null;
		}

		//if(!isValidNode(dtnode))dtnode.setStatus(RunNode.INVALID);

		setLogical();
		return onode;
	}

	/**
	 * add a physical node with a specified address
	 * 
	 * @param addr
	 *            the physical address of the node
	 * @param olid
	 *            the overlay id of the node
	 */
	synchronized public OverlayNode addNode(XYAddress addr, String olid, String ip) {
		reroute = true;
		Enumeration e = nodes.elements();
		OverlayNode onode;
		boolean unique = true;

		if (olid == null)
			olid = "Null";

		while (e.hasMoreElements()) {
			onode = (OverlayNode) e.nextElement();
			RunNode rnode = (RunNode) onode.getProtocol();
			/**
			 * if (onode.getAddress().equals(addr) &&
			 * rnode.getOverlayID().equals(olid)) { lotusDebug.errorln(
			 * "Warning: non unique node insertion detected between " +
			 * rnode.getPhysicalNetworkAddress() + " and " + ip);
			 * lotusDebug.errorln( " non unique address and OL Id:" +
			 * addr.getX() + ":" + addr.getY() + ":" + rnode.getOverlayID());
			 * unique = false; break; }
			 */
		}

		onode = new OverlayNode(addr, this, null); //there must be a name??
		nodes.addElement(onode);
		return onode;

	}

	public OverlayNode findNode(I_LogicalAddress addr, String olid) {
		if(addr==null)
			return null;
		
		for (int i = 0; i < nodes.size(); i++) {
			OverlayNode onode = (OverlayNode) nodes.get(i);
			RunNode rnode = (RunNode) (onode.getProtocol());

			if (rnode == null)
				continue;

			I_LogicalAddress da = rnode.getLogicalAddress();
			
			if(da==null)continue;

			if (da.equals(addr) && olid.equals(rnode.getOverlayID()))
				return onode;

		}
		return null; //not found
	}

	abstract public I_LogicalAddress parseLogicalAddress(String la);

	public boolean activateNode(OverlayNode o, RunNode runnode) {

		addOverlayID(runnode.getOverlayID());

		o.setProtocol(runnode);

		if (active_nodes.contains(o)) {

			return false;
		}

		active_nodes.addElement(o);
		return true;
	}

	public RunNode createRunNodeInst() {
		return createRunNodeInst(null, -1, -1);
	};

	public abstract RunNode createRunNodeInst(physicalInterface p, int pIndex, int sIndex);

	// five old abstract methods
	public boolean fireMenu(OverlayNode onode, int index) {
		switch (index) {
		case 0:
			activateNode(onode, createRunNodeInst());
			break;

		}
		return true;
	}

	boolean start() {
		return true;
	}

	void reset() {
		clearNodes();
		try {
			runControl.getSocketStatusTable().clearPortalList();
			//monitor.clearPortalList();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	boolean buildOverlay() {
		return true;
	}

	public String[] getMenus() {
		String menus[] = { "Statistic Information" };
		return menus;
	}

	class PortalResult {
		int numOfSocks;

		MonMessage result;

		long time_stamp;

		boolean alive;

		long seqNum;

		//int error = 0;

		PortalResult(int ns, MonMessage rslt) {
			numOfSocks = ns;
			result = rslt;
			touch();
			time_stamp = 0;
		}

		void touch() {
			time_stamp = System.currentTimeMillis();
			alive = true;
			//error=0;
		}

		void updateResult(int ns, MonMessage rslt) {
			if (ns >= 0)
				numOfSocks = ns;
			if (rslt != null)
				result = rslt;

		}
	}

	// old overlay network stuff
	//the protocol menu and instance creation part
	final static String[] theMobilemode = { "Random Motion", "Linear Motion", };

	/** @return current supported protocols */
	static String[] getMobilemode() {

		//Thread.dumpStack();

		return theMobilemode;
	}

	//the protocol menu and instance creation part
	final static String[] theProtocols = { "DT-precise", };

	/** @return current supported protocols */
	static String[] getProtocols() {

		//Thread.dumpStack();

		return theProtocols;
	}

	/**
	 * Create a simulation network
	 */
	//Why do we need such a method to create a instance??
	static RunNetwork createInst(int index) {
		//Thread.dumpStack();

		switch (index) {
		//case 0: return new simDTPrecise();
		//should return a Run node ??
		case 0:
			return new RunDTNetwork(null);
		}
		return null;
	}
	
	
	//start the simulation: start the timers of each node and sending the Iam
	// leader to
	//the server
	boolean simRunning = false;

	/**
	 * @return whether the simulation is running
	 */
	boolean isRunning() {
		//Thread.dumpStack();

		return simRunning;
	}

	
	
	/**
	 * An abstract function, which must be implemented by each overlay protocol.
	 * 
	 * @param src
	 *            the source node
	 * @param dst
	 *            the destination node
	 * @return the next hope node in the logical network
	 */
	public abstract OverlayNode getLogicalNextHop(OverlayNode src, OverlayNode dst);

	/**
	 * remove an edge
	 * 
	 * @param from
	 *            the starting node
	 * @param to
	 *            the ending node
	 */
	public boolean removeEdge(OverlayNode from, OverlayNode to) {
		//Thread.dumpStack();

		if (from.hasPhyEdge(to) == false || to.hasPhyEdge(from) == false)
			return false;
		from.removePhyEdge(to);
		to.removePhyEdge(from);
		reroute = true;
		return true;
	}

	/**
	 * add a physical node with a specified address
	 * 
	 * @param addr
	 *            the physical address of the node
	 */
	public OverlayNode addNode(final XYAddress addr) {
		//Thread.dumpStack();

		reroute = true;
		Enumeration e = nodes.elements();
		OverlayNode onode;
		boolean unique = true;

		while (e.hasMoreElements()) {
			onode = (OverlayNode) e.nextElement();
			if (onode.getAddress().equals(addr)) {
				lotusDebug.errorln(" non unique node insertion detected");
				lotusDebug.errorln(" non unique address:" + addr.getX() + ":" + addr.getY());
				unique = false;
				break;
			}
		}

		onode = new OverlayNode(addr, this, null); //there must be a name??
		nodes.addElement(onode);
		return onode;

	}

	

	/**
	 * @return the vector containing the nodes in the overlay network
	 */
	public Vector getActiveNodes() {
		//Thread.dumpStack();

		return active_nodes;
	}

	/**
	 * add an physical edge. Note, edges are always bi-directional
	 * 
	 * @param from
	 *            starting node
	 * @param to
	 *            ending node
	 * @param distance
	 *            the lenght of the edge
	 */
	public boolean addEdge(OverlayNode from, OverlayNode to, float distance) {
		//add edges in a symmetrically way, i.e. always add two edges

		//Thread.dumpStack();

		if (from.hasPhyEdge(to))
			return false;

		reroute = true;
		from.insertPhyEdge(to, distance);
		to.insertPhyEdge(from, distance);
		return true;
	}

	final static Color phyNodeColor = Color.black;

	final static Color phyEdgeColor = Color.black;

		/** a vector that holds all the nodes in the network */
	Vector nodes; // OverlayNode

	/** a vector that holds all the nodes in the overlay network */
	Vector active_nodes;

	/** indicate whether a re-calculation of routing table is needed */
	boolean reroute = true;

	public float sim_time;
}
