/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

/* 
 * @author HyperCast Team
 */

// TODO No comments in this file

package gui;

/**************************************************************************
    Graphics RunControl interface      
**************************************************************************/
import java.awt.Color;
import java.util.Properties;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import hypercast.HyperCastConfig;
import hypercast.HyperCastStatsException;
import hypercast.I_LogicalAddress;
import hypercast.MonitorAndControl.MonMessage;
import hypercast.util.XmlUtil;
import hypercast.util.XmlUtil;

/**
 * This class implements a Run overlay node
 */
public abstract class RunNode implements protocolInterface
{
 	static final String IP_PHYSICAL_ADDRESS_SPLITTER=":";

	//the portal entry owning this node
	public int portalIndex;
	public int socketIndex;
	//public boolean alive;
	public RunNetwork network;
	public String overlayID;

	public physicalInterface phy;

	public static int ACTIVE = 0;
	public static int DEAD = 1;
	public static int NEW = 2;

	public boolean running;

	public int status = 0; //active dead or new 

	String phyAddress; //the network address of the node
	String hostName;
	public int neighborCount = -1;

	long time_stamp;
	int RUNNODE_TIMEOUT = 10000;

	Color edgeColor = Color.yellow;
	int edgeWidth = 2;

	public RunNode(physicalInterface p, int pIndex, int sIndex) {
		phy = p;
		portalIndex = pIndex;
		socketIndex = sIndex;
		touch();
	}
	abstract public XYAddress getXYAddress();
	public void touch() {
		time_stamp = System.currentTimeMillis();
	}
	public boolean isOld() {
		if (network != null)
			return (network.time_stamp - time_stamp) > RUNNODE_TIMEOUT;
		//network.update_interval*2;
		else
			return (System.currentTimeMillis() - time_stamp) > RUNNODE_TIMEOUT;
	}
	public boolean isTooOld() {
		if (network != null)
			return (network.time_stamp - time_stamp) > RUNNODE_TIMEOUT * 2;
		//network.update_interval*4;
		else
			return (System.currentTimeMillis() - time_stamp) > RUNNODE_TIMEOUT * 2;
	}

	public boolean matchSearch(String searchStr) {
		return getDetails().toUpperCase().indexOf(searchStr.trim().toUpperCase()) >= 0;
	}

	public String getOverlayID() {
		if (overlayID == null)
			return "Null";
		return overlayID;
	}

	public void setOverlayID(String olid) {
		overlayID = olid;
	}

	public GeoAddress getGeoAddress() {
		return LocationMapper.getGeoAddress(phyAddress);
	}

	//status= {0,1,2}
	public int getStatus() {
		return status;
	}

	public void setStatus(int s) {
		status = s;
	}

	public void setRunNetwork(RunNetwork rn) {
		network = rn;
	}

	public int getPortalIndex() {
		return portalIndex;
	}
	public int getSocketIndex() {
		return socketIndex;
	}

	public void setPhysical(physicalInterface p) {
		phy = p;
	}

	public String getHostName() {
		if (hostName != null)
			return hostName;

		//get host name from the dns cache
 		int i = phyAddress.indexOf (IP_PHYSICAL_ADDRESS_SPLITTER);
		String hn = DNSCache.getHostName(phyAddress.substring(0, i));
		if (hn != null)
			return hn;
		else
			return "Unknown Host";

	}

	public String getPhysicalNetworkAddress() {
		return phyAddress;
	}
	public void setPhysicalNetworkAddress(String pa) {
		phyAddress = pa.trim();
	}

	protected Node findChildNode(Node node, String name) {
		for (int i = 0; i < node.getChildNodes().getLength(); i++) {
			if (node.getChildNodes().item(i).getNodeName().equals(name))
				return node.getChildNodes().item(i);
		}
		return null;
	}

	public Color getColor() {
		/*if( neighbors.isLeader() )
		    return Color.orange;
		else*/
		if (!running)
			return Color.lightGray;

		if (status == RunNode.NEW)
			return Color.green; //new node
		else if (status == RunNode.DEAD)
			return Color.gray; //dead node
		else
			return Color.orange;

	}

	abstract public I_LogicalAddress getLogicalAddress();
	abstract public void setLogicalAddress(I_LogicalAddress la);
	abstract public void setLogicalAddress(String la);

	public String getDetails() {
		RunNode rnode = this;
		GeoAddress gaddr = rnode.getGeoAddress();

		return rnode.getPortalIndex()
			+ ","
			+ rnode.getSocketIndex()
			+ ":"
			+ rnode.getOverlayID()
			+ ":"
			+ rnode.getLogicalAddress()
			+ ":"
			+ rnode.getPhysicalNetworkAddress()
			+ ":"
			+ rnode.getHostName()
			+ ":"
			+ gaddr.toString();
	}

	public boolean equals(RunNode rnode) {
		try {

			if (!this.getLogicalAddress().equals(rnode.getLogicalAddress()))
				return false;

			//if(portalIndex!=dtnode.getPortalIndex())return false;
			//if(socketIndex!=dtnode.getSocketIndex())return false;

			if (!getOverlayID().equals(rnode.getOverlayID()))
				return false;

			return true;

		} catch (Exception e) {
			return false;
		}
	}

	public int getMsgType(Element Sock) {

		Node Config = XmlUtil.findChildNode(Sock, "Config");

		if (Config != null)
			return 0; //query
		else
			return 1; //detail query
	}

	
	/** Create a list of queries for the matching overlay socket 
	 * The default statistics created in RunNode class are found in the code of this method.
	 * User can override the whole list or just add new statistics to it
	*/

    public String [] createQuery()
    {
        String [] query = new String []
            {
                "/Running",
                "/Node/LogicalAddress",
                "/Node/PhysicalAddress",
                "/Config/Public/OverlayID",
                "/Node/NeighborTable",
            };
		
        return query;
    }

	/** Read the query results which are stored in a replying MonMessage 
	 * This method has to be overridden if the createQuery has been overridden
	 * to interpret reply for the newly added queries.
	 * 
	 * This function read the stats from the reply message by XPath expression
	 * to update the properties of this node
     *
     * @return true if subclasses should continue to read the reply
	 * @throws HyperCastStatsException
	 */
    public boolean readQueryResult(MonMessage reply, String prefix) throws HyperCastStatsException{
    	String la= XmlUtil.getScalarValue(reply.getStatValue(XmlUtil.createXPath(prefix+"/Node/LogicalAddress")));
    	setLogicalAddress(la);
    	
    	String pa= XmlUtil.getScalarValue(reply.getStatValue(XmlUtil.createXPath(prefix+"/Node/PhysicalAddress")));
    	setPhysicalNetworkAddress(pa);
    	
    	setOverlayID( XmlUtil.getScalarValue(reply.getStatValue(XmlUtil.createXPath(prefix+"/Config/Public/OverlayID"))) );
    	
    	if(network==null || phy==null)return true;
		
		if(phy!=null)((OverlayNode)phy).clearLogicalEdges();
    	
		Element [] neighbors = reply.getStatValue(XmlUtil.createXPath(prefix+"/Node/NeighborTable"));
		
		for(int i=0;i<neighbors.length;i++){
			
			Element neighborElement = neighbors[i];
			
			Element paElmt = (Element)neighborElement.getElementsByTagName("PhysicalAddress").item(0);
			String pAddress = null;
			if(paElmt!=null)
				pAddress = XmlUtil.getScalarValue(paElmt);
			
			Element laElmt = (Element)neighborElement.getElementsByTagName("LogicalAddress").item(0);
			String lAddress = null;
			if(laElmt!=null)
				lAddress = XmlUtil.getScalarValue(laElmt);
			
			if(pAddress!=null && pAddress.trim().equals(""))
				pAddress=null;
			if(lAddress!=null && lAddress.trim().equals(""))
				lAddress=null;
			
			if(pAddress==null && lAddress==null)continue;
			
			OverlayNode onode = null;

			if (pAddress != null)
				onode = network.findNode(pAddress);

			if (onode == null && lAddress != null)
				onode = network.findNode(network.parseLogicalAddress(lAddress), getOverlayID());

			if (onode != null) {
					
				phy.removeLogicalEdge(onode);
				phy.insertLogicalEdge(onode, edgeColor, edgeWidth);
			} else {
				//lotusDebug.errorln("neighbor not found!");	
			}
		}
    	
    	return true;
    }
    
	public int getShape() {
		return protocolInterface.shapeCircle;
	}

		
}
