/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

/*
 * @author HyperCast Team
 * @author Greg Mattes
 * @author gd4j
 * Created on May 7, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */

package gui;

import java.awt.Color;
import java.awt.Graphics;

import java.util.Properties;

import runcontrol.RunControl;

import hypercast.HyperCastStatsException;
import hypercast.I_Address;
import hypercast.I_LogicalAddress;
import hypercast.MonitorAndControl.MonMessage;


public class RunPastryNetwork extends RunNetwork {
	/**
		 * @param rc2inst
		 */
	public RunPastryNetwork(RunControl rc2inst) {
		super(rc2inst);

	}

	public void paint(Graphics g, int viewX, int viewY, float scale) {
		//do nothing
		g.setColor(Color.black);
		g.drawOval(
			(int) ((50 - viewX) * scale),
			(int) ((50 - viewY) * scale),
			(int) (300 * scale),
			(int) (300 * scale));
	}

	private Pastry_LogicalAddress maxLA;
	public boolean removeNode(OverlayNode node) {
		maxLA = null;
		boolean result = super.removeNode(node);
		this.setLogical();
		return result;
	}
	
	public int getMaxID() {
		Pastry_LogicalAddress max = this.getMaxLogicalAddress();
		int maxn;
		if (max != null)
			maxn = Integer.parseInt(max.toString());
		else
			maxn = 1;

		int result = 1;
		while (result <= maxn) {
			result *= 2;
		}
		return result - 1;
	}

	public Pastry_LogicalAddress getMaxLogicalAddress() {
		if (maxLA == null) {

			for (int i = 0; i < active_nodes.size(); i++) {
				OverlayNode onode = (OverlayNode) active_nodes.get(i);
				RunPastryNode hcnode = (RunPastryNode) onode.getProtocol();
				Pastry_LogicalAddress la = (Pastry_LogicalAddress) hcnode.getLogicalAddress();
				if ((maxLA == null || la.greaterThan(maxLA)) && !la.toString().equals("Unknown")) {
					maxLA = la;
				}
			}
		}
		return maxLA;
	}
	synchronized public OverlayNode addNode(
		MonMessage reply,
		String prefix,
		int pIndex,
		int sIndex,
		boolean running) {

		RunNode rnode = this.createRunNodeInst(null, pIndex, sIndex);
		try {
			if (rnode.readQueryResult(reply, prefix))
			        {
			            Pastry_LogicalAddress hcla = (Pastry_LogicalAddress) rnode.getLogicalAddress();
			            if (hcla.toString().equals("Unknown"))
				return null;

			            OverlayNode onode = super.addNode(reply, prefix, pIndex, sIndex, running);
			            maxLA = null;

			            this.setLogical();
			            return onode;
			        }
			        else
			        {
			            return null;
			        }
		} catch (HyperCastStatsException e) {
			lotusDebug.errorln(e.toString());
			return null;
		}
	}

	/* (non-Javadoc)
	 * @see RunNetwork#parseLogicalAddress(java.lang.String)
	 */

	public hypercast.I_LogicalAddress parseLogicalAddress(String la) {
            //lotusDebug.println("parseLogicalAddress: " + la);
		return new Pastry_LogicalAddress(la);
	}

	/* (non-Javadoc)
	 * @see RunNetwork#createRunNodeInst(physicalInterface, int, int)
	 */
	public RunNode createRunNodeInst(physicalInterface p, int pIndex, int sIndex) {
		return new RunPastryNode(p, null, pIndex, sIndex, this);
	}

	/* (non-Javadoc)
	 * @see OverlayNetwork#getLogicalNextHop(OverlayNode, OverlayNode)
	 */
	public OverlayNode getLogicalNextHop(OverlayNode src, OverlayNode dst) {
		// TODO Auto-generated method stub
		return null;
	}

}

// *** PASTRY *** - comment this class; it is a place holder for the "real" Pastry_LogicalAddress

/**
 *  A logical address for the Pastry protocol. This class implements
 *  the object adapter pattern, delegating Pastry_LogicalAddress
 *  method calls to an instance of NodeId from Pastry.
 * 
 *  @author Greg Mattes
 *  @version May 2004 
 */

final class Pastry_LogicalAddress implements hypercast.I_LogicalAddress
{
    
    String laStr;
    public Pastry_LogicalAddress (String nodeId)
    {
        laStr=nodeId;
    }

    public boolean equals (Object object)
    {
        if (! (object instanceof Pastry_LogicalAddress))
        {
            throw new IllegalArgumentException ("Object is not of type Pastry_LogicalAddress");
        }
        if(object==null)
        	return false;
        
        Pastry_LogicalAddress other = (Pastry_LogicalAddress) object;
        return this.toString().equals(object.toString());
    }

    
    public String toString ()
    {
        return laStr;
    }

	/* (non-Javadoc)
	 * @see hypercast.I_Address#greaterThan(hypercast.I_Address)
	 */
	public boolean greaterThan(I_Address a) {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see hypercast.I_Address#toByteArray()
	 */
	public byte[] toByteArray() {
		// TODO Auto-generated method stub
		return null;
	}

	/* (non-Javadoc)
	 * @see hypercast.I_Address#getSize()
	 */
	public int getSize() {
		// TODO Auto-generated method stub
		return 0;
	}

	

    
}


