/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

/* 
 * @author HyperCast Team
 * @author Greg Mattes
 * @author gd4j
 * Created on May 7, 2004
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */

package gui;

import java.awt.Color;

import java.util.Properties;

import hypercast.HyperCastConfig;
import hypercast.HyperCastStatsException;
import hypercast.I_LogicalAddress;
import hypercast.MonitorAndControl.MonMessage;
import hypercast.util.XmlUtil;

public class RunPastryNode extends RunNode {
	Pastry_LogicalAddress logicalAddress;
	XYAddress coords;
	RunPastryNetwork network;
	
	/** Synchronize with hypercast.Pastry.Pastry_Node_BuddyList.java */
	public static final int STATE_STOPPED        = 0; // after init or call to leave group
    public static final int STATE_BUDDY_SEARCH   = 1; // after call to join group
    public static final int STATE_BUDDY_FOUND    = 2; // after NODE_ID_RESPONSE received from buddy
    public static final int STATE_JOIN_INITIATED = 3; // after call to startNewPastryRing or joinEstablishedPastryRing
    public static final int STATE_JOINED         = 4; // after call to nodeIsReady (from the innards of the Pastry code)
    
    public static String getStringOfState (int x)
    {
        switch (x)
        {
            case STATE_STOPPED:        return "STATE_STOPPED";
            case STATE_BUDDY_SEARCH:   return "STATE_BUDDY_SEARCH";
            case STATE_BUDDY_FOUND:    return "STATE_BUDDY_FOUND";
            case STATE_JOIN_INITIATED: return "STATE_JOIN_INITIATED";
            case STATE_JOINED:         return "STATE_JOINED";
            default: return "UNKNOWN STATE " + x;
        }
    }
    
    int state = STATE_STOPPED;

	/**
	 * @param p
	 * @param pIndex
	 * @param sIndex
	 */
	public RunPastryNode(
		physicalInterface p,
		Pastry_LogicalAddress la,
		int pIndex,
		int sIndex,
		RunPastryNetwork net) {
		super(p, pIndex, sIndex);
		logicalAddress = la;
		network = net;
		//coords = new XYAddress(200, 200);
	}

	/* (non-Javadoc)
	 * @see RunNode#getLogicalAddress()
	 */
	public I_LogicalAddress getLogicalAddress() {
		return logicalAddress;
	}

	/* (non-Javadoc)
	 * @see RunNode#setLogicalAddress(edu.virginia.cs.mng.hypercast.I_LogicalAddress)
	 */
	public void setLogicalAddress(I_LogicalAddress la) {
		logicalAddress = (Pastry_LogicalAddress) la;
	}

	/* (non-Javadoc)
	 * @see RunNode#setLogicalAddress(java.lang.String)
	 */
	public void setLogicalAddress(String la) {
		if (!la.equals("Unknown"))
			logicalAddress = new Pastry_LogicalAddress(la);
		else
			logicalAddress = null;
	}

    public int getState ()
    {
        return state;
    }

    public String getDetails ()
    {
        return super.getDetails()
            + ":State="
            + getStringOfState(getState());
    }

    public Color getColor ()
    {
        Color color = null;
        if (!running)
        {
            color = Color.lightGray;
        }
        else if (status == RunNode.NEW)
        {
            color = Color.green;
        }
        else if (status == RunNode.DEAD || isOld())
        {
             //possibly dead node
            color = Color.gray;
        }
        else
        {
            switch (getState())
            {
                case STATE_STOPPED:
                    color = Color.lightGray;
                    break;
                case STATE_BUDDY_SEARCH:
                    color = Color.red;
                    break;
                case STATE_BUDDY_FOUND:
                    color = Color.pink;
                    break;
                case STATE_JOIN_INITIATED:
                    color = Color.orange;
                    break;
                case STATE_JOINED:
                    color = Color.yellow;
                    break;
                default:
                    // TODO: error?
            }
	}
        return color;
    }

    public String [] createQuery()
    {
        String []common=super.createQuery();
        String []query=new String[common.length+1];
        for(int i=0;i<common.length;i++)
            query[i]=common[i];
		
        query[common.length]="/Node/State";
		
        return query;
    }

    public boolean readQueryResult (MonMessage reply, String prefix) throws NumberFormatException, HyperCastStatsException
    {
        if (super.readQueryResult (reply, prefix))
        {
            this.state = Integer.parseInt (XmlUtil.getScalarValue(reply.getStatValue(XmlUtil.createXPath(prefix+"/Node/State"))));
            return true;
        }
        else return false;
    }

    /* (non-Javadoc)
     * @see RunNode#getCoords()
     */
    public XYAddress getXYAddress ()
    {
        //if (Pastry_LogicalAddress.getSizeStatic () < 4) throw new RuntimeException ("Pastry Address has less than 4 hex digits");

        long max4HexDigits = 0xFFFFL;

        long highOrder4HexDigits;

        String entireLogicalAddressString = null;
        if (logicalAddress != null && ! logicalAddress.toString ().equals ("Unknown"))
        {
            entireLogicalAddressString = logicalAddress.toString ();
            String highOrder4HexDigitsString = entireLogicalAddressString.substring (3, 7);
            highOrder4HexDigits = Integer.parseInt (highOrder4HexDigitsString, 16);

            //lotusDebug.println ("entireLogicalAddressString: " + entireLogicalAddressString);
            //lotusDebug.println ("highOrder4HexDigitsString: " + highOrder4HexDigitsString);
            //lotusDebug.println ("highOrder4HexDigits: " + highOrder4HexDigits);
        }
        else
        {
            return new XYAddress (200, 200);
        }

        // all units radians
        double circleFractionThisNode = ((double) highOrder4HexDigits) / ((double) max4HexDigits); 
        double fullCircle = 2.0 * Math.PI;
        double oneQuarterCircle = Math.PI / 2;
        // correct one quarter circle so that 0 is at the top of the screen
        double angle = fullCircle * circleFractionThisNode - oneQuarterCircle;

        double x = 150 * Math.cos (angle);
        double y = 150 * Math.sin (angle);

        if (phy != null
            &&
            entireLogicalAddressString != null)
        {
            ((OverlayNode) phy).setName (logicalAddress.toString ());
        }

        return new XYAddress ((int) (200 + x), (int) (200 + y));
    }
}
