/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

/* 
 * @author HyperCast Team
 */

// TODO Add comments 

package gui;

/**************************************************************************
    Graphics RunControl interface      
**************************************************************************/
import hypercast.*;
import hypercast.MonitorAndControl.*;

import java.net.*;

import java.awt.*;
import java.util.*;
import java.io.*;

import org.w3c.dom.*;

/**
 * This class implements a Tree style overlay node
 */
public  abstract class  RunTreeNode extends RunNode
{
	I_LogicalAddress parentAddress;
	RunNetwork network;
	
    /**
     * @param p TODO
     * @param pIndex TODO
     * @param sIndex TODO
     * @param net TODO
     */
	public RunTreeNode(physicalInterface p, int pIndex, int sIndex, RunNetwork net)
    {
        super(p, pIndex, sIndex);
        network=net;
        parentAddress=null;
    }

   	public void setParent(I_LogicalAddress p){
   		parentAddress=p;
   	}
    public RunTreeNode getParent(){
    	if(parentAddress==null || parentAddress.equals(getLogicalAddress()))return null;
    	
    	OverlayNode onode=network.findNode(parentAddress, this.getOverlayID());
    	if(onode==null)return null;
    	RunTreeNode rtnode=(RunTreeNode)onode.getProtocol();
    	
    	if(rtnode.running)return rtnode;
    	else
    		return null;
    }
    public boolean isRoot(){
    	return getParent()==null;	
    }
    private int getRootIndex(){
    	//if(!isRoot())return -1;
    	{
    		int index=0;
    		for(int i=0;i<network.nodes.size();i++){
    			OverlayNode onode=(OverlayNode)network.nodes.get(i);
				RunTreeNode tn=(RunTreeNode)onode.getProtocol();
    			if(tn==this)return index;
    			if(tn.running && tn.isRoot())index++;
    		}
    		return index; //??
    	}
    }
    private int indexOfChild(RunTreeNode p){
    	if(p.getParent()!=this)return -1;
    	else{
	    	int index=0;
    		for(int i=0;i<network.nodes.size();i++){
				OverlayNode onode=(OverlayNode)network.nodes.get(i);
				RunTreeNode tn=(RunTreeNode)onode.getProtocol();
	    		if(tn==p)return index;
	    		if(tn.running && tn.getParent()==this)index++;
	    	}
	    	return -1;
    	}
    }
    public int getNumberOfChildren(){
    	int count=0;
    	for(int i=0;i<network.nodes.size();i++){
			OverlayNode onode=(OverlayNode)network.nodes.get(i);
			RunTreeNode tn=(RunTreeNode)onode.getProtocol();
	    	if(tn.running && tn.getParent()==this)count++;
	    }
	    return count;
    }
    public XYAddress getXYAddress(){
    	
    	if(!this.running)return new XYAddress(100,100);
    	
    	return getXYAddress(0);	
    }
    private XYAddress getXYAddress(int depth)  { //depth is used to prevent infinite recursion
    	
    	if(isRoot() || depth>200){
    		
    		int rIndex=getRootIndex();
    		return new XYAddress(100*(rIndex+1)+200, 100);
    	}else {
			RunTreeNode parent=getParent();
			
    		int cIndex=parent.indexOfChild(this);
    		int nOfChild=parent.getNumberOfChildren();
    		XYAddress pAddr=parent.getXYAddress(depth+1);
    		
    		return new XYAddress(pAddr.getX()+ childPosition(nOfChild, cIndex), pAddr.getY()+100);
    	}
    }
    private int childPosition(int nOfChild, int cIndex){
    	if(nOfChild<=1)return -100;
    	else return 200*cIndex/(nOfChild-1)-100;
    }
}
