/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

/* 
 * @author HyperCast Team
 */

// TODO Add comments 

package runcontrol.commands;

import runcontrol.RunControl;
import runcontrol.RunControlCommand;
import runcontrol.CommandRunner;
import runcontrol.CommandFailureException;

import java.io.BufferedReader;
import java.io.PrintWriter;

import java.util.StringTokenizer;

import hypercast.HyperCastConfig;
import hypercast.MonitorAndControl.I_Monitor;
import hypercast.MonitorAndControl.MonMessage;

import hypercast.util.XmlUtil;

/* Command to check the synchronization of the server clocks.*/
public final class CheckClockSynchronizationCommand
    extends RunControlCommand
{
	private RunControl runControl;

	public CheckClockSynchronizationCommand(RunControl r) {
		runControl = r;
	}

	public String name() {
		return "check_clock_synchronization";
	}

	public String description() {
		return " checks if clocks are synchronized.";
	}

	public String longDescription() {
		return "This command prints a warning if the clocks of the servers are no synchronized\n"
			+ "to the control.";
	}

    public String parameterDescription() { 
        return "";
    }

    public int minParameters() { return 0; }
    
    public int maxParameters() { return 0; }

	public void action(
		String command,
		String firstArg,
		String secondArg,
		String thirdArg,
		StringTokenizer furtherArgs,
		BufferedReader in,
		PrintWriter out,
		PrintWriter err)
		throws Exception {
		checkAllServers(runControl, err);
	}

	public static void checkAllServers(RunControl runControl, PrintWriter err) throws Exception {
		int nodesOnServers[] = runControl.getSocketStatusTable().getNodesOnServer();
		for (int i = 0; i < nodesOnServers.length; i++) {
			if (runControl.getMonitor().isPortalActive(i)) {
				checkOneServer(runControl, i, err);
			}
		}
	}

	public static void checkOneServer(RunControl runControl, int RSIndex, PrintWriter err) throws Exception {
		
		I_Monitor monitor=runControl.getMonitor();
		CommandRunner cmdRunner=runControl.getCommandRunner();
		
		MonMessage msg=monitor.createGetRequest();
		msg.addStat(XmlUtil.createXPath("/Portal/Appl/CurrentTime"));
		
		long sendTime = System.currentTimeMillis();
		MonMessage reply=cmdRunner.sendQuery(RSIndex, msg);
		
		long receiveTime = System.currentTimeMillis();

		if (reply == null) {
			throw new Exception("RunServer " + RSIndex + " did not report.");
		} else if (reply.isStatError(0)) {
			throw new Exception(
				"Runserver "
					+ RSIndex
					+ " encountered error: "
					+ XmlUtil.getScalarValue( reply.getStatValue(0) )
					+ ")");
		} else {
			long timeOfFlight = receiveTime - sendTime;

			// if retransmission occured, try again.
			//TODO ?? ask greg
			//if (timeOfFlight >= cmdRunner.getTimeout()) {
				//checkOneServer(runControl, RSIndex, err);
			//} else 
			{
				// read out value
				long currentTimeLong = Long.parseLong(XmlUtil.getScalarValue( reply.getStatValue(0) ));
				
				if (currentTimeLong < sendTime) {
					err.println(
						"RunServer at index "
							+ RSIndex
							+ " has a clock off by at least "
							+ Math.abs(currentTimeLong - sendTime)
							+ " milliseconds.");
				} else if (currentTimeLong > receiveTime) {
					err.println(
						"RunServer at index "
							+ RSIndex
							+ " has a clock off by at least "
							+ Math.abs(currentTimeLong - receiveTime)
							+ " milliseconds.");
				} else{
					err.println("RunServer at index "+ RSIndex +" has exactly the same clock");
				}
			}
		}
	}
}
