/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

/* 
 * @author HyperCast Team
 */

// TODO No comments in this file

package runcontrol.commands;

import runcontrol.RunControl;
import runcontrol.RunControlCommand;
import runcontrol.CommandRunner;
import runcontrol.CommandFailureException;

import java.io.BufferedReader;
import java.io.PrintWriter;

import java.util.StringTokenizer;

import org.w3c.dom.Element;

import hypercast.HyperCastConfig;
import hypercast.HyperCastStatsException;

import hypercast.DT.DT_Node_ServerJoin;
import hypercast.MonitorAndControl.I_Monitor;
import hypercast.MonitorAndControl.MonMessage;
import hypercast.MonitorAndControl.NoSuchPortalException;

import hypercast.HyperCastStatsException;
import hypercast.util.XmlUtil;
import hypercast.util.XmlUtil;

public  final class GetExactStableTimeCommand
    extends RunControlCommand
{

    private RunControl runControl;

    public GetExactStableTimeCommand(RunControl r) 
    {
        runControl = r;
    }

    public String name() 
    {
        return "get_stable_time";
    }

    public String description() 
    {
        return " prints exact time topology went stable.";
    }

    public String longDescription() 
    {
        return "prints the most recent time any node changed its neighborhood.";
    }

    public void action(String command, String firstArg, String secondArg, String thirdArg, StringTokenizer furtherArgs, BufferedReader in, PrintWriter out, PrintWriter err) throws Exception    
    {
        getMaxTime(runControl, "LastChangeToNeighborhood", "stable time", true, out, err);
    }
     


    public String parameterDescription()
    {
        return "";
    }

    public int minParameters()
    {
        return 0;
    }

    public int maxParameters()
    {
        return 0;
    }

    public static void getMaxTime (RunControl runControl, String timeField, String timeName, boolean checkStable, PrintWriter out, PrintWriter err)
        throws InterruptedException, CommandFailureException, HyperCastStatsException
    {
        synchronized (runControl.getSocketStatusTable().getLock()) 
        {
            int nodesOnServers[] = runControl.getSocketStatusTable().getNodesOnServer();

            int msgsNeeded = 0;
            int socketsUsed = 0;
            for (int i = 0; i < nodesOnServers.length; i++) 
            {
                if (nodesOnServers[i] > 0)
                {
                    msgsNeeded++;
                    socketsUsed += nodesOnServers[i];
                }
            }
         
            // get addresses messages
            int[] addrs = new int[msgsNeeded];
            int j = 0;
            for (int i = 0; i < nodesOnServers.length;i++) 
            {
                if (nodesOnServers[i] > 0) 
                {
                    addrs[j] = i;
                    j++;
                }
            }
            I_Monitor monitor=runControl.getMonitor();
            CommandRunner cmdRunner=runControl.getCommandRunner();
            // create messages
            MonMessage[] msgs = new MonMessage[msgsNeeded];
            j = 0;
            for (int i = 0; i < nodesOnServers.length; i++) 
            {
                if (nodesOnServers[i] > 0) 
                {
                    msgs[j] = monitor.createGetRequest();
                    for (int k = 1; k <= nodesOnServers[i]; k++) 
                    {
                        msgs[j].addStat(XmlUtil.createXPath("/Portal/Appl/Socket["+k+"]/Node/"+timeField));
                        if(checkStable)
                        	msgs[j].addStat(XmlUtil.createXPath("/Portal/Appl/Socket["+k+"]/Node/State"));
                    }

                    j++;
                }
            }
         
         
            // send messages
            MonMessage[] result; try { result = cmdRunner.sendQueries(addrs, msgs);
            }
            catch (final NoSuchPortalException TODO)
            {
                TODO.printStackTrace();
                throw new CommandFailureException ("NoSuchPortalException caught");
            }
         
            // read out data
            long maxTimeReported = 0;
            j = 0;
            boolean stable = true;
            for (int i = 0; i < nodesOnServers.length; i++) 
            {
                if (nodesOnServers[i] > 0)
                {
                    if (result[j] == null)
                    {
                        err.println("RunServer " + i + " did not report.");
                    }
                    else 
                    {
                        for (int k = 1; k <= nodesOnServers[i]; k++)
                        {
                            if(result[j].isStatError(XmlUtil.createXPath("/Portal/Appl/Socket["+k+"]/Node/"+timeField))){
                                err.println("Runserver " + i + " encountered error: " + XmlUtil.getScalarValue( result[j].getStatValue(XmlUtil.createXPath("/Portal/Appl/Socket["+k+"]/Node/"+timeField))) + ")");
                                continue;
                            }
                            Element Result = result[j].getStatValue(XmlUtil.createXPath("/Portal/Appl/Socket["+k+"]/Node/"+timeField))[0];
                            if ((timeField.equals(Result.getNodeName())) && (Result.getChildNodes().getLength() >0))
                                maxTimeReported = Math.max(maxTimeReported, Long.parseLong(Result.getChildNodes().item(0).getNodeValue()));
                            
                            if(checkStable){
                            	if(result[j].isStatError(XmlUtil.createXPath("/Portal/Appl/Socket["+k+"]/Node/State"))){
                                    err.println("Runserver " + i + " encountered error: " + XmlUtil.getScalarValue( result[j].getStatValue(XmlUtil.createXPath("/Portal/Appl/Socket["+k+"]/Node/State"))) + ")");
                                    continue;
                                }
	                            String stateStr = XmlUtil.getScalarValue(result[j].getStatValue(XmlUtil.createXPath("/Portal/Appl/Socket["+k+"]/Node/State")));
	                            int state = Integer.parseInt(stateStr);
	                            if(state != DT_Node_ServerJoin.STABLE && state != DT_Node_ServerJoin.LEADER_STABLE){
	                            	err.println("warnning: some sockets on RS "+i +" are not stable");
	                            	stable = false;
	                            	break;
	                            }
                            }
                             
                        }       
                    }   
                    // else "NACK" 
                    //  do nothing.
                     
                    j++;
                }
            }
            
            if(!checkStable){
            	stable = true;
            }
            if(stable){
            	if(maxTimeReported == 0)
            		out.println("no socket is running.");
            	else
            		out.println(timeName+" = "+maxTimeReported);
            }else{
            	out.println("network is unstable, "+timeName+" not available.");
            }
        }
    }
}
