/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

/* 
 * @author HyperCast Team
 */

// TODO Improve comments 

package runcontrol.commands;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintWriter;

import java.util.StringTokenizer;

import org.apache.xpath.XPath;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.Attr;
import org.w3c.dom.NodeList;

import hypercast.HyperCastConfig;
import hypercast.HyperCastStatsException;
import hypercast.StatsProcessor;
import hypercast.MonitorAndControl.I_Monitor;
import hypercast.MonitorAndControl.MonMessage;
import hypercast.MonitorAndControl.NoSuchPortalException;
import hypercast.util.XmlUtil;
import hypercast.util.XmlUtil;

import runcontrol.RunControl;
import runcontrol.RunControlCommand;
import runcontrol.CommandRunner;
import runcontrol.CommandFailureException;

/**
 * Command to get the value of a specified variable from a specified Portal.
 */
public final class GetValueCommand
    extends RunControlCommand
{
    private RunControl runControl;

    public GetValueCommand (final RunControl runControl)
    {
        this.runControl = runControl;
    }

    public String name()
    {
        return "get_value";
    }

    public String parameterDescription()
    {
        return "[monitor | portalindex | all]  XPathExpression";
    }

    public String description()
    {
        return "Accesses a statistic at a RunServer application or the monitor.";
    }

    public String longDescription()
    {
        return "Accesses a statistic at a RunServer application or the monitor. \n" +
        		"The command has two mandatory arguments. The first argument can \n" +
        		"specify a RunServer by the index of a portal in the portal list \n" +
        		"of the monitor. If the argument is set to monitor, the command \n" +
        		"accesses a statistics of the monitor in the RunControl application. \n" +
        		"The second argument specifies the requested statistic in terms of a \n" +
        		"XPath expression. ";
    }

    public int minParameters()
    {
        return 2;
    }
    public int maxParameters()
    {
        return 2;
    }
    
    public void action (final String          command,
                        final String          firstArg,
                        final String          secondArg,
                        final String          thirdArg,
                        final StringTokenizer furtherArgs,
                        final BufferedReader  in,
                        final PrintWriter     out,
                        final PrintWriter     err)
        throws Exception
    {
    	final I_Monitor monitor = runControl.getMonitor();
    	
    	if(firstArg.equals("monitor")){
    		//get RunControl statistics
    		Document doc=XmlUtil.createDocument();
    		try{
    			Element [] resultArray = runControl.getMonitor().getStats(doc, XmlUtil.createXPath(secondArg));
    			if(resultArray.length>0)
    				displayResult (resultArray, out, err);
    			else err.println("ERROR: statistics not found for "+secondArg);

    		}catch(HyperCastStatsException hcse){
    			err.println("ERROR: "+hcse.toString());
    		}
			
			return;
    	}else if(firstArg.equals("all")){
    		//get statistics on all portals
    		int portals [] = monitor.getActivePortalIndices();
    		for(int i=0;i<portals.length;i++){
    			this.getPortalStats(portals[i], secondArg, out, err);
    		}
    	}else{
	    	int portalIndex = toInt (firstArg);
	        
	        if (portalIndex > (monitor.getAllPortalIndices().length - 1))
	            throw new CommandFailureException ("\"" + portalIndex + "\" is not the index of Portal.");
	
	        if (! monitor.isPortalActive (portalIndex))
	            throw new CommandFailureException ("Portal " + portalIndex + " is no longer alive.");
	        
	        this.getPortalStats(portalIndex, secondArg, out, err);
    	}
        
    }

    private void getPortalStats(int portalIndex, String secondArg, PrintWriter out, PrintWriter err) throws InterruptedException, NoSuchPortalException, HyperCastStatsException{
    	final I_Monitor monitor = runControl.getMonitor();
        final CommandRunner cmdRunner = runControl.getCommandRunner();
        

        final MonMessage query = monitor.createGetRequest();
        query.addStat (XmlUtil.createXPath(secondArg));

        /*
         * Send query to portal and block until response is
         * received, timeout, or interrupt.
         */
        final MonMessage reply = cmdRunner.sendQuery (portalIndex, query);

        if (reply == null)
        {
            err.println ("Message not acked by RS " + portalIndex);
        }
        else
        {
            if (reply.isStatError (XmlUtil.createXPath(secondArg)))
            {
                err.println ("ERROR: " + XmlUtil.getScalarValue(reply.getStatValue (XmlUtil.createXPath(secondArg))));
            }
            else
            {
                out.println ("Portal "+portalIndex+": "+secondArg);
                displayResult (reply.getStatValue (0), out, err);
            }
        }
    }

    /** Display a subtree with "node" as the root.
     * @param node The root of the subtree.
     * @param indentDistance The number of base indent space for displaying statements.
     * @param out The output device.
     */
    private void displayResult(final Element[]result, PrintWriter out, PrintWriter err){
    	for(int i=0;i<result.length;i++){
    		OutputStream outputStream = new java.io.ByteArrayOutputStream();
    		Document doc = XmlUtil.createDocument();
    		Element value = (Element) doc.importNode(result[i], true);
    		doc.appendChild(value);
    		try {
    			XmlUtil.writeXml (doc, outputStream, true);
				out.print (((java.io.ByteArrayOutputStream) outputStream).toString());
			} catch (IOException e) {
				err.println("Cannot display statistics");
			}
			
    		
    		//this.displayNodeRecursive(result[i], 2, out);
    	}
    }
    private void displayNodeRecursive (final Node node, final int indentDistance, final PrintWriter out)
    {
        if (node == null)
            return;

        String indentString = "";
        for (int i = 0; i < indentDistance; ++i)
        {
            indentString = indentString + " ";
        }
		
        if (node.getNodeType() == Node.TEXT_NODE)
        {
            out.println(indentString + "=" + node.getNodeValue());
        }
        else
        {
            if ("ROOT".equals (node.getNodeName()))
            {
                out.print (indentString + "/");
            }
            else
            {
                out.print (indentString + node.getNodeName());
            }

            if (node instanceof Element)
            {
                Attr attr = ((Element) node).getAttributeNode ("index");
                if (attr != null)
                {
                    out.print ("[" + attr.getNodeValue() + "]");
                }
            }

            out.println();

            final NodeList nodeList = node.getChildNodes();

            for (int i = 0; i < nodeList.getLength(); ++i)
            {
                if (nodeList.item (i) == null)
                {
                    System.out.println ("nodeList.item(" + i + ") is null");
                }
                else
                {
                    displayNodeRecursive (nodeList.item (i), indentDistance + 2, out);
                }
            }
        }
    }
}
