/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

/* 
 * @author HyperCast Team
 */

// TODO Add comments 

package runcontrol.commands;

import runcontrol.RunControl;
import runcontrol.RunControlCommand;
import runcontrol.CommandFailureException;

import java.io.BufferedReader;
import java.io.PrintWriter;

import java.util.TreeSet;
import java.util.StringTokenizer;
import java.util.Set;
import java.util.Iterator;

/* Generic command to get help.*/
public final class HelpCommand
    extends RunControlCommand
{
    private RunControl runControl;

    public HelpCommand(RunControl r) {
        runControl = r;
    }

    public String parameterDescription() {
        return " [command]";
    }

    public String name()
    {
        return "help";
    }

    public String description() {
        return "without argument, lists all commands; with argument, describes command.";
    }

    public String longDescription() {
        return "Provides help.\n"
            + "Without any argument, the command prints out a list of all commands.\n"
            + "With a command name as an argument, \n"
            + "it prints the long description of that command.";
    }

    public int maxParameters() {
        return 1;
    }

    public int minParameters() {
        return 0;
    }

    public void action(
        String command,
        String firstArg,
        String secondArg,
        String thirdArg,
        StringTokenizer furtherArgs,
        BufferedReader in,
        PrintWriter out,
        PrintWriter err)
        throws Exception {
        if (firstArg == null)
            listCommands(out);
        else
            printLongHelp(firstArg, out);
    }

    private void listCommands (final PrintWriter out)
    {
        final Set keys = new TreeSet (runControl.getCommands().keySet());
        final Iterator itor = keys.iterator();
        while (itor.hasNext())
        {
            final String key = (String) itor.next();
            final RunControlCommand commandObject = (RunControlCommand) runControl.getCommands().get(key);
            out.println (key +
                         " " +
                         commandObject.parameterDescription() +
                         " - " +
                         commandObject.description());
        }
    }

    private void printLongHelp(String command, PrintWriter out) {
        RunControlCommand commandObject = (RunControlCommand) runControl.getCommands().get(command);
        if (commandObject == null)
            throw new IllegalArgumentException("No command with name \"" + command + "\"");
        out.println(command + " " + commandObject.parameterDescription());
        out.println(commandObject.longDescription());
    }
}

