/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

/* 
 * @author HyperCast Team
 */

package runcontrol.commands;

import java.io.BufferedReader;
import java.io.PrintWriter;

import java.util.StringTokenizer;

import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.Attr;
import org.w3c.dom.NodeList;

import hypercast.HyperCastConfig;
import hypercast.MonitorAndControl.I_Monitor;
import hypercast.MonitorAndControl.MonMessage;
import hypercast.MonitorAndControl.NoSuchPortalException;
import hypercast.util.XmlUtil;
import hypercast.util.XmlUtil;

import runcontrol.RunControl;
import runcontrol.RunControlCommand;
import runcontrol.CommandRunner;
import runcontrol.CommandFailureException;

/**
 * Command to get the value of a specified variable from a specified Portal.
 */
public final class KillRemoteServersCommand
    extends RunControlCommand
{
    private RunControl runControl;

    public KillRemoteServersCommand (final RunControl runControl)
    {
        this.runControl = runControl;
    }

    public String name()
    {
        return "kill_remote_servers";
    }

    public String description()
    {
        return "stops all RunServers.";
    }

    public String longDescription()
    {
        return "This command sends kill messages causing all connected (alive) RunServers to halt.";
    }

    public String parameterDescription() { return "";}

    public int minParameters() { return 0; }

    public int maxParameters() { return 0; }

    public void action (final String          command,
                        final String          firstArg,
                        final String          secondArg,
                        final String          thirdArg,
                        final StringTokenizer furtherArgs,
                        final BufferedReader  in,
                        final PrintWriter     out,
                        final PrintWriter     err)
        throws Exception
    {		
        final I_Monitor monitor = runControl.getMonitor();
        final CommandRunner cmdRunner = runControl.getCommandRunner();
		
        final long currentTime = System.currentTimeMillis();

        final boolean [] toBeKilled = new boolean [monitor.getAllPortalIndices().length];
        int numAlive = 0;
        for (int i = 0; i < toBeKilled.length; ++i)
        {
            if (monitor.isPortalActive (i))
            {
                toBeKilled[i] = true;
                numAlive++;
            }
            else
            {
                toBeKilled[i] = false;
            }
        }

        // get Portal indices
        final int [] portalIndices = new int [numAlive];
        int j = 0;
        for (int i = 0; i < toBeKilled.length; ++i)
        {
            if (toBeKilled [i])
            {
                portalIndices [j] = i;
                j++;
            }
        }

        // create messages

        final MonMessage [] killQueries = new MonMessage [numAlive];
        for (j = 0; j < numAlive; j++)
        {
            killQueries [j] = monitor.createSetRequest();
            killQueries[j].addStat(XmlUtil.createXPath("/Portal/Appl/KillApp"), XmlUtil.getXmlValue(XmlUtil.createDocument(), "/Portal/Appl/KillApp", "true"));
            //RunControl.addScalarStatsIntoMsg(killQueries[j], "/Portal/Appl/KillApp", "true");
        }

        err.println ("Killing " + numAlive + " servers.");

        // send messages

        try
        {
            // No response is received from killed servers
            cmdRunner.sendUnblockedQueries (portalIndices, killQueries);
        }
        catch (final NoSuchPortalException nspe)
        {
            nspe.printStackTrace();
            throw new CommandFailureException ("NoSuchPortalException caught", nspe);
        }
    }
}
