/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

/* 
 * @author HyperCast Team
 */

package runcontrol.commands;

import runcontrol.RunControl;
import runcontrol.RunControlCommand;
import runcontrol.CommandFailureException;

import java.io.BufferedReader;
import java.io.PrintWriter;

import java.util.StringTokenizer;

import hypercast.MonitorAndControl.I_Monitor;
import hypercast.I_LogicalAddress;

/**
 * Command to list known Portals.
 *
 * TODO: take an argument of portal index or range of indices:
 *    list
 *    list 3
 *    list 10-20
 *    list 1,5,7-12
 */
public final class ListCommand
    extends RunControlCommand
{
    private RunControl runControl;

    public ListCommand (final RunControl runControl)
    {
        this.runControl = runControl;
    }

    public String name()
    {
        return "list_portals";
    }

    public String parameterDescription()
    {
        return "[all]";
    }

    public String description()
    {
        return "Lists known Portals.";
    }

    public String longDescription()
    {
        return
            "Lists known Portals. Each Portal is displayed with its non-negative"   + "\n" +
            "integer index as well as whether the Portal is considered to be Alive" + "\n" +
            "or Dead. By default, only Alive Portals are displayed, use argument"   + "\n" +
            "\"all\" to see all Portals. ";
    }

    public int minParameters()
    {
        return 0;
    }

    public int maxParameters()
    {
        return 1;
    }

    public void action (final String          command,
                        final String          firstArg,
                        final String          secondArg,
                        final String          thirdArg,
                        final StringTokenizer furtherArgs,
                        final BufferedReader  in,
                        final PrintWriter     out,
                        final PrintWriter     err)
        throws Exception
    {
        final I_Monitor monitor = runControl.getMonitor();

        //synchronized (runControl.getExperimentLock())
        {
            final int [] portalIndices;
            //boolean currentExperimentOnly = false;

            if (firstArg != null
                &&
                ! firstArg.equals (""))
            {
                if (firstArg.equals ("all"))
                {
                    portalIndices = monitor.getAllPortalIndices();
                }
                else
                {
                    throw new CommandFailureException ("Unknown option: " + firstArg);
                }
            }
            else
            {
                portalIndices = monitor.getActivePortalIndices();
            }

            int numberListed = 0;
            
            
            
            for (int i = 0; i < portalIndices.length; ++i)
            {
                final I_LogicalAddress logicalAddress = monitor.getPortalAddress (portalIndices [i]);
                final boolean isActive = monitor.isPortalActive (portalIndices [i]);

                final String portalInfo = "<Portal "+portalIndices[i] + ": Address=" + logicalAddress + " Status=" + (isActive ? "Active" : "Inactive") + "> ";

                boolean hasSocketsForCurrentExperiment = false;
                    
            
                boolean needNewline = false;
                
                ++numberListed;
                out.print (portalInfo);
                    
                out.println();

            }

            out.println (numberListed + " Portals listed");
        }
    }
}
