/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/


package runcontrol.commands;

import hypercast.MonitorAndControl.NoSuchPortalException;

import java.io.BufferedReader;
import java.io.PrintWriter;

import runcontrol.RunControl;
import runcontrol.RunControlCommand;
import runcontrol.CommandFailureException;
import runcontrol.SocketStatusTable;

/* 
 * @author HyperCast Team
 * @author gd4j
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public final class ListSocketsCommand
    extends RunControlCommand
{
	
    RunControl runControl;
    public ListSocketsCommand(RunControl rc){
        runControl = rc;
    }
    /* (non-Javadoc)
     * @see runcontrol.RunControlCommand#action(java.lang.String, java.lang.String[], java.io.BufferedReader, java.io.PrintWriter, java.io.PrintWriter)
     */
    public void action(String command, String[] args, BufferedReader in, PrintWriter out, PrintWriter err) 
        throws Exception
    {
		
        SocketStatusTable table = runControl.getSocketStatusTable();
		
        boolean verbose = false;
        int row = -1;
		
        for(int i=0;i<args.length;i++){
            if(args[i].equals("row")){
                if(i<args.length-1){
                    i++;
                    row = Integer.parseInt(args[i]);
                }else{
                    throw new IllegalArgumentException("Need a row value");
                }
            }else if(args[i].equals("verbose")){
                verbose = true;
            }else {
                throw new IllegalArgumentException("Unsupported parameter: "+args[i]);
            }
        }
        synchronized(runControl.getSocketStatusTable().getLock()){
            if(row>=0){
                out.println("RunServer #\tCreated\tStarted\tAvailable");
                out.println(getSocketInformation(row));
                out.println("");
            }
            else{
                if(verbose){
			
                    out.println("RunServer #\tCreated\tStarted\tAvailable");
                    for(int i=0;i<table.size();i++){
                        out.println(getSocketInformation(i));
                    }
                    out.println("----------------------------------");
                }
				
                out.println("Sockets created   = "+table.getTotalSocketCount());
                out.println("Sockets started   = "+table.getRunningSocketCount());
                out.println("Sockets available = "+table.getAvaiableSocketCount());
                out.println("");
				
            }
        }
		
    }
    private String getSocketInformation(int i) throws NoSuchPortalException, InterruptedException{
        SocketStatusTable table = runControl.getSocketStatusTable();
        String info = ("RunServer "+i+":\t"+table.getTotalSocketCount(i)+"\t"+table.getRunningSocketCount(i)+"\t"+table.getAvailableSocketCount(i));
        String status="";
        if(!table.isSync(i)){
            status+="Unsync";
            if(!table.isActive(i)){
                status+="/Dead";
            }
        }
        else if(!table.isActive(i)){
            status+="Dead";
        }
        info+=("\t"+status);
		
        return info;
    }
    public String name() {
        return "list_sockets";
    }
	
    public String parameterDescription()
    {
        return "[verbose] [row i]";
    }

    public int minParameters() { return 0; }

    public int maxParameters() { return 3; }

    /* (non-Javadoc)
     * @see runcontrol.RunControlCommand#description()
     */
    public String description() {
        return "Lists the information at RunControl about the number of overlay sockets";
    }

    /* (non-Javadoc)
     * @see runcontrol.RunControlCommand#longDescription()
     */
    public String longDescription() {
        return "Lists the information at RunControl about the number of overlay sockets. The\n " +
            "command lists the number of sockets that have been created at the overlay\n " +
            "socket (created sockets), the number of sockets that have joined the overlay\n " +
            "sockets (started sockets), and the number of overlay sockets that have not been\n" +
            "started (available sockets).  Without an argument, the commands lists aggregate\n" +
            "values for all remote RunServers. With the argument verbose, the command additionally\n" +
            "displays the status at each remote RunServer. With the argument row I, the command\n" +
            "displays the state only at the ith RunServer.";
    }
}
