/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

/* 
 * @author HyperCast Team
 */

// TODO Add comments 

package runcontrol.commands;

import runcontrol.RunControl;
import runcontrol.RunControlCommand;
import runcontrol.CommandFailureException;

import java.io.FileReader;
import java.io.BufferedReader;
import java.io.PrintWriter;

import java.util.StringTokenizer;
import java.util.Vector;

/** A command to run a list of commands stored in a script file */
public final class RunScriptCommand extends RunControlCommand { 
	RunControl rc;
	public RunScriptCommand(RunControl r){
		rc=r;
	}
	public String parameterDescription() {
		return " scriptfile [args...]";
	}

    public String name()
    {
        return "run";
    }

	public String description() {
		return "Run a script file";
	}

	public String longDescription() {
		return "Run a script file containing a list of commands. DOS style parameter symbol is supported";
	}

	public int minParameters() {
		return 1;
	}

	public int maxParameters() {
		return 10;
	}

	public void action(
		String command,
		String firstArg,
		String secondArg,
		String thirdArg,
		StringTokenizer furtherArgs,
		BufferedReader in,
		PrintWriter out,
		PrintWriter err)
		throws Exception {
		
		out.println("Run script file: "+firstArg);
		
		String lines[]=readScriptFile(firstArg, err);
		
		try {
			/*
			 * replace the '%n' in the script file with parameters
			 * so that the script can have arguments in the same way as a DOS batch file
			 */
			if(furtherArgs!=null){
				int n=3;
				while(furtherArgs.hasMoreElements()){
					String arg=furtherArgs.nextToken();
					String toReplace="%"+n;
					for(int i=0;i<lines.length;i++)
						lines[i]=lines[i].replaceAll(toReplace, arg);
					n++;
				}
			}
			for (int i = 0; i < lines.length; i++){
				if(secondArg!=null)
					lines[i]=lines[i].replaceAll("%1", secondArg);
				if(thirdArg!=null)
					lines[i]=lines[i].replaceAll("%2", thirdArg);
				
				out.println(lines[i]);
				/* run the command line one by one*/
				rc.processCommand(lines[i], in, out, err);
				out.println("\n");
			}
		} catch (Exception e) {
			if (!"break".equals(e.getMessage()))
				throw e;
		}
			 
		return;
	}
	/** 
	 * Read all command lines from a script file
	 * @param scriptfile	
	 * @param err
	 * @return	the list of commands
	 */
	private String []readScriptFile(String scriptfile, PrintWriter err){
		Vector lns=new Vector();
		
		BufferedReader input=null;
		
		try{
			input=new BufferedReader(new FileReader(scriptfile));
			
			String line=input.readLine().trim();
			
			while(line!=null){
				line=line.trim();
				if(!line.equals("")){
					lns.addElement(line);
				}
				
				line=input.readLine();
			}
		}
		catch(Exception e){
			err.println("Can't read from "+scriptfile+" : "+e.toString());	
			
		}
		finally{
			try{
				if(input!=null)input.close();
			}catch(Exception e){}
		}
		
		String[]lines=new String[lns.size()];
		for(int i=0;i<lines.length;i++){
			lines[i]=(String)(lns.elementAt(i));
		}
		
		return lines;
	}
}

