/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

/* 
 * @author HyperCast Team
 */

// TODO Add comments 

package runcontrol.commands;

import runcontrol.RunControl;
import runcontrol.RunControlCommand;
import runcontrol.CommandRunner;
import runcontrol.CommandFailureException;

import java.io.BufferedReader;
import java.io.PrintWriter;
import java.io.IOException;

import java.util.StringTokenizer;

import org.w3c.dom.Attr;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


import hypercast.MonitorAndControl.I_Monitor;
import hypercast.MonitorAndControl.MonMessage;
import hypercast.MonitorAndControl.NoSuchPortalException;

import hypercast.HyperCastConfig;
import hypercast.HyperCastStatsException;

import hypercast.util.XmlUtil;
import hypercast.util.XmlUtil;

/* Command to save the schema of a specified Portal to a speicified file.*/
public final class SaveReadSchemaCommand
    extends RunControlCommand
{
	private RunControl runControl;

	public SaveReadSchemaCommand(RunControl r) {
		runControl = r;
	}

	public String parameterDescription() {
		return "[monitor | portalindex | all] [Xpathexpression] [stdout | fname]";
	}

	public String description() {
		return "Retrieves a schema of readable statistics from a RunServer application";
	}

	public String name() {
            return "get_readschema";
	}

	public String longDescription() {
		return "Retrieves a schema of readable statistics from a RunServer application. The first \n" +
				"argument specifies the remote RunServer application through an index of the \n" +
				"corresponding portal. The second argument is an XPath expression that identifies \n" +
				"the requested schema. The third argument specifies if the output should be written \n" +
				"to the monitor (stdout) or to a file with name fname.";
	}

	public int minParameters() {
		return 2;
	}
	public int maxParameters() {
		return 3;
	}

    public void action (String          command,
                        String          firstArg,
                        String          secondArg,
                        String          thirdArg,
                        StringTokenizer furtherArgs,
                        BufferedReader  in,
                        PrintWriter     out,
                        PrintWriter     err)
        throws Exception
    {
        java.io.OutputStream outputStream;
        I_Monitor monitor = runControl.getMonitor();
        
        if (thirdArg == null
            ||
            thirdArg.equals ("")
            ||
            thirdArg.equals ("stdout"))
        {
            outputStream = new java.io.ByteArrayOutputStream();
        }
        else
        {
            try
            {
                outputStream = new java.io.FileOutputStream (thirdArg, true);
            }
            catch (IOException ioe)
            {
                throw new CommandFailureException ("\"" + thirdArg + "\" is not a legal file name (" + ioe.getMessage() + ")");
            }
        }
		
        try
        {
            if(firstArg.equals("monitor")){
            	//get RunControl statistics
            	try{
	        		Document doc=XmlUtil.createDocument();
	        		Element [] schemaArray = runControl.getMonitor().getReadSchema(doc, XmlUtil.createXPath(secondArg));
	        		if(schemaArray.length>0)
	        			this.printSchemas("<!-- RunControl: \""+secondArg +"\" ReadSchema  -->\n", schemaArray, outputStream, out);
	        		else
	        			err.println("ERROR: statistics not found for "+secondArg);
            	}catch(HyperCastStatsException hcse){
        			err.println("ERROR: "+hcse.toString());
        		}
        		
            }else if(firstArg.equals("all")){
            	int portals [] = monitor.getActivePortalIndices();
        		for(int i=0;i<portals.length;i++){
        			this.getPortalReadSchema(portals[i], secondArg, outputStream, out, err);
        		}
            }else{
	            int portalIndex = toInt (firstArg);
	            if (portalIndex > monitor.getAllPortalIndices().length - 1
	                ||
	                ! monitor.isPortalActive (portalIndex))
	            {
	                throw new CommandFailureException ("\"" + firstArg + "\" is not an active portal index");
	            }
	            this.getPortalReadSchema(portalIndex, secondArg, outputStream, out, err);
            }
            
        }
        finally
        {
            if (outputStream != null)
            {
                outputStream.close();
            }
        }
    }
    
    private void getPortalReadSchema(int portalIndex, String xpath, java.io.OutputStream outputStream, PrintWriter out, PrintWriter err) throws InterruptedException, NoSuchPortalException, HyperCastStatsException, DOMException, IOException{
    	I_Monitor monitor = runControl.getMonitor();
        CommandRunner commandRunner = runControl.getCommandRunner();
        
        MonMessage doc = monitor.createReadSchemaRequest();
        doc.addStat (XmlUtil.createXPath(xpath));
	
        MonMessage reply = commandRunner.sendQuery (portalIndex, doc);

        if (reply == null)
        {
            err.println ("Message was unacknowledged.");
        }
        else if (reply.isStatError (0))
        {
            err.println ("ERROR: " + XmlUtil.getScalarValue( reply.getStatValue (0)) );
        }
		else if (reply.getStatValue(0).length == 0)
		{
			err.println ("The reply is empty.");
		}
		else 
		{
			//this.printSchemas("<!-- Portal "+portalIndex+": \""+xpath +"\" ReadSchema  -->\n", reply.getStatValue(0), outputStream, out);
			this.printSchemas("<!-- Portal "+portalIndex+": \""+xpath +"\" WriteSchema  -->\n", this.createSchemaDocument(reply.getStatValue(0)[0].getOwnerDocument(), reply.getStatValue(0)), outputStream, out);
		}

    }

	/** 
	 * Convert the schema subtree represented by the given Element to
	 * a schema document, and return the root Element of the schema document.
	 * 
	 * @param doc   Document used for create new elements or nodes.
	 * @param el	Element instance representing the schema subtree.
	 * 
	 */
	private Element[] createSchemaDocument(Document doc, Element[] el)
	{
		Element[] schemaTopElement = new Element[el.length];
		
		for (int i=0; i<el.length; i++)
		{
			/*
			 * Create entire schema tree which contains the result Element.
			 */
			schemaTopElement[i] = doc.createElement ("xsd:schema");
			schemaTopElement[i].setAttribute ("xmlns:xsd", "http://www.w3.org/2000/10/XMLSchema");

			Element simpletypeElement1 = doc.createElement ("xsd:simpleType");
			simpletypeElement1.setAttribute ("name", "IPv4AndOnePort");
			schemaTopElement[i].appendChild (simpletypeElement1);
		
			Element restrictionBaseElement1 = doc.createElement ("xsd:restriction");
			restrictionBaseElement1.setAttribute ("base", "xsd:string");
			simpletypeElement1.appendChild (restrictionBaseElement1);
				
			Element patternValueElement1 = doc.createElement ("xsd:pattern");
			patternValueElement1.setAttribute ("value", "\\d+\\.\\d+\\.\\d+\\.\\d+\\:\\d+");
			restrictionBaseElement1.appendChild (patternValueElement1);

			Element simpletypeElement2 = doc.createElement ("xsd:simpleType");
			simpletypeElement2.setAttribute ("name", "IPv4AndTwoPorts");
			schemaTopElement[i].appendChild (simpletypeElement2);
		
			Element restrictionBaseElement2 = doc.createElement ("xsd:restriction");
			restrictionBaseElement2.setAttribute ("base", "xsd:string");
			simpletypeElement2.appendChild (restrictionBaseElement2);
				
			Element patternValueElement2 = doc.createElement ("xsd:pattern");
			patternValueElement2.setAttribute ("value", "\\d+\\.\\d+\\.\\d+\\.\\d+\\:\\d+\\:\\d+");
			restrictionBaseElement2.appendChild (patternValueElement2);

			schemaTopElement[i].appendChild (el[i]);
		}

		return schemaTopElement;
	}

    private void printSchemas(String name,  Element []schemas, java.io.OutputStream outputStream, PrintWriter out) throws IOException{
        
        outputStream.write((name).getBytes());
        
        for(int i=0;i<schemas.length;i++){
        	this.printSchema(schemas[i], outputStream, out);
        }
        
        if (! (outputStream instanceof java.io.FileOutputStream))
        {
            out.print (((java.io.ByteArrayOutputStream) outputStream).toString());
        }

        out.println ("ReadSchema written succesfully.");
    }
    private void printSchema(Element schemaElement, java.io.OutputStream outputStream, PrintWriter out) throws IOException{
    	Document schema = XmlUtil.createDocument();
        Node node = schema.importNode (schemaElement, true);
        schema.appendChild(node);
        
        XmlUtil.writeXml (schema, outputStream);

    }
    
}
