/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

/* 
 * @author HyperCast Team
 */

// TODO Add comments 

package runcontrol.commands;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.text.ParseException;
import java.util.StringTokenizer;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

import hypercast.HyperCastConfig;
import hypercast.HyperCastStatsException;
import hypercast.MonitorAndControl.I_Monitor;
import hypercast.MonitorAndControl.MonMessage;
import hypercast.MonitorAndControl.NoSuchPortalException;
import hypercast.util.XmlUtil;
import hypercast.util.XmlUtil;
import runcontrol.CommandRunner;
import runcontrol.RunControl;
import runcontrol.RunControlCommand;

/* Command to set a specified variable on a specified Portal to a specified value.*/
public final class SetValueCommand extends RunControlCommand {
	private RunControl runControl;

	public SetValueCommand(RunControl r) {
		runControl = r;
	}

	public String parameterDescription() {
		return "[monitor | portalindex | all] [Xpathexpression] [newValue | -f fname]";
	}

    public String name()
    {
        return "set_value";
    }

	public String description() {
		return "Modifies a statistic at a RunServer application or at the monitor";
	}

	public String longDescription() {
		return "Modifies a statistic at a RunServer application or at the monitor. \n" +
				"The command has three mandatory arguments. The first argument can \n" +
				"specify a RunServer by the index of a portal in the portal list of \n" +
				"the monitor. If the argument is set to monitor, the command accesses \n" +
				"the monitor in the RunControl application. The second argument \n" +
				"specifies the requested statistic in terms of a XPath expression.  \n" +
				"The third argument is the new value of the statistic. For statistics \n" +
				"that use a built-in type of XML schema, e.g., integers, strings, etc., \n" +
				"the  new value of the statistic can be typed in as a text string. \n" +
				"Otherwise, the new value of the statistic is provided as a well-formed \n" +
				"XML document stored in file fname.\n";
	}

	public int minParameters() {
		return 3;
	}
	public int maxParameters() {
		return 4;
	}

	/** Uses <code>SetValueCommand.createSimpleMessage</code> to create
	 * the message and then sends it using
	 * <code>ReliableXmlSocket.sendOne</code>. */
	public void action(
		String command,
		String firstArg,
		String secondArg,
		String thirdArg,
		StringTokenizer furtherArgs,
		BufferedReader in,
		PrintWriter out,
		PrintWriter err)
		throws Exception {
		
		I_Monitor monitor=runControl.getMonitor();
		String fourthArg = null;
		if(furtherArgs.hasMoreElements())
			fourthArg = furtherArgs.nextToken();
		
		if(firstArg.equals("monitor")){
    		//set RunControl statistics
			try{
	    		Document doc=XmlUtil.createDocument();
	    		Element [] resultArray = runControl.getMonitor().setStats(doc, XmlUtil.createXPath(secondArg), makeValue(secondArg, thirdArg, fourthArg));
                
			}catch(HyperCastStatsException hcse){
    			err.println("ERROR: "+hcse.toString());
    		}
			return;
    	}else if(firstArg.equals("all")){
    		//set statistics on all portals
    		int portals [] = monitor.getActivePortalIndices();
    		for(int i=0;i<portals.length;i++){
    			this.setPortalStats(portals[i], secondArg, makeValue(secondArg, thirdArg, fourthArg), out, err);
    		}
    	}else{
    		
    		int portalIndex=toInt(firstArg);
    		
    		if (portalIndex>monitor.getAllPortalIndices().length-1)
    			throw new IllegalArgumentException(
    				"\"" + portalIndex + "\" is not the index of any Portal.");
    		
    		this.setPortalStats(portalIndex, secondArg, makeValue(secondArg, thirdArg, fourthArg), out, err);
			
    	}
		
	}
	private Element makeValue(String xpath, String thirdArg, String fourthArg) throws FileNotFoundException, ParseException, IOException{
		Element value;
		if(thirdArg.equals("-f")){
			if(fourthArg==null){
				throw new IllegalArgumentException("must specify a file name");
			}
			Document valueDoc = XmlUtil.createDocument(fourthArg);
			value = valueDoc.getDocumentElement();
		}else{
			value = XmlUtil.getXmlValue(XmlUtil.createDocument(), xpath, thirdArg);
		}
		return value;
	}
	
	private void setPortalStats(int portalIndex, String xpath, Element value, PrintWriter out, PrintWriter err) throws InterruptedException, NoSuchPortalException, HyperCastStatsException{
		I_Monitor monitor=runControl.getMonitor();
		CommandRunner cmdRunner=runControl.getCommandRunner();
		
		MonMessage msg = monitor.createSetRequest();
		msg.addStat(XmlUtil.createXPath(xpath), (value));
		
		MonMessage reply = cmdRunner.sendQuery(portalIndex, msg);

		if (reply == null) {
			err.println("Message was unacknowledged.");
		} else if (reply.isStatError(XmlUtil.createXPath(xpath))) {
			err.println("ERROR: " + XmlUtil.getScalarValue(reply.getStatValue(XmlUtil.createXPath(xpath))));
		} else {
			out.println("Portal "+portalIndex+": the set_value command finishes successfully.");
		}
	}
}
