/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/

/* 
 * @author HyperCast Team
 */

// TODO Add comments 

package runcontrol.commands;

import java.io.BufferedReader;
import java.io.PrintWriter;

import runcontrol.RunControl;
import runcontrol.RunControlCommand;


public final class StopSocketsCommand
    extends RunControlCommand
{

	private RunControl runControl;
	public String name() {
		return "stop_sockets";
	}

	public String description() {
		return "Requests that overlay sockets at the remote RunServers leave the overlay network.";
	}
	
	public String parameterDescription()
    {
        return "[n | all ] [uniform] [rate R]";
    }

	public String longDescription() {
		return "Requests that overlay sockets at the remote RunServers leave the overlay network.\n" +
				"Sockets are stopped by setting the statistic Running in the overlay socket to \n" +
				"false. If the request is larger than the total number of overlay sockets that are\n" +
				"currently started, then all overlay sockets are stopped and a message is displayed.\n" +
				"The arguments n, all, uniform and rate R have the same interpretation as in the \n" +
				"start_sockets command";
	}
	
	public int minParameters() { return 0; }
    
    public int maxParameters() { return 4; }
	
	public StopSocketsCommand(RunControl rc){
		runControl = rc;
	}
	/* (non-Javadoc)
	 * @see runcontrol.RunControlCommand#action(java.lang.String, java.lang.String[], java.io.BufferedReader, java.io.PrintWriter, java.io.PrintWriter)
	 */
	public void action(String command, String[] args, BufferedReader in, PrintWriter out, PrintWriter err)
                throws Exception
{
		int rate = 0;
		boolean uniform=false;
		String toStop=null;
		
		for(int i=0;i<args.length;i++){
			if(args[i].equals("rate")){
				if(i<args.length-1){
					i++;
					rate = Integer.parseInt(args[i]);
				}else{
					throw new IllegalArgumentException("Need a rate value");
				}
			}else if(args[i].equals("uniform")){
				uniform = true;
			}else if(toStop==null){
				toStop = args[i];
			}else {
				throw new IllegalArgumentException("Unsupported parameter: "+args[i]);
			}
		}
		
		int stopped;
		
		if(toStop == null){
			toStop = "all"; 
		}
		synchronized(runControl.getSocketStatusTable().getLock()){
			if(toStop.equalsIgnoreCase("all")){
				out.println("stopping all sockets");
				int running = runControl.getSocketStatusTable().getRunningSocketCount();
				stopped = runControl.getSocketStatusTable().stopSockets(running, false, rate);
			}else{
				int toStopNum = Integer.parseInt(toStop);
				if(toStopNum > runControl.getSocketStatusTable().getRunningSocketCount()){
					toStopNum = runControl.getSocketStatusTable().getRunningSocketCount();
					out.println("not enough running sockets, only stopping "+toStopNum+" sockets");
				}else{
					out.println("stopping "+toStopNum+" sockets");
				}
				stopped = runControl.getSocketStatusTable().stopSockets(toStopNum, uniform, rate);
			}
		}
		
		out.println(""+stopped+" sockets has been stopped");
		
	}
	

}
