/***********************************************************************/
/* Copyright (c) 1999-2005, The Rector and Board of Visitors of the    */
/* University of Virginia.                                             */
/* All rights reserved.                                                */
/*                                                                     */
/* This program is being made available under the terms of version 2.1 */
/* of theGNU Lesser General Public License ("LGPL") as published as of */
/* February, 1999 by the Free Software  Foundation; and subject to the */
/* restrictions of the LGPL, you are authorized to  modify and/or      */
/* redistribute it.                                                    */
/*                                                                     */
/* As stated in the LGPL, THIS PROGRAM IS BEING MADE AVAILABLE "AS IS",*/
/* AND WITHOUT A WARRANTY OF ANY KIND, EITHER EXPRESSED OR IMPLIED,    */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            */
/* MERCHANTABILITY AND  FITNESS FOR A PARTICULAR PURPOSE. FURTHER, YOU */
/* AGREE NOT TO HOLD THE COPYRIGHT  OWNER(S) LIABLE FOR DAMAGES,       */
/* INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES,*/
/* ARISING OUT OF THE USE OR INABILITY TO USE  THE PROGRAM.            */
/* See the GNU General Public License for more details.                */
/*                                                                     */
/* You should have received a copy of version 2.1 of the GNU General   */
/* Public License along with this program; if not, you may obtain a    */
/* copy  from the Free Software Foundation website at the following URL*/
/* address (link  active as of September 1, 2005):                     */
/* http://www.fsf.org/licensing/licenses/lgpl.html; or that failing,   */
/* you can request a copy from Free Software Foundation,  Inc., 51     */
/* Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.                */
/*                                                                     */
/* This program is part of the HyperCast software distribution. To     */
/* learn more about HyperCast, please visit the website at             */
/* http://www.hypercast.org.                                           */
/***********************************************************************/


// TODO Add comments 

package runserver;

import java.util.Vector;

import hypercast.HyperCastConfig;
import hypercast.HyperCastStatsException;
import hypercast.I_Stats;
import hypercast.SimpleStats;
import hypercast.StatsProcessor;
import hypercast.MonitorAndControl.I_Portal;
import hypercast.util.XmlUtil;

import org.apache.xpath.XPath;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

/**
 * 
 * This is a simple class that enables the application to have a statistics
 * structure similar to a RunServer, such that the overlay socket used by the
 * application can be monitored by a RunControl. What the application need
 * to do is just creating a instance of this class, passing the portal configuration
 * file name and the Overlay Socket object into the constructor. The application
 * itself doesn't have to implements a I_Stats interface. 
 * 
 * @author HyperCast Team
 * @author Guangyu Dong, Feb 22th, 2005
 *
 * 
 */
public class SimpleRunServer implements I_Stats {
	/**
     * A StatsProcessor instance which handles the statistics in this object.
     */
    private StatsProcessor statsProcessor;
    /**
	 * The statistics name for this object (as an I_Stats instance)
	 */
	private String statisticsName = null;
	
	/**
	 * Portal
	 */
	private I_Portal portal;
	
	private int numOfSockets=0;
	
	private Vector sockets;
	
	/**
	 * Initiate with one socket
	 * @param portalFile
	 * @param sock
	 */
	public SimpleRunServer(String portalFile){
		this(portalFile, null);
	}
	public SimpleRunServer(String portalFile, I_Stats sock){
		
		sockets = new Vector();
		numOfSockets = 0;
		initStatisticsStructure();
		
		addSocket(sock);
		
		final HyperCastConfig portalConfig  = HyperCastConfig.createConfig (portalFile);
        this.portal = portalConfig.createPortal (this, "Appl");
        this.portal.openPortal();
	}
	public void addSocket(I_Stats newSock){
		if(newSock !=null){
			for(int i=0;i<sockets.size();i++){
				if(sockets.elementAt(i)==newSock){
					return;
				}
			}
			numOfSockets ++;
			statsProcessor.addStatsElement("Socket", newSock, 0, -1);
		}
	}
	public void removeSocket(I_Stats sock){
		if(sock==null)return;
		for(int i=0;i<sockets.size();i++){
			if(sockets.elementAt(i)==sock){
				sockets.removeElementAt(i);
				statsProcessor.removeStatsElement(sock);
				numOfSockets --;
				break;
			}
		}
	}
	
	private void initStatisticsStructure()
    {
		statsProcessor = new StatsProcessor(this/* I_Stats object */, true/* readable flag */, true/* writable flag */);
		
        statsProcessor.addStatsElement ("NumOfSockets"/* name */, new SocketNumQuery()/* I_Stats object */, 1/* minoccurs */, 1/* maxoccurs */);

        statsProcessor.addStatsElement ("CurrentTime", new CurrentTime(), 1, 1);

        statsProcessor.addStatsElement ("KillApp", new KillApp(), 1, 1);
    }
    
	/**
	 * This class manages simple statistic "KillApp".
	 */
	class KillApp extends SimpleStats
	{
		/**
		 * Gets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#getStats()
		 */
		protected String getStats()
		{
			return null;
		}
	
		/**
		 * Sets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#setStats(String newvalue)
		 */
		protected String setStats(String newValue) throws HyperCastStatsException
		{
			if (newValue.equals ("true"))
			{
				// NB: no response sent to sender
				try{
					System.out.println("Closing Portal...");
					portal.closePortal();
				}finally{
					System.exit (0);
				}
			}
			
			return newValue;
		}

		/** 
		 * @see I_Stats#getQriteSchame
		 */
		public Element[] getWriteSchema(Document doc, XPath xpath) throws HyperCastStatsException
		{
			return XmlUtil.createSchemaElement(doc/* document instance */, statisticsName/* element name */, "xsd:Boolean"/* type */, 
												null/* restriction base */, null/* pattern value */);
		}
	}

	/**
	 * This class manages simple statistic "NumOfSockets".
	 */
	class SocketNumQuery extends SimpleStats
	{
		/**
		 * Gets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#getStats()
		 */
		protected String getStats()
		{
			return Integer.toString (numOfSockets);
		}
	
		/**
		 * Sets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#setStats(String newvalue)
		 */
		protected String setStats(String newValue) throws HyperCastStatsException
		{
			throw new HyperCastStatsException(statisticsName, HyperCastStatsException.WriteToAReadOnlyValue);
		}

		/** 
		 * @see I_Stats#getReadSchame
		 */
		public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException
		{
			return XmlUtil.createSchemaElement(doc/* document instance */, statisticsName/* element name */, "xsd:Integer"/* type */, 
												null/* restriction base */, null/* pattern value */);
		}
		
	}


	/**
	 * This class manages simple statistic "CurrentTime".
	 */
	class CurrentTime extends SimpleStats
	{
		/**
		 * Gets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#getStats()
		 */
		protected String getStats()
		{
			return Long.toString (System.currentTimeMillis());
		}
	
		/**
		 * Sets the value of a simple statistic managed by the implementation of this class.
		 * 
		 * @see SimpleStats#setStats(String newvalue)
		 */
		protected String setStats(String newValue) throws HyperCastStatsException
		{
			throw new HyperCastStatsException(statisticsName, HyperCastStatsException.WriteToAReadOnlyValue);
		}

		/** 
		 * @see I_Stats#getReadSchame
		 */
		public Element[] getReadSchema(Document doc, XPath xpath) throws HyperCastStatsException
		{
			return XmlUtil.createSchemaElement(doc/* document instance */, statisticsName/* element name */, "xsd:Long"/* type */, 
												null/* restriction base */, null/* pattern value */);
		}
	}
	
	
	/** @see I_Stats#getStats */
    public Element[] getStats (final Document doc, final XPath xpath)
        throws HyperCastStatsException
    {
        return statsProcessor.getStatsResult (doc, xpath);
    }

    /** @see I_Stats#setStats */
    public Element[] setStats (final Document doc, final XPath xpath, final Element newValue)
        throws HyperCastStatsException
    {
        return statsProcessor.setStatsResult (doc, xpath, newValue);
    }

    /** @see I_Stats#getReadSchema */
    public Element[] getReadSchema (final Document doc, final XPath xpath)
        throws HyperCastStatsException
    {
        return statsProcessor.getReadSchemaResult (doc, xpath);
    }

    /** @see I_Stats#getWriteSchema */
    public Element[] getWriteSchema (final Document doc, final XPath xpath)
        throws HyperCastStatsException
    {
        return statsProcessor.getWriteSchemaResult (doc, xpath);
    }


	/** @see I_Statis#getStatsName */
	public String getStatsName ()
	{
		return statisticsName;
	}

	/** @see I_Statis#setStatsName */
	public void setStatsName (String name)
	{
		statisticsName = name;
	}

	
}
